
#include <string.h>

#include "nrf_gpio.h"
#include "nrf.h"
#include "radio.h"

#define MAX_PACKET_LENGTH 128
#define CRC_LED 15


static uint8_t packet[MAX_PACKET_LENGTH+3];

static volatile int8_t rssi = 0;
static volatile bool packet_received = false;
static volatile bool crc_ok = false;
static bool m_is_transmitter = false;
static int8_t m_tx_power = 9;

int8_t get_rssi()
{
    return rssi;
}

bool get_crc()
{
    return crc_ok;
}

bool is_packet_received()
{
    bool ret_val = packet_received;
    if(packet_received)
    {
        packet_received = false;
    }
    return ret_val;
}

void clocks_start( void )
{
    NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_HFCLKSTART = 1;

    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0);
}

void radio_set_mode(radio_modes_t mode)
{
    NRF_RADIO->POWER = 0;
    radio_setup(m_is_transmitter, m_tx_power, mode);
}

void radio_set_tx_power(int8_t power)
{
    NRF_RADIO->TXPOWER = power;
    /*
    switch(power)
    {
        case POWER_ZERO_DB:
            NRF_RADIO->TXPOWER = (RADIO_TXPOWER_TXPOWER_0dBm << RADIO_TXPOWER_TXPOWER_Pos);
            break;
        case POWER_FOUR_DB:
            NRF_RADIO->TXPOWER = (RADIO_TXPOWER_TXPOWER_Pos4dBm << RADIO_TXPOWER_TXPOWER_Pos);
            break;
        case POWER_NINE_DB:
            NRF_RADIO->TXPOWER = (9 << RADIO_TXPOWER_TXPOWER_Pos);
            break;
        default:
            break;
    }
    */
}

void radio_setup(bool is_transmitter, int8_t tx_power, radio_modes_t mode)
{
    m_is_transmitter = is_transmitter;
    m_tx_power = tx_power;
    
    clocks_start();
    
    if(!is_transmitter)
    {
        nrf_gpio_cfg_output(CRC_LED);
        nrf_gpio_pin_set(CRC_LED);
    }
    
    NRF_RADIO->POWER        = 1;
    
    uint32_t preamble_mask = 0;
    if(mode == MODE_125_KBIT)
    {
        preamble_mask  = ((RADIO_PCNF0_PLEN_LongRange << RADIO_PCNF0_PLEN_Pos) & (3 << RADIO_PCNF0_PLEN_Pos));
        preamble_mask |= ((2                          << RADIO_PCNF0_CILEN_Pos) & RADIO_PCNF0_CILEN_Msk);
        preamble_mask |= ((3                          << RADIO_PCNF0_TERMLEN_Pos) & RADIO_PCNF0_TERMLEN_Msk); 
    }
    
    NRF_RADIO->PCNF0        =   (((1UL) << RADIO_PCNF0_S0LEN_Pos                               ) & RADIO_PCNF0_S0LEN_Msk)
                              | (((2UL) << RADIO_PCNF0_S1LEN_Pos                               ) & RADIO_PCNF0_S1LEN_Msk)
                              | (((8UL) << RADIO_PCNF0_LFLEN_Pos                               ) & RADIO_PCNF0_LFLEN_Msk)
                              | preamble_mask;
    NRF_RADIO->PCNF1        =   (((RADIO_PCNF1_ENDIAN_Little)     << RADIO_PCNF1_ENDIAN_Pos    ) & RADIO_PCNF1_ENDIAN_Msk)
                              | (((3UL)                           << RADIO_PCNF1_BALEN_Pos     ) & RADIO_PCNF1_BALEN_Msk)
                              | (((0UL)                           << RADIO_PCNF1_STATLEN_Pos   ) & RADIO_PCNF1_STATLEN_Msk)
                              | ((((uint32_t) MAX_PACKET_LENGTH)                 << RADIO_PCNF1_MAXLEN_Pos    ) & RADIO_PCNF1_MAXLEN_Msk)
                              | ((RADIO_PCNF1_WHITEEN_Enabled     << RADIO_PCNF1_WHITEEN_Pos   ) & RADIO_PCNF1_WHITEEN_Msk);
    NRF_RADIO->CRCCNF       =   (((RADIO_CRCCNF_SKIPADDR_Skip)    << RADIO_CRCCNF_SKIPADDR_Pos ) & RADIO_CRCCNF_SKIPADDR_Msk)
                              | (((RADIO_CRCCNF_LEN_Three)        << RADIO_CRCCNF_LEN_Pos      ) & RADIO_CRCCNF_LEN_Msk);
    NRF_RADIO->CRCPOLY      = 0x0000065b;
    NRF_RADIO->RXADDRESSES  = ((RADIO_RXADDRESSES_ADDR0_Enabled)  << RADIO_RXADDRESSES_ADDR0_Pos);
    NRF_RADIO->SHORTS       = ((1 << RADIO_SHORTS_READY_START_Pos) | (1 << RADIO_SHORTS_END_DISABLE_Pos)
                            | RADIO_SHORTS_ADDRESS_RSSISTART_Msk | RADIO_SHORTS_DISABLED_RSSISTOP_Msk);
    
    switch(mode)
    {
        case MODE_125_KBIT:
            NRF_RADIO->MODE         = ((5)        << RADIO_MODE_MODE_Pos       ) & RADIO_MODE_MODE_Msk;
            break;
        case MODE_1_MBIT:
            NRF_RADIO->MODE         = (RADIO_MODE_MODE_Ble_1Mbit        << RADIO_MODE_MODE_Pos       ) & RADIO_MODE_MODE_Msk;
            break;
        default:
            break;
    }
    
    NRF_RADIO->TIFS         = 150;
    
    if(!is_transmitter)
    {
        NRF_RADIO->INTENSET     = (1 << RADIO_INTENSET_DISABLED_Pos) | (1 << RADIO_INTENSET_ADDRESS_Pos);// | (1 << RADIO_INTENSET_CRCERROR_Pos);
    }
    
    NRF_RADIO->PREFIX0      = 0x0000008e; //access_addr[3]
    NRF_RADIO->BASE0        = 0x89bed600; //access_addr[0:3]
    NRF_RADIO->CRCINIT      = 0x00555555;
    NRF_RADIO->PACKETPTR    = (uint32_t) packet;
    
    if(is_transmitter)
    {
        radio_set_tx_power(tx_power);
    }
    
    NRF_RADIO->FREQUENCY    = 82;
    NRF_RADIO->DATAWHITEIV  = 37;
    
    memset(packet, 0, sizeof(packet));
    
    //NVIC_SetPriority(RADIO_IRQn, 7);
    NVIC_EnableIRQ(RADIO_IRQn);
    NRF_RADIO->TASKS_DISABLE = 1;
}

void radio_send_packet(uint8_t *p_packet, uint8_t length)
{
    memcpy(&packet[3], p_packet, length);
    packet[1] = length;
    NRF_RADIO->TASKS_TXEN = 1;
}

void radio_receive_packet()
{
    NRF_RADIO->TASKS_RXEN = 1;
}

uint8_t radio_get_packet(uint8_t *p_packet)
{
    memcpy(p_packet, &packet[3], packet[1]);
    return packet[1];
}

void RADIO_IRQHandler(void)
{
    if (NRF_RADIO->EVENTS_ADDRESS == 1)
    {
        NRF_RADIO->EVENTS_ADDRESS = 0;
        
    }
    if (NRF_RADIO->EVENTS_DISABLED && (NRF_RADIO->INTENSET & RADIO_INTENSET_DISABLED_Msk))
    {
        NRF_RADIO->EVENTS_DISABLED = 0;
        
        //packet_received = true;
        
        if(NRF_RADIO->CRCSTATUS & (RADIO_CRCSTATUS_CRCSTATUS_CRCOk << RADIO_CRCSTATUS_CRCSTATUS_Pos))
        {
            rssi = -NRF_RADIO->RSSISAMPLE;
            packet_received = true;
        }
        else
        {
            packet_received = false;
            nrf_gpio_pin_toggle(CRC_LED);
        }
        
        radio_receive_packet();
    }
    if(NRF_RADIO->EVENTS_CRCERROR && (NRF_RADIO->INTENSET & RADIO_INTENSET_CRCERROR_Msk))
    {
        NRF_RADIO->EVENTS_CRCERROR = 0;
    }
}
