/* Copyright (c) 2015 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#ifndef FONTS_H__
#define FONTS_H__

#include <stdint.h>

typedef struct
{
    int8_t           width;             ///< Character width (-1 means dynamic width).
    int8_t           height;            ///< Character height (-1 means dynamic height).
    int8_t           offset;            ///< Bitmap offset (-1 means dynamic offset).
    uint8_t          descr_entry_size;  ///< Deescriptor entry size.
    uint8_t          start_char;        ///< The first character of the font.
    uint8_t          end_char;          ///< The last character of the font.
    uint8_t   const *p_descriptor;      ///< The font decriptors.
    uint8_t   const *p_bitmap;          ///< The font bitmaps.
}font_info_t;


#ifdef FONTS_DEFAULT_FONT /* If a declared project specific font is to be used as the default font. */
#define FONTS_DEFAULT_FONT_DECLARATION FONTS_DEFAULT_FONT
#else                     /* Otherwise the default font will be declared in the framebuffer module. */
#define FONTS_DEFAULT_FONT_DECLARATION fb_default_font
#endif


/** @brief Reference to the default font. */
extern const font_info_t FONTS_DEFAULT_FONT_DECLARATION;


/** @brief Declaration macro for the corierNew_8pt font. */
#define FONTS_CONST_CORIERNEW_8PT_DECLARE(font_name) \
const uint8_t fonts_coriernew_8pt_bitmaps[] = FONTS_CORIERNEW_8PT_BITMAPS;\
const uint8_t fonts_coriernew_8pt_descriptors[] = FONTS_CORIERNEW_8PT_DESCRIPTORS;\
const font_info_t font_name = FONTS_CORIERNEW_8PT_INFO


/** @brief Declaration macro for the corierNew_b16pt font. */
#define FONTS_CONST_CORIERNEW_B16PT_DECLARE(font_name) \
const uint8_t fonts_coriernew_b16pt_bitmaps[] = FONTS_CORIERNEW_B16PT_BITMAPS;\
const uint8_t fonts_coriernew_b16pt_descriptors[] = FONTS_CORIERNEW_B16PT_DESCRIPTORS;\
const font_info_t font_name = FONTS_CORIERNEW_B16PT_INFO


/** @brief Declaration macro for the corierNew_b7pt font. */
#define FONTS_CONST_CORIERNEW_B7PT_DECLARE(font_name) \
const uint8_t fonts_coriernew_b7pt_bitmaps[] = FONTS_CORIERNEW_B7PT_BITMAPS;\
const uint8_t fonts_coriernew_b7pt_descriptors[] = FONTS_CORIERNEW_B7PT_DESCRIPTORS;\
const font_info_t font_name = FONTS_CORIERNEW_B7PT_INFO


/** @brief Declaration macro for the generic_8pt font. */
#define FONTS_CONST_GENERIC_8PT_DECLARE(font_name) \
const uint8_t fonts_generic_8pt_bitmaps[] = FONTS_GENERIC_8PT_BITMAPS;\
const uint8_t fonts_generic_8pt_descriptors[] = FONTS_GENERIC_8PT_DESCRIPTORS;\
const font_info_t font_name = FONTS_GENERIC_8PT_INFO


/** @brief Declaration macro for the digital7_limited_b20pt font. */
#define FONTS_CONST_DIGITAL7_LIMITED_B20PT_DECLARE(font_name) \
const uint8_t fonts_digital7_limited_b20pt_bitmaps[] = FONTS_DIGITAL7_LIMITED_B20PT_BITMAPS;\
const uint8_t fonts_digital7_limited_b20pt_descriptors[] = FONTS_DIGITAL7_LIMITED_B20PT_DESCRIPTORS;\
const font_info_t font_name = FONTS_DIGITAL7_LIMITED_B20PT_INFO


/** @brief Declaration macro for the arial_16pt font. */
#define FONTS_CONST_ARIAL_16PT_DECLARE(font_name) \
const uint8_t fonts_arial_16pt_bitmaps[] = FONTS_ARIAL_16PT_BITMAPS;\
const uint8_t fonts_arial_16pt_descriptors[] = FONTS_ARIAL_16PT_DESCRIPTORS;\
const font_info_t font_name = FONTS_ARIAL_16PT_INFO


/** @brief Declaration macro for the corierNew_b8pt font. */
#define FONTS_CONST_CORIERNEW_B8PT_DECLARE(font_name) \
const uint8_t fonts_coriernew_b8pt_bitmaps[] = FONTS_CORIERNEW_B8PT_BITMAPS;\
const uint8_t fonts_coriernew_b8pt_descriptors[] = FONTS_CORIERNEW_B8PT_DESCRIPTORS;\
const font_info_t font_name = FONTS_CORIERNEW_B8PT_INFO


/** @brief Declaration macro for the irisUPC_b8pt font. */
#define FONTS_CONST_IRISUPC_B8PT_DECLARE(font_name) \
const uint8_t fonts_irisupc_b8pt_bitmaps[] = FONTS_IRISUPC_B8PT_BITMAPS;\
const uint8_t fonts_irisupc_b8pt_descriptors[] = FONTS_IRISUPC_B8PT_DESCRIPTORS;\
const font_info_t font_name = FONTS_IRISUPC_B8PT_INFO


/** @brief Declaration macro for the arial_20pt font. */
#define FONTS_CONST_ARIAL_20PT_DECLARE(font_name) \
const uint8_t fonts_arial_20pt_bitmaps[] = FONTS_ARIAL_20PT_BITMAPS;\
const uint8_t fonts_arial_20pt_descriptors[] = FONTS_ARIAL_20PT_DESCRIPTORS;\
const font_info_t font_name = FONTS_ARIAL_20PT_INFO


/** @brief Initialization macro for the descriptors of the corierNew_8pt font. */
#define FONTS_CORIERNEW_8PT_DESCRIPTORS \
{\
         8,  /*   */\
         1,  /* ! */\
         3,  /* " */\
         5,  /* # */\
         4,  /* $ */\
         4,  /* % */\
         5,  /* & */\
         1,  /* ' */\
         2,  /* ( */\
         2,  /* ) */\
         5,  /* * */\
         5,  /* + */\
         3,  /* , */\
         5,  /* - */\
         2,  /* . */\
         5,  /* / */\
         5,  /* 0 */\
         5,  /* 1 */\
         5,  /* 2 */\
         5,  /* 3 */\
         5,  /* 4 */\
         5,  /* 5 */\
         5,  /* 6 */\
         5,  /* 7 */\
         5,  /* 8 */\
         5,  /* 9 */\
         2,  /* : */\
         3,  /* ; */\
         5,  /* < */\
         5,  /* = */\
         5,  /* > */\
         4,  /* ? */\
         5,  /* @ */\
         7,  /* A */\
         6,  /* B */\
         5,  /* C */\
         6,  /* D */\
         5,  /* E */\
         5,  /* F */\
         6,  /* G */\
         7,  /* H */\
         5,  /* I */\
         5,  /* J */\
         7,  /* K */\
         5,  /* L */\
         7,  /* M */\
         7,  /* N */\
         5,  /* O */\
         5,  /* P */\
         5,  /* Q */\
         6,  /* R */\
         5,  /* S */\
         5,  /* T */\
         7,  /* U */\
         7,  /* V */\
         7,  /* W */\
         7,  /* X */\
         7,  /* Y */\
         5,  /* Z */\
         2,  /* [ */\
         4,  /* \ */\
         2,  /* ] */\
         5,  /* ^ */\
         7,  /* _ */\
         2,  /* ` */\
         6,  /* a */\
         6,  /* b */\
         5,  /* c */\
         6,  /* d */\
         5,  /* e */\
         4,  /* f */\
         6,  /* g */\
         7,  /* h */\
         5,  /* i */\
         4,  /* j */\
         6,  /* k */\
         5,  /* l */\
         7,  /* m */\
         7,  /* n */\
         5,  /* o */\
         6,  /* p */\
         6,  /* q */\
         5,  /* r */\
         5,  /* s */\
         6,  /* t */\
         7,  /* u */\
         7,  /* v */\
         7,  /* w */\
         5,  /* x */\
         7,  /* y */\
         5,  /* z */\
         3,  /* { */\
         1,  /* | */\
         3,  /* } */\
         5,  /* ~ */\
}


/** @brief Initialization macro for the bitmaps of the corierNew_8pt font. */
#define FONTS_CORIERNEW_8PT_BITMAPS \
{\
\
    /* @0 ' ' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @10 '!' (1 pixels wide) */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @20 '"' (3 pixels wide) */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @30 '#' (5 pixels wide) */\
    0x14, /*   # # */\
    0x0A, /*  # #  */\
    0x1F, /* ##### */\
    0x0A, /*  # #  */\
    0x0A, /*  # #  */\
    0x1F, /* ##### */\
    0x0A, /*  # #  */\
    0x05, /* # #   */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @40 '$' (4 pixels wide) */\
    0x04, /*   #  */\
    0x0E, /*  ### */\
    0x01, /* #    */\
    0x06, /*  ##  */\
    0x08, /*    # */\
    0x07, /* ###  */\
    0x02, /*  #   */\
    0x02, /*  #   */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @50 '%' (4 pixels wide) */\
    0x02, /*  #   */\
    0x05, /* # #  */\
    0x02, /*  #   */\
    0x0F, /* #### */\
    0x04, /*   #  */\
    0x0A, /*  # # */\
    0x04, /*   #  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @60 '&' (5 pixels wide) */\
    0x00, /*       */\
    0x0C, /*   ##  */\
    0x02, /*  #    */\
    0x02, /*  #    */\
    0x0D, /* # ##  */\
    0x09, /* #  #  */\
    0x1E, /*  #### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @70 ''' (1 pixels wide) */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @80 '(' (2 pixels wide) */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @90 ')' (2 pixels wide) */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @100 '*' (5 pixels wide) */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x04, /*   #   */\
    0x0A, /*  # #  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @110 '+' (5 pixels wide) */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @120 ',' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x06, /*  ## */\
    0x03, /* ##  */\
    0x01, /* #   */\
    0x00, /*     */\
\
    /* @130 '-' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @140 '.' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @150 '/' (5 pixels wide) */\
    0x10, /*     # */\
    0x08, /*    #  */\
    0x08, /*    #  */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x02, /*  #    */\
    0x02, /*  #    */\
    0x01, /* #     */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @160 '0' (5 pixels wide) */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @170 '1' (5 pixels wide) */\
    0x06, /*  ##   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @180 '2' (5 pixels wide) */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x08, /*    #  */\
    0x04, /*   #   */\
    0x02, /*  #    */\
    0x01, /* #     */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @190 '3' (5 pixels wide) */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x10, /*     # */\
    0x0C, /*   ##  */\
    0x10, /*     # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @200 '4' (5 pixels wide) */\
    0x08, /*    #  */\
    0x0C, /*   ##  */\
    0x0A, /*  # #  */\
    0x09, /* #  #  */\
    0x1F, /* ##### */\
    0x08, /*    #  */\
    0x1C, /*   ### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @210 '5' (5 pixels wide) */\
    0x1E, /*  #### */\
    0x02, /*  #    */\
    0x02, /*  #    */\
    0x0E, /*  ###  */\
    0x10, /*     # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @220 '6' (5 pixels wide) */\
    0x1C, /*   ### */\
    0x02, /*  #    */\
    0x01, /* #     */\
    0x0F, /* ####  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @230 '7' (5 pixels wide) */\
    0x1F, /* ##### */\
    0x11, /* #   # */\
    0x10, /*     # */\
    0x08, /*    #  */\
    0x08, /*    #  */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @240 '8' (5 pixels wide) */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @250 '9' (5 pixels wide) */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x1E, /*  #### */\
    0x10, /*     # */\
    0x08, /*    #  */\
    0x07, /* ###   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @260 ':' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @270 ';' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x06, /*  ## */\
    0x00, /*     */\
    0x00, /*     */\
    0x06, /*  ## */\
    0x03, /* ##  */\
    0x01, /* #   */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @280 '<' (5 pixels wide) */\
    0x00, /*       */\
    0x10, /*     # */\
    0x0C, /*   ##  */\
    0x03, /* ##    */\
    0x0C, /*   ##  */\
    0x10, /*     # */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @290 '=' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @300 '>' (5 pixels wide) */\
    0x00, /*       */\
    0x01, /* #     */\
    0x06, /*  ##   */\
    0x18, /*    ## */\
    0x06, /*  ##   */\
    0x01, /* #     */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @310 '?' (4 pixels wide) */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x08, /*    # */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @320 '@' (5 pixels wide) */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x19, /* #  ## */\
    0x15, /* # # # */\
    0x15, /* # # # */\
    0x19, /* #  ## */\
    0x01, /* #     */\
    0x1E, /*  #### */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @330 'A' (7 pixels wide) */\
    0x0C, /*   ##    */\
    0x08, /*    #    */\
    0x14, /*   # #   */\
    0x14, /*   # #   */\
    0x1C, /*   ###   */\
    0x22, /*  #   #  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @340 'B' (6 pixels wide) */\
    0x1F, /* #####  */\
    0x22, /*  #   # */\
    0x22, /*  #   # */\
    0x1E, /*  ####  */\
    0x22, /*  #   # */\
    0x22, /*  #   # */\
    0x1F, /* #####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @350 'C' (5 pixels wide) */\
    0x1E, /*  #### */\
    0x11, /* #   # */\
    0x01, /* #     */\
    0x01, /* #     */\
    0x01, /* #     */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @360 'D' (6 pixels wide) */\
    0x0F, /* ####   */\
    0x12, /*  #  #  */\
    0x22, /*  #   # */\
    0x22, /*  #   # */\
    0x22, /*  #   # */\
    0x12, /*  #  #  */\
    0x0F, /* ####   */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @370 'E' (5 pixels wide) */\
    0x1F, /* ##### */\
    0x12, /*  #  # */\
    0x0A, /*  # #  */\
    0x0E, /*  ###  */\
    0x0A, /*  # #  */\
    0x12, /*  #  # */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @380 'F' (5 pixels wide) */\
    0x1F, /* ##### */\
    0x12, /*  #  # */\
    0x0A, /*  # #  */\
    0x0E, /*  ###  */\
    0x0A, /*  # #  */\
    0x02, /*  #    */\
    0x07, /* ###   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @390 'G' (6 pixels wide) */\
    0x1E, /*  ####  */\
    0x11, /* #   #  */\
    0x01, /* #      */\
    0x01, /* #      */\
    0x39, /* #  ### */\
    0x11, /* #   #  */\
    0x0E, /*  ###   */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @400 'H' (7 pixels wide) */\
    0x77, /* ### ### */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x3E, /*  #####  */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @410 'I' (5 pixels wide) */\
    0x1F, /* ##### */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @420 'J' (5 pixels wide) */\
    0x1E, /*  #### */\
    0x08, /*    #  */\
    0x08, /*    #  */\
    0x08, /*    #  */\
    0x09, /* #  #  */\
    0x09, /* #  #  */\
    0x06, /*  ##   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @430 'K' (7 pixels wide) */\
    0x77, /* ### ### */\
    0x22, /*  #   #  */\
    0x12, /*  #  #   */\
    0x0E, /*  ###    */\
    0x12, /*  #  #   */\
    0x22, /*  #   #  */\
    0x67, /* ###  ## */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @440 'L' (5 pixels wide) */\
    0x07, /* ###   */\
    0x02, /*  #    */\
    0x02, /*  #    */\
    0x02, /*  #    */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @450 'M' (7 pixels wide) */\
    0x77, /* ### ### */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x2A, /*  # # #  */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @460 'N' (7 pixels wide) */\
    0x77, /* ### ### */\
    0x26, /*  ##  #  */\
    0x26, /*  ##  #  */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x32, /*  #  ##  */\
    0x37, /* ### ##  */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @470 'O' (5 pixels wide) */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @480 'P' (5 pixels wide) */\
    0x0F, /* ####  */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x0E, /*  ###  */\
    0x02, /*  #    */\
    0x07, /* ###   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @490 'Q' (5 pixels wide) */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x1C, /*   ### */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @500 'R' (6 pixels wide) */\
    0x0F, /* ####   */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x0E, /*  ###   */\
    0x12, /*  #  #  */\
    0x27, /* ###  # */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @510 'S' (5 pixels wide) */\
    0x16, /*  ## # */\
    0x19, /* #  ## */\
    0x01, /* #     */\
    0x0E, /*  ###  */\
    0x10, /*     # */\
    0x11, /* #   # */\
    0x0F, /* ####  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @520 'T' (5 pixels wide) */\
    0x1F, /* ##### */\
    0x15, /* # # # */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @530 'U' (7 pixels wide) */\
    0x77, /* ### ### */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x1C, /*   ###   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @540 'V' (7 pixels wide) */\
    0x77, /* ### ### */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x12, /*  #  #   */\
    0x14, /*   # #   */\
    0x14, /*   # #   */\
    0x0C, /*   ##    */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @550 'W' (7 pixels wide) */\
    0x77, /* ### ### */\
    0x22, /*  #   #  */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x14, /*   # #   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @560 'X' (7 pixels wide) */\
    0x63, /* ##   ## */\
    0x22, /*  #   #  */\
    0x14, /*   # #   */\
    0x08, /*    #    */\
    0x14, /*   # #   */\
    0x22, /*  #   #  */\
    0x63, /* ##   ## */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @570 'Y' (7 pixels wide) */\
    0x77, /* ### ### */\
    0x22, /*  #   #  */\
    0x14, /*   # #   */\
    0x08, /*    #    */\
    0x08, /*    #    */\
    0x08, /*    #    */\
    0x1C, /*   ###   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @580 'Z' (5 pixels wide) */\
    0x1F, /* ##### */\
    0x11, /* #   # */\
    0x08, /*    #  */\
    0x04, /*   #   */\
    0x02, /*  #    */\
    0x11, /* #   # */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @590 '[' (2 pixels wide) */\
    0x03, /* ## */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @600 '\' (4 pixels wide) */\
    0x01, /* #    */\
    0x02, /*  #   */\
    0x02, /*  #   */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x08, /*    # */\
    0x08, /*    # */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @610 ']' (2 pixels wide) */\
    0x03, /* ## */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @620 '^' (5 pixels wide) */\
    0x04, /*   #   */\
    0x0A, /*  # #  */\
    0x11, /* #   # */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @630 '_' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x7F, /* ####### */\
\
    /* @640 '`' (2 pixels wide) */\
    0x01, /* #  */\
    0x02, /*  # */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @650 'a' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x0E, /*  ###   */\
    0x11, /* #   #  */\
    0x1E, /*  ####  */\
    0x11, /* #   #  */\
    0x3F, /* ###### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @660 'b' (6 pixels wide) */\
    0x03, /* ##     */\
    0x02, /*  #     */\
    0x1A, /*  # ##  */\
    0x26, /*  ##  # */\
    0x22, /*  #   # */\
    0x22, /*  #   # */\
    0x1F, /* #####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @670 'c' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x1E, /*  #### */\
    0x11, /* #   # */\
    0x01, /* #     */\
    0x01, /* #     */\
    0x1E, /*  #### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @680 'd' (6 pixels wide) */\
    0x18, /*    ##  */\
    0x10, /*     #  */\
    0x16, /*  ## #  */\
    0x19, /* #  ##  */\
    0x11, /* #   #  */\
    0x11, /* #   #  */\
    0x3E, /*  ##### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @690 'e' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x1F, /* ##### */\
    0x01, /* #     */\
    0x1E, /*  #### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @700 'f' (4 pixels wide) */\
    0x0C, /*   ## */\
    0x02, /*  #   */\
    0x0F, /* #### */\
    0x02, /*  #   */\
    0x02, /*  #   */\
    0x02, /*  #   */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @710 'g' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x36, /*  ## ## */\
    0x19, /* #  ##  */\
    0x11, /* #   #  */\
    0x11, /* #   #  */\
    0x1E, /*  ####  */\
    0x10, /*     #  */\
    0x0E, /*  ###   */\
    0x00, /*        */\
\
    /* @720 'h' (7 pixels wide) */\
    0x03, /* ##      */\
    0x02, /*  #      */\
    0x1A, /*  # ##   */\
    0x26, /*  ##  #  */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @730 'i' (5 pixels wide) */\
    0x04, /*   #   */\
    0x00, /*       */\
    0x07, /* ###   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @740 'j' (4 pixels wide) */\
    0x04, /*   #  */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x08, /*    # */\
    0x08, /*    # */\
    0x08, /*    # */\
    0x08, /*    # */\
    0x08, /*    # */\
    0x07, /* ###  */\
    0x00, /*      */\
\
    /* @750 'k' (6 pixels wide) */\
    0x03, /* ##     */\
    0x02, /*  #     */\
    0x3A, /*  # ### */\
    0x12, /*  #  #  */\
    0x0E, /*  ###   */\
    0x12, /*  #  #  */\
    0x3B, /* ## ### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @760 'l' (5 pixels wide) */\
    0x06, /*  ##   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @770 'm' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x17, /* ### #   */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x7F, /* ####### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @780 'n' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x1B, /* ## ##   */\
    0x26, /*  ##  #  */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @790 'o' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @800 'p' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x1B, /* ## ##  */\
    0x26, /*  ##  # */\
    0x22, /*  #   # */\
    0x22, /*  #   # */\
    0x1E, /*  ####  */\
    0x02, /*  #     */\
    0x07, /* ###    */\
    0x00, /*        */\
\
    /* @810 'q' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x36, /*  ## ## */\
    0x19, /* #  ##  */\
    0x11, /* #   #  */\
    0x11, /* #   #  */\
    0x1E, /*  ####  */\
    0x10, /*     #  */\
    0x38, /*    ### */\
    0x00, /*        */\
\
    /* @820 'r' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x1B, /* ## ## */\
    0x06, /*  ##   */\
    0x02, /*  #    */\
    0x02, /*  #    */\
    0x0F, /* ####  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @830 's' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x1E, /*  #### */\
    0x01, /* #     */\
    0x0E, /*  ###  */\
    0x10, /*     # */\
    0x0F, /* ####  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @840 't' (6 pixels wide) */\
    0x00, /*        */\
    0x02, /*  #     */\
    0x1F, /* #####  */\
    0x02, /*  #     */\
    0x02, /*  #     */\
    0x22, /*  #   # */\
    0x1C, /*   ###  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @850 'u' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x33, /* ##  ##  */\
    0x22, /*  #   #  */\
    0x22, /*  #   #  */\
    0x32, /*  #  ##  */\
    0x6C, /*   ## ## */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @860 'v' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x22, /*  #   #  */\
    0x12, /*  #  #   */\
    0x14, /*   # #   */\
    0x0C, /*   ##    */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @870 'w' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x22, /*  #   #  */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x14, /*   # #   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @880 'x' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x1B, /* ## ## */\
    0x0A, /*  # #  */\
    0x04, /*   #   */\
    0x0A, /*  # #  */\
    0x1B, /* ## ## */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @890 'y' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x22, /*  #   #  */\
    0x14, /*   # #   */\
    0x14, /*   # #   */\
    0x08, /*    #    */\
    0x08, /*    #    */\
    0x0E, /*  ###    */\
    0x00, /*         */\
\
    /* @900 'z' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x09, /* #  #  */\
    0x04, /*   #   */\
    0x12, /*  #  # */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @910 '{' (3 pixels wide) */\
    0x04, /*   # */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x03, /* ##  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x04, /*   # */\
    0x00, /*     */\
\
    /* @920 '|' (1 pixels wide) */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @930 '}' (3 pixels wide) */\
    0x01, /* #   */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x06, /*  ## */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x01, /* #   */\
    0x00, /*     */\
\
    /* @940 '~' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x16, /*  ## # */\
    0x0D, /* # ##  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
}


/** @brief Initialization macro for the corierNew_8pt font information. */
#define FONTS_CORIERNEW_8PT_INFO \
{\
    -1, /* Character width (-1 means dynamic width). */\
    10, /* Character height (-1 means dynamic height). */\
    10, /* Bitmap offset (-1 means dynamic offset). */\
    1, /* Deescriptor entry size. */\
    ' ', /* Start character. */\
    '~', /* End character. */\
    fonts_coriernew_8pt_descriptors, /*  Character descriptor array. */\
    fonts_coriernew_8pt_bitmaps, /* Character bitmap array. */\
}


/** @brief Initialization macro for the descriptors of the corierNew_b16pt font. */
#define FONTS_CORIERNEW_B16PT_DESCRIPTORS \
{\
         8, (   0 & 0xFF)       , (   0 >> 8) & 0xFF,  /*   */\
         3, (  21 & 0xFF)       , (  21 >> 8) & 0xFF,  /* ! */\
         8, (  42 & 0xFF)       , (  42 >> 8) & 0xFF,  /* " */\
        10, (  63 & 0xFF)       , (  63 >> 8) & 0xFF,  /* # */\
         8, ( 105 & 0xFF)       , ( 105 >> 8) & 0xFF,  /* $ */\
        10, ( 126 & 0xFF)       , ( 126 >> 8) & 0xFF,  /* % */\
        10, ( 168 & 0xFF)       , ( 168 >> 8) & 0xFF,  /* & */\
         3, ( 210 & 0xFF)       , ( 210 >> 8) & 0xFF,  /* ' */\
         4, ( 231 & 0xFF)       , ( 231 >> 8) & 0xFF,  /* ( */\
         4, ( 252 & 0xFF)       , ( 252 >> 8) & 0xFF,  /* ) */\
         8, ( 273 & 0xFF)       , ( 273 >> 8) & 0xFF,  /* * */\
        10, ( 294 & 0xFF)       , ( 294 >> 8) & 0xFF,  /* + */\
         5, ( 336 & 0xFF)       , ( 336 >> 8) & 0xFF,  /* , */\
         9, ( 357 & 0xFF)       , ( 357 >> 8) & 0xFF,  /* - */\
         3, ( 399 & 0xFF)       , ( 399 >> 8) & 0xFF,  /* . */\
         9, ( 420 & 0xFF)       , ( 420 >> 8) & 0xFF,  /* / */\
        10, ( 462 & 0xFF)       , ( 462 >> 8) & 0xFF,  /* 0 */\
         9, ( 504 & 0xFF)       , ( 504 >> 8) & 0xFF,  /* 1 */\
        10, ( 546 & 0xFF)       , ( 546 >> 8) & 0xFF,  /* 2 */\
        10, ( 588 & 0xFF)       , ( 588 >> 8) & 0xFF,  /* 3 */\
         9, ( 630 & 0xFF)       , ( 630 >> 8) & 0xFF,  /* 4 */\
        10, ( 672 & 0xFF)       , ( 672 >> 8) & 0xFF,  /* 5 */\
         9, ( 714 & 0xFF)       , ( 714 >> 8) & 0xFF,  /* 6 */\
         9, ( 756 & 0xFF)       , ( 756 >> 8) & 0xFF,  /* 7 */\
        10, ( 798 & 0xFF)       , ( 798 >> 8) & 0xFF,  /* 8 */\
         9, ( 840 & 0xFF)       , ( 840 >> 8) & 0xFF,  /* 9 */\
         3, ( 882 & 0xFF)       , ( 882 >> 8) & 0xFF,  /* : */\
         5, ( 903 & 0xFF)       , ( 903 >> 8) & 0xFF,  /* ; */\
        12, ( 924 & 0xFF)       , ( 924 >> 8) & 0xFF,  /* < */\
        12, ( 966 & 0xFF)       , ( 966 >> 8) & 0xFF,  /* = */\
        12, (1008 & 0xFF)       , (1008 >> 8) & 0xFF,  /* > */\
         9, (1050 & 0xFF)       , (1050 >> 8) & 0xFF,  /* ? */\
         7, (1092 & 0xFF)       , (1092 >> 8) & 0xFF,  /* @ */\
        13, (1113 & 0xFF)       , (1113 >> 8) & 0xFF,  /* A */\
        11, (1155 & 0xFF)       , (1155 >> 8) & 0xFF,  /* B */\
        11, (1197 & 0xFF)       , (1197 >> 8) & 0xFF,  /* C */\
        10, (1239 & 0xFF)       , (1239 >> 8) & 0xFF,  /* D */\
        10, (1281 & 0xFF)       , (1281 >> 8) & 0xFF,  /* E */\
        11, (1323 & 0xFF)       , (1323 >> 8) & 0xFF,  /* F */\
        12, (1365 & 0xFF)       , (1365 >> 8) & 0xFF,  /* G */\
        11, (1407 & 0xFF)       , (1407 >> 8) & 0xFF,  /* H */\
         8, (1449 & 0xFF)       , (1449 >> 8) & 0xFF,  /* I */\
        11, (1470 & 0xFF)       , (1470 >> 8) & 0xFF,  /* J */\
        12, (1512 & 0xFF)       , (1512 >> 8) & 0xFF,  /* K */\
        11, (1554 & 0xFF)       , (1554 >> 8) & 0xFF,  /* L */\
        13, (1596 & 0xFF)       , (1596 >> 8) & 0xFF,  /* M */\
        11, (1638 & 0xFF)       , (1638 >> 8) & 0xFF,  /* N */\
        11, (1680 & 0xFF)       , (1680 >> 8) & 0xFF,  /* O */\
        10, (1722 & 0xFF)       , (1722 >> 8) & 0xFF,  /* P */\
        11, (1764 & 0xFF)       , (1764 >> 8) & 0xFF,  /* Q */\
        12, (1806 & 0xFF)       , (1806 >> 8) & 0xFF,  /* R */\
         9, (1848 & 0xFF)       , (1848 >> 8) & 0xFF,  /* S */\
        10, (1890 & 0xFF)       , (1890 >> 8) & 0xFF,  /* T */\
        11, (1932 & 0xFF)       , (1932 >> 8) & 0xFF,  /* U */\
        13, (1974 & 0xFF)       , (1974 >> 8) & 0xFF,  /* V */\
        13, (2016 & 0xFF)       , (2016 >> 8) & 0xFF,  /* W */\
        11, (2058 & 0xFF)       , (2058 >> 8) & 0xFF,  /* X */\
        12, (2100 & 0xFF)       , (2100 >> 8) & 0xFF,  /* Y */\
         9, (2142 & 0xFF)       , (2142 >> 8) & 0xFF,  /* Z */\
         4, (2184 & 0xFF)       , (2184 >> 8) & 0xFF,  /* [ */\
        10, (2205 & 0xFF)       , (2205 >> 8) & 0xFF,  /* \ */\
         4, (2247 & 0xFF)       , (2247 >> 8) & 0xFF,  /* ] */\
         9, (2268 & 0xFF)       , (2268 >> 8) & 0xFF,  /* ^ */\
        15, (2310 & 0xFF)       , (2310 >> 8) & 0xFF,  /* _ */\
         4, (2352 & 0xFF)       , (2352 >> 8) & 0xFF,  /* ` */\
        11, (2373 & 0xFF)       , (2373 >> 8) & 0xFF,  /* a */\
        11, (2415 & 0xFF)       , (2415 >> 8) & 0xFF,  /* b */\
        11, (2457 & 0xFF)       , (2457 >> 8) & 0xFF,  /* c */\
        11, (2499 & 0xFF)       , (2499 >> 8) & 0xFF,  /* d */\
        11, (2541 & 0xFF)       , (2541 >> 8) & 0xFF,  /* e */\
        10, (2583 & 0xFF)       , (2583 >> 8) & 0xFF,  /* f */\
        11, (2625 & 0xFF)       , (2625 >> 8) & 0xFF,  /* g */\
        11, (2667 & 0xFF)       , (2667 >> 8) & 0xFF,  /* h */\
        10, (2709 & 0xFF)       , (2709 >> 8) & 0xFF,  /* i */\
         8, (2751 & 0xFF)       , (2751 >> 8) & 0xFF,  /* j */\
        11, (2772 & 0xFF)       , (2772 >> 8) & 0xFF,  /* k */\
        10, (2814 & 0xFF)       , (2814 >> 8) & 0xFF,  /* l */\
        14, (2856 & 0xFF)       , (2856 >> 8) & 0xFF,  /* m */\
        11, (2898 & 0xFF)       , (2898 >> 8) & 0xFF,  /* n */\
        11, (2940 & 0xFF)       , (2940 >> 8) & 0xFF,  /* o */\
        11, (2982 & 0xFF)       , (2982 >> 8) & 0xFF,  /* p */\
        11, (3024 & 0xFF)       , (3024 >> 8) & 0xFF,  /* q */\
        11, (3066 & 0xFF)       , (3066 >> 8) & 0xFF,  /* r */\
         9, (3108 & 0xFF)       , (3108 >> 8) & 0xFF,  /* s */\
        11, (3150 & 0xFF)       , (3150 >> 8) & 0xFF,  /* t */\
        11, (3192 & 0xFF)       , (3192 >> 8) & 0xFF,  /* u */\
        11, (3234 & 0xFF)       , (3234 >> 8) & 0xFF,  /* v */\
        13, (3276 & 0xFF)       , (3276 >> 8) & 0xFF,  /* w */\
        11, (3318 & 0xFF)       , (3318 >> 8) & 0xFF,  /* x */\
        12, (3360 & 0xFF)       , (3360 >> 8) & 0xFF,  /* y */\
         9, (3402 & 0xFF)       , (3402 >> 8) & 0xFF,  /* z */\
         6, (3444 & 0xFF)       , (3444 >> 8) & 0xFF,  /* { */\
         2, (3465 & 0xFF)       , (3465 >> 8) & 0xFF,  /* | */\
         6, (3486 & 0xFF)       , (3486 >> 8) & 0xFF,  /* } */\
        10, (3507 & 0xFF)       , (3507 >> 8) & 0xFF,  /* ~ */\
}


/** @brief Initialization macro for the bitmaps of the corierNew_b16pt font. */
#define FONTS_CORIERNEW_B16PT_BITMAPS \
{\
\
    /* @0 ' ' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @21 '!' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @42 '"' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xE7, /* ###  ### */\
    0xE7, /* ###  ### */\
    0xE7, /* ###  ### */\
    0x42, /*  #    #  */\
    0x42, /*  #    #  */\
    0x42, /*  #    #  */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @63 '#' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @105 '$' (8 pixels wide) */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0xFC, /*   ###### */\
    0xFE, /*  ####### */\
    0xC3, /* ##    ## */\
    0x03, /* ##       */\
    0x07, /* ###      */\
    0x7E, /*  ######  */\
    0xFC, /*   ###### */\
    0xC0, /*       ## */\
    0xC3, /* ##    ## */\
    0xE3, /* ##   ### */\
    0x7F, /* #######  */\
    0x3F, /* ######   */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @126 '%' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x0E, 0x00, /*  ###       */\
    0x11, 0x00, /* #   #      */\
    0x11, 0x00, /* #   #      */\
    0x11, 0x00, /* #   #      */\
    0x0E, 0x03, /*  ###    ## */\
    0xE0, 0x03, /*      ##### */\
    0x78, 0x00, /*    ####    */\
    0x1F, 0x00, /* #####      */\
    0xC3, 0x01, /* ##    ###  */\
    0x20, 0x02, /*      #   # */\
    0x20, 0x02, /*      #   # */\
    0x20, 0x02, /*      #   # */\
    0xC0, 0x01, /*       ###  */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @168 '&' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xF8, 0x00, /*    #####   */\
    0xFC, 0x00, /*   ######   */\
    0x06, 0x00, /*  ##        */\
    0x06, 0x00, /*  ##        */\
    0x0E, 0x00, /*  ###       */\
    0x1C, 0x00, /*   ###      */\
    0xBE, 0x03, /*  ##### ### */\
    0xB7, 0x03, /* ### ## ### */\
    0xF3, 0x00, /* ##  ####   */\
    0xE3, 0x00, /* ##   ###   */\
    0xFE, 0x03, /*  ######### */\
    0xBC, 0x03, /*   #### ### */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @210 ''' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @231 '(' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @252 ')' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @273 '*' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0xDB, /* ## ## ## */\
    0xFF, /* ######## */\
    0x3C, /*   ####   */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0x66, /*  ##  ##  */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @294 '+' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @336 ',' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1C, /*   ### */\
    0x0E, /*  ###  */\
    0x06, /*  ##   */\
    0x02, /*  #    */\
    0x03, /* ##    */\
    0x01, /* #     */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @357 '-' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @399 '.' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @420 '/' (9 pixels wide) */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0xC0, 0x00, /*       ##  */\
    0xC0, 0x00, /*       ##  */\
    0x60, 0x00, /*      ##   */\
    0x60, 0x00, /*      ##   */\
    0x70, 0x00, /*     ###   */\
    0x30, 0x00, /*     ##    */\
    0x38, 0x00, /*    ###    */\
    0x18, 0x00, /*    ##     */\
    0x1C, 0x00, /*   ###     */\
    0x0C, 0x00, /*   ##      */\
    0x0C, 0x00, /*   ##      */\
    0x06, 0x00, /*  ##       */\
    0x06, 0x00, /*  ##       */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @462 '0' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFE, 0x00, /*  #######   */\
    0x86, 0x01, /*  ##    ##  */\
    0x83, 0x03, /* ##     ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x07, 0x03, /* ###     ## */\
    0x86, 0x01, /*  ##    ##  */\
    0xFC, 0x01, /*   #######  */\
    0x78, 0x00, /*    ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @504 '1' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x30, 0x00, /*     ##    */\
    0x3F, 0x00, /* ######    */\
    0x3F, 0x00, /* ######    */\
    0x30, 0x00, /*     ##    */\
    0x30, 0x00, /*     ##    */\
    0x30, 0x00, /*     ##    */\
    0x30, 0x00, /*     ##    */\
    0x30, 0x00, /*     ##    */\
    0x30, 0x00, /*     ##    */\
    0x30, 0x00, /*     ##    */\
    0x30, 0x00, /*     ##    */\
    0xFE, 0x01, /*  ######## */\
    0xFE, 0x01, /*  ######## */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @546 '2' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFE, 0x01, /*  ########  */\
    0x87, 0x03, /* ###    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x80, 0x03, /*        ### */\
    0xC0, 0x01, /*       ###  */\
    0xE0, 0x00, /*      ###   */\
    0x38, 0x00, /*    ###     */\
    0x1C, 0x00, /*   ###      */\
    0x06, 0x00, /*  ##        */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @588 '3' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xF8, 0x00, /*    #####   */\
    0xFE, 0x01, /*  ########  */\
    0x86, 0x03, /*  ##    ### */\
    0x00, 0x03, /*         ## */\
    0x80, 0x03, /*        ### */\
    0xF0, 0x01, /*     #####  */\
    0xF0, 0x01, /*     #####  */\
    0x80, 0x03, /*        ### */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x83, 0x03, /* ##     ### */\
    0xFF, 0x01, /* #########  */\
    0xFE, 0x00, /*  #######   */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @630 '4' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0xE0, 0x00, /*      ###  */\
    0xF0, 0x00, /*     ####  */\
    0xF0, 0x00, /*     ####  */\
    0xD8, 0x00, /*    ## ##  */\
    0xDC, 0x00, /*   ### ##  */\
    0xCC, 0x00, /*   ##  ##  */\
    0xC6, 0x00, /*  ##   ##  */\
    0xC6, 0x00, /*  ##   ##  */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0xC0, 0x00, /*       ##  */\
    0xF0, 0x01, /*     ##### */\
    0xF0, 0x01, /*     ##### */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @672 '5' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xFE, 0x01, /*  ########  */\
    0xFE, 0x01, /*  ########  */\
    0x06, 0x00, /*  ##        */\
    0x06, 0x00, /*  ##        */\
    0xF6, 0x00, /*  ## ####   */\
    0xFE, 0x01, /*  ########  */\
    0x86, 0x03, /*  ##    ### */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x83, 0x03, /* ##     ### */\
    0xFF, 0x01, /* #########  */\
    0xFC, 0x00, /*   ######   */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @714 '6' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0xF0, 0x01, /*     ##### */\
    0xFC, 0x01, /*   ####### */\
    0x1E, 0x00, /*  ####     */\
    0x06, 0x00, /*  ##       */\
    0x07, 0x00, /* ###       */\
    0x7B, 0x00, /* ## ####   */\
    0xFF, 0x00, /* ########  */\
    0xC7, 0x01, /* ###   ### */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0xC6, 0x01, /*  ##   ### */\
    0xFE, 0x00, /*  #######  */\
    0x78, 0x00, /*    ####   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @756 '7' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x83, 0x01, /* ##     ## */\
    0xC0, 0x01, /*       ### */\
    0xC0, 0x00, /*       ##  */\
    0xC0, 0x00, /*       ##  */\
    0xC0, 0x00, /*       ##  */\
    0x60, 0x00, /*      ##   */\
    0x60, 0x00, /*      ##   */\
    0x60, 0x00, /*      ##   */\
    0x70, 0x00, /*     ###   */\
    0x30, 0x00, /*     ##    */\
    0x30, 0x00, /*     ##    */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @798 '8' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xFC, 0x00, /*   ######   */\
    0xFE, 0x01, /*  ########  */\
    0x87, 0x03, /* ###    ### */\
    0x03, 0x03, /* ##      ## */\
    0x87, 0x03, /* ###    ### */\
    0xFE, 0x01, /*  ########  */\
    0xFE, 0x01, /*  ########  */\
    0x87, 0x03, /* ###    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x87, 0x03, /* ###    ### */\
    0xFE, 0x01, /*  ########  */\
    0xFC, 0x00, /*   ######   */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @840 '9' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x3C, 0x00, /*   ####    */\
    0xFE, 0x00, /*  #######  */\
    0xC7, 0x00, /* ###   ##  */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0xC7, 0x01, /* ###   ### */\
    0xFE, 0x01, /*  ######## */\
    0xBC, 0x01, /*   #### ## */\
    0xC0, 0x01, /*       ### */\
    0xC0, 0x00, /*       ##  */\
    0xF0, 0x00, /*     ####  */\
    0x7F, 0x00, /* #######   */\
    0x1F, 0x00, /* #####     */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @882 ':' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @903 ';' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1C, /*   ### */\
    0x1C, /*   ### */\
    0x1C, /*   ### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x06, /*  ##   */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x01, /* #     */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @924 '<' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x0C, /*           ## */\
    0x00, 0x0F, /*         #### */\
    0xC0, 0x03, /*       ####   */\
    0xF0, 0x00, /*     ####     */\
    0x3C, 0x00, /*   ####       */\
    0x0F, 0x00, /* ####         */\
    0x3C, 0x00, /*   ####       */\
    0xF0, 0x00, /*     ####     */\
    0xC0, 0x03, /*       ####   */\
    0x00, 0x0F, /*         #### */\
    0x00, 0x0C, /*           ## */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @966 '=' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @1008 '>' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x03, 0x00, /* ##           */\
    0x0F, 0x00, /* ####         */\
    0x3C, 0x00, /*   ####       */\
    0xF0, 0x00, /*     ####     */\
    0xC0, 0x03, /*       ####   */\
    0x00, 0x0F, /*         #### */\
    0xC0, 0x03, /*       ####   */\
    0xF0, 0x00, /*     ####     */\
    0x3C, 0x00, /*   ####       */\
    0x0F, 0x00, /* ####         */\
    0x03, 0x00, /* ##           */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @1050 '?' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x7E, 0x00, /*  ######   */\
    0xFF, 0x00, /* ########  */\
    0xC3, 0x01, /* ##    ### */\
    0x83, 0x01, /* ##     ## */\
    0xC0, 0x01, /*       ### */\
    0xF0, 0x00, /*     ####  */\
    0x38, 0x00, /*    ###    */\
    0x18, 0x00, /*    ##     */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x1C, 0x00, /*   ###     */\
    0x1C, 0x00, /*   ###     */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @1092 '@' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x38, /*    ###  */\
    0x26, /*  ##  #  */\
    0x42, /*  #    # */\
    0x41, /* #     # */\
    0x41, /* #     # */\
    0x71, /* #   ### */\
    0x49, /* #  #  # */\
    0x49, /* #  #  # */\
    0x49, /* #  #  # */\
    0x71, /* #   ### */\
    0x01, /* #       */\
    0x02, /*  #      */\
    0x42, /*  #    # */\
    0x3C, /*   ####  */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @1113 'A' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0xFC, 0x00, /*   ######      */\
    0xFC, 0x00, /*   ######      */\
    0xB0, 0x01, /*     ## ##     */\
    0xB0, 0x01, /*     ## ##     */\
    0xB8, 0x03, /*    ### ###    */\
    0x18, 0x03, /*    ##   ##    */\
    0x18, 0x03, /*    ##   ##    */\
    0xFC, 0x07, /*   #########   */\
    0xFC, 0x07, /*   #########   */\
    0x06, 0x0C, /*  ##       ##  */\
    0x1F, 0x1F, /* #####   ##### */\
    0x1F, 0x1F, /* #####   ##### */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @1155 'B' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xFF, 0x00, /* ########    */\
    0xFF, 0x01, /* #########   */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x86, 0x03, /*  ##    ###  */\
    0xFE, 0x01, /*  ########   */\
    0xFE, 0x03, /*  #########  */\
    0x06, 0x07, /*  ##     ### */\
    0x06, 0x06, /*  ##      ## */\
    0x06, 0x06, /*  ##      ## */\
    0xFF, 0x07, /* ########### */\
    0xFF, 0x01, /* #########   */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1197 'C' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xF8, 0x06, /*    ##### ## */\
    0xFC, 0x07, /*   ######### */\
    0x0E, 0x07, /*  ###    ### */\
    0x07, 0x06, /* ###      ## */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x07, 0x06, /* ###      ## */\
    0x0E, 0x07, /*  ###    ### */\
    0xFC, 0x03, /*   ########  */\
    0xF8, 0x01, /*    ######   */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1239 'D' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x7F, 0x00, /* #######    */\
    0xFF, 0x01, /* #########  */\
    0x86, 0x01, /*  ##    ##  */\
    0x06, 0x03, /*  ##     ## */\
    0x06, 0x03, /*  ##     ## */\
    0x06, 0x03, /*  ##     ## */\
    0x06, 0x03, /*  ##     ## */\
    0x06, 0x03, /*  ##     ## */\
    0x06, 0x03, /*  ##     ## */\
    0x86, 0x01, /*  ##    ##  */\
    0xFF, 0x01, /* #########  */\
    0x7F, 0x00, /* #######    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1281 'E' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x06, 0x03, /*  ##     ## */\
    0x06, 0x03, /*  ##     ## */\
    0x66, 0x00, /*  ##  ##    */\
    0x7E, 0x00, /*  ######    */\
    0x7E, 0x00, /*  ######    */\
    0x66, 0x00, /*  ##  ##    */\
    0x06, 0x03, /*  ##     ## */\
    0x06, 0x03, /*  ##     ## */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1323 'F' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xFF, 0x07, /* ########### */\
    0xFF, 0x07, /* ########### */\
    0x06, 0x06, /*  ##      ## */\
    0x06, 0x06, /*  ##      ## */\
    0x66, 0x00, /*  ##  ##     */\
    0x7E, 0x00, /*  ######     */\
    0x7E, 0x00, /*  ######     */\
    0x66, 0x00, /*  ##  ##     */\
    0x06, 0x00, /*  ##         */\
    0x06, 0x00, /*  ##         */\
    0x3F, 0x00, /* ######      */\
    0x3F, 0x00, /* ######      */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1365 'G' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0xF8, 0x06, /*    ##### ##  */\
    0xFE, 0x07, /*  ##########  */\
    0x06, 0x07, /*  ##     ###  */\
    0x03, 0x06, /* ##       ##  */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0xC3, 0x0F, /* ##    ###### */\
    0xC3, 0x0F, /* ##    ###### */\
    0x03, 0x06, /* ##       ##  */\
    0x06, 0x06, /*  ##      ##  */\
    0xFE, 0x07, /*  ##########  */\
    0xF8, 0x01, /*    ######    */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @1407 'H' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x8F, 0x07, /* ####   #### */\
    0x8F, 0x07, /* ####   #### */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0xFE, 0x03, /*  #########  */\
    0xFE, 0x03, /*  #########  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x8F, 0x07, /* ####   #### */\
    0x8F, 0x07, /* ####   #### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1449 'I' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @1470 'J' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xF8, 0x07, /*    ######## */\
    0xF8, 0x07, /*    ######## */\
    0x80, 0x01, /*        ##   */\
    0x80, 0x01, /*        ##   */\
    0x80, 0x01, /*        ##   */\
    0x80, 0x01, /*        ##   */\
    0x83, 0x01, /* ##     ##   */\
    0x83, 0x01, /* ##     ##   */\
    0x83, 0x01, /* ##     ##   */\
    0xC3, 0x01, /* ##    ###   */\
    0xFF, 0x00, /* ########    */\
    0x7C, 0x00, /*   #####     */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1512 'K' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0xDF, 0x07, /* ##### #####  */\
    0xDF, 0x07, /* ##### #####  */\
    0x86, 0x01, /*  ##    ##    */\
    0xC6, 0x00, /*  ##   ##     */\
    0x66, 0x00, /*  ##  ##      */\
    0x76, 0x00, /*  ## ###      */\
    0xFE, 0x00, /*  #######     */\
    0xCE, 0x01, /*  ###  ###    */\
    0x86, 0x01, /*  ##    ##    */\
    0x86, 0x03, /*  ##    ###   */\
    0x1F, 0x0F, /* #####   #### */\
    0x1F, 0x0F, /* #####   #### */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @1554 'L' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x3F, 0x00, /* ######      */\
    0x3F, 0x00, /* ######      */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x06, /*   ##     ## */\
    0x0C, 0x06, /*   ##     ## */\
    0x0C, 0x06, /*   ##     ## */\
    0xFF, 0x07, /* ########### */\
    0xFF, 0x07, /* ########### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1596 'M' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x07, 0x1C, /* ###       ### */\
    0x0F, 0x1E, /* ####     #### */\
    0x0E, 0x0E, /*  ###     ###  */\
    0x1E, 0x0F, /*  ####   ####  */\
    0xBE, 0x0F, /*  ##### #####  */\
    0xB6, 0x0D, /*  ## ## ## ##  */\
    0xE6, 0x0C, /*  ##  ###  ##  */\
    0xE6, 0x0C, /*  ##  ###  ##  */\
    0x46, 0x0C, /*  ##   #   ##  */\
    0x06, 0x0C, /*  ##       ##  */\
    0x1F, 0x1F, /* #####   ##### */\
    0x1F, 0x1F, /* #####   ##### */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @1638 'N' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xC7, 0x07, /* ###   ##### */\
    0xCF, 0x07, /* ####  ##### */\
    0x0E, 0x03, /*  ###    ##  */\
    0x1E, 0x03, /*  ####   ##  */\
    0x3E, 0x03, /*  #####  ##  */\
    0x36, 0x03, /*  ## ##  ##  */\
    0x66, 0x03, /*  ##  ## ##  */\
    0xE6, 0x03, /*  ##  #####  */\
    0xC6, 0x03, /*  ##   ####  */\
    0x86, 0x03, /*  ##    ###  */\
    0x9F, 0x03, /* #####  ###  */\
    0x1F, 0x03, /* #####   ##  */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1680 'O' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xF8, 0x00, /*    #####    */\
    0xFC, 0x01, /*   #######   */\
    0x8E, 0x03, /*  ###   ###  */\
    0x07, 0x07, /* ###     ### */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x07, 0x07, /* ###     ### */\
    0x8E, 0x03, /*  ###   ###  */\
    0xFC, 0x01, /*   #######   */\
    0xF8, 0x00, /*    #####    */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1722 'P' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xFF, 0x00, /* ########   */\
    0xFF, 0x01, /* #########  */\
    0x86, 0x03, /*  ##    ### */\
    0x06, 0x03, /*  ##     ## */\
    0x06, 0x03, /*  ##     ## */\
    0x86, 0x03, /*  ##    ### */\
    0xFE, 0x01, /*  ########  */\
    0xFE, 0x00, /*  #######   */\
    0x06, 0x00, /*  ##        */\
    0x06, 0x00, /*  ##        */\
    0x3F, 0x00, /* ######     */\
    0x3F, 0x00, /* ######     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1764 'Q' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xF8, 0x00, /*    #####    */\
    0xFC, 0x01, /*   #######   */\
    0x8E, 0x03, /*  ###   ###  */\
    0x07, 0x07, /* ###     ### */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x07, 0x07, /* ###     ### */\
    0x8E, 0x03, /*  ###   ###  */\
    0xFC, 0x01, /*   #######   */\
    0xF8, 0x00, /*    #####    */\
    0x78, 0x06, /*    ####  ## */\
    0xFC, 0x07, /*   ######### */\
    0x8C, 0x03, /*   ##   ###  */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1806 'R' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0xFF, 0x00, /* ########     */\
    0xFF, 0x01, /* #########    */\
    0x86, 0x03, /*  ##    ###   */\
    0x06, 0x03, /*  ##     ##   */\
    0x86, 0x03, /*  ##    ###   */\
    0xFE, 0x01, /*  ########    */\
    0xFE, 0x00, /*  #######     */\
    0xC6, 0x01, /*  ##   ###    */\
    0x86, 0x01, /*  ##    ##    */\
    0x86, 0x03, /*  ##    ###   */\
    0x1F, 0x0F, /* #####   #### */\
    0x1F, 0x0E, /* #####    ### */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @1848 'S' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0xBC, 0x01, /*   #### ## */\
    0xFE, 0x01, /*  ######## */\
    0xC7, 0x01, /* ###   ### */\
    0x83, 0x01, /* ##     ## */\
    0x07, 0x00, /* ###       */\
    0x7E, 0x00, /*  ######   */\
    0xFC, 0x00, /*   ######  */\
    0xC0, 0x01, /*       ### */\
    0x83, 0x01, /* ##     ## */\
    0xC7, 0x01, /* ###   ### */\
    0xFF, 0x00, /* ########  */\
    0x7B, 0x00, /* ## ####   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @1890 'T' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x33, 0x03, /* ##  ##  ## */\
    0x33, 0x03, /* ##  ##  ## */\
    0x33, 0x03, /* ##  ##  ## */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0xFC, 0x00, /*   ######   */\
    0xFC, 0x00, /*   ######   */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1932 'U' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xDF, 0x07, /* ##### ##### */\
    0xDF, 0x07, /* ##### ##### */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x8E, 0x03, /*  ###   ###  */\
    0xFC, 0x01, /*   #######   */\
    0xF8, 0x00, /*    #####    */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1974 'V' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x1F, 0x1F, /* #####   ##### */\
    0x1F, 0x1F, /* #####   ##### */\
    0x06, 0x0C, /*  ##       ##  */\
    0x0C, 0x06, /*   ##     ##   */\
    0x0C, 0x06, /*   ##     ##   */\
    0x18, 0x03, /*    ##   ##    */\
    0x18, 0x03, /*    ##   ##    */\
    0xB0, 0x01, /*     ## ##     */\
    0xB0, 0x01, /*     ## ##     */\
    0xE0, 0x00, /*      ###      */\
    0xE0, 0x00, /*      ###      */\
    0xE0, 0x00, /*      ###      */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @2016 'W' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x1F, 0x1F, /* #####   ##### */\
    0x1F, 0x1F, /* #####   ##### */\
    0x06, 0x0C, /*  ##       ##  */\
    0xE6, 0x0C, /*  ##  ###  ##  */\
    0xE6, 0x0C, /*  ##  ###  ##  */\
    0xF6, 0x0D, /*  ## ##### ##  */\
    0xB6, 0x0D, /*  ## ## ## ##  */\
    0xB4, 0x05, /*   # ## ## #   */\
    0xBC, 0x07, /*   #### ####   */\
    0x1C, 0x07, /*   ###   ###   */\
    0x1C, 0x07, /*   ###   ###   */\
    0x0C, 0x06, /*   ##     ##   */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @2058 'X' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xDF, 0x07, /* ##### ##### */\
    0xDF, 0x07, /* ##### ##### */\
    0x8C, 0x01, /*   ##   ##   */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0x8C, 0x01, /*   ##   ##   */\
    0xDF, 0x07, /* ##### ##### */\
    0xDF, 0x07, /* ##### ##### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2100 'Y' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x9F, 0x0F, /* #####  ##### */\
    0x9F, 0x0F, /* #####  ##### */\
    0x0C, 0x03, /*   ##    ##   */\
    0x98, 0x01, /*    ##  ##    */\
    0xF0, 0x00, /*     ####     */\
    0xF0, 0x00, /*     ####     */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0xF8, 0x01, /*    ######    */\
    0xF8, 0x01, /*    ######    */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @2142 'Z' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x83, 0x01, /* ##     ## */\
    0xC3, 0x00, /* ##    ##  */\
    0x63, 0x00, /* ##   ##   */\
    0x30, 0x00, /*     ##    */\
    0x18, 0x00, /*    ##     */\
    0x8C, 0x01, /*   ##   ## */\
    0x86, 0x01, /*  ##    ## */\
    0x83, 0x01, /* ##     ## */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @2184 '[' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @2205 '\' (10 pixels wide) */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x06, 0x00, /*  ##        */\
    0x06, 0x00, /*  ##        */\
    0x0C, 0x00, /*   ##       */\
    0x0C, 0x00, /*   ##       */\
    0x18, 0x00, /*    ##      */\
    0x38, 0x00, /*    ###     */\
    0x30, 0x00, /*     ##     */\
    0x70, 0x00, /*     ###    */\
    0x60, 0x00, /*      ##    */\
    0xC0, 0x00, /*       ##   */\
    0xC0, 0x00, /*       ##   */\
    0x80, 0x01, /*        ##  */\
    0x80, 0x01, /*        ##  */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @2247 ']' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @2268 '^' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x10, 0x00, /*     #     */\
    0x38, 0x00, /*    ###    */\
    0x7C, 0x00, /*   #####   */\
    0x6C, 0x00, /*   ## ##   */\
    0xC6, 0x00, /*  ##   ##  */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @2310 '_' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0xFF, 0x7F, /* ############### */\
    0xFF, 0x7F, /* ############### */\
\
    /* @2352 '`' (4 pixels wide) */\
    0x00, /*      */\
    0x01, /* #    */\
    0x06, /*  ##  */\
    0x08, /*    # */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @2373 'a' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xFC, 0x01, /*   #######   */\
    0xFE, 0x03, /*  #########  */\
    0x00, 0x03, /*         ##  */\
    0xF8, 0x03, /*    #######  */\
    0xFE, 0x03, /*  #########  */\
    0x07, 0x03, /* ###     ##  */\
    0x83, 0x03, /* ##     ###  */\
    0xFF, 0x07, /* ########### */\
    0x7E, 0x07, /*  ###### ### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2415 'b' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x07, 0x00, /* ###         */\
    0x07, 0x00, /* ###         */\
    0x06, 0x00, /*  ##         */\
    0x06, 0x00, /*  ##         */\
    0xF6, 0x00, /*  ## ####    */\
    0xFE, 0x03, /*  #########  */\
    0x0E, 0x03, /*  ###    ##  */\
    0x06, 0x06, /*  ##      ## */\
    0x06, 0x06, /*  ##      ## */\
    0x06, 0x06, /*  ##      ## */\
    0x0E, 0x03, /*  ###    ##  */\
    0xFF, 0x03, /* ##########  */\
    0xF7, 0x00, /* ### ####    */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2457 'c' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xF8, 0x06, /*    ##### ## */\
    0xFE, 0x07, /*  ########## */\
    0x06, 0x07, /*  ##     ### */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x07, 0x06, /* ###      ## */\
    0xFE, 0x07, /*  ########## */\
    0xF8, 0x01, /*    ######   */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2499 'd' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x80, 0x03, /*        ###  */\
    0x80, 0x03, /*        ###  */\
    0x00, 0x03, /*         ##  */\
    0x00, 0x03, /*         ##  */\
    0x78, 0x03, /*    #### ##  */\
    0xFE, 0x03, /*  #########  */\
    0x86, 0x03, /*  ##    ###  */\
    0x03, 0x03, /* ##      ##  */\
    0x03, 0x03, /* ##      ##  */\
    0x03, 0x03, /* ##      ##  */\
    0x87, 0x03, /* ###    ###  */\
    0xFE, 0x07, /*  ########## */\
    0x78, 0x07, /*    #### ### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2541 'e' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xF8, 0x00, /*    #####    */\
    0xFE, 0x03, /*  #########  */\
    0x06, 0x03, /*  ##     ##  */\
    0xFF, 0x07, /* ########### */\
    0xFF, 0x07, /* ########### */\
    0x03, 0x00, /* ##          */\
    0x06, 0x06, /*  ##      ## */\
    0xFE, 0x07, /*  ########## */\
    0xF8, 0x01, /*    ######   */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2583 'f' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xF0, 0x03, /*     ###### */\
    0xF8, 0x03, /*    ####### */\
    0x0C, 0x00, /*   ##       */\
    0x0C, 0x00, /*   ##       */\
    0xFF, 0x00, /* ########   */\
    0xFF, 0x00, /* ########   */\
    0x0C, 0x00, /*   ##       */\
    0x0C, 0x00, /*   ##       */\
    0x0C, 0x00, /*   ##       */\
    0x0C, 0x00, /*   ##       */\
    0x0C, 0x00, /*   ##       */\
    0xFF, 0x00, /* ########   */\
    0xFF, 0x00, /* ########   */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @2625 'g' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x78, 0x07, /*    #### ### */\
    0xFE, 0x07, /*  ########## */\
    0x86, 0x03, /*  ##    ###  */\
    0x03, 0x03, /* ##      ##  */\
    0x03, 0x03, /* ##      ##  */\
    0x03, 0x03, /* ##      ##  */\
    0x86, 0x03, /*  ##    ###  */\
    0xFE, 0x03, /*  #########  */\
    0x78, 0x03, /*    #### ##  */\
    0x00, 0x03, /*         ##  */\
    0x80, 0x03, /*        ###  */\
    0xFC, 0x01, /*   #######   */\
    0xFC, 0x00, /*   ######    */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2667 'h' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x07, 0x00, /* ###         */\
    0x07, 0x00, /* ###         */\
    0x06, 0x00, /*  ##         */\
    0x06, 0x00, /*  ##         */\
    0xF6, 0x00, /*  ## ####    */\
    0xFE, 0x01, /*  ########   */\
    0x0E, 0x03, /*  ###    ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x8F, 0x07, /* ####   #### */\
    0x8F, 0x07, /* ####   #### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2709 'i' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x3E, 0x00, /*  #####     */\
    0x3E, 0x00, /*  #####     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @2751 'j' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x30, /*     ##   */\
    0x30, /*     ##   */\
    0x00, /*          */\
    0x00, /*          */\
    0xFE, /*  ####### */\
    0xFE, /*  ####### */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xE0, /*      ### */\
    0x7F, /* #######  */\
    0x3F, /* ######   */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2772 'k' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x07, 0x00, /* ###         */\
    0x07, 0x00, /* ###         */\
    0x06, 0x00, /*  ##         */\
    0x06, 0x00, /*  ##         */\
    0xE6, 0x03, /*  ##  #####  */\
    0xE6, 0x03, /*  ##  #####  */\
    0x76, 0x00, /*  ## ###     */\
    0x3E, 0x00, /*  #####      */\
    0x3E, 0x00, /*  #####      */\
    0x76, 0x00, /*  ## ###     */\
    0xE6, 0x00, /*  ##  ###    */\
    0xC7, 0x07, /* ###   ##### */\
    0xC7, 0x07, /* ###   ##### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2814 'l' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x3E, 0x00, /*  #####     */\
    0x3E, 0x00, /*  #####     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @2856 'm' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x77, 0x0F, /* ### ### ####   */\
    0xFF, 0x1F, /* #############  */\
    0xCE, 0x19, /*  ###  ###  ##  */\
    0xC6, 0x18, /*  ##   ##   ##  */\
    0xC6, 0x18, /*  ##   ##   ##  */\
    0xC6, 0x18, /*  ##   ##   ##  */\
    0xC6, 0x18, /*  ##   ##   ##  */\
    0xCF, 0x39, /* ####  ###  ### */\
    0xCF, 0x39, /* ####  ###  ### */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @2898 'n' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xF7, 0x00, /* ### ####    */\
    0xFF, 0x01, /* #########   */\
    0x0E, 0x03, /*  ###    ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x8F, 0x07, /* ####   #### */\
    0x8F, 0x07, /* ####   #### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2940 'o' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xF8, 0x00, /*    #####    */\
    0xFE, 0x03, /*  #########  */\
    0x06, 0x03, /*  ##     ##  */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x06, 0x03, /*  ##     ##  */\
    0xFE, 0x03, /*  #########  */\
    0xF8, 0x00, /*    #####    */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @2982 'p' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xF7, 0x00, /* ### ####    */\
    0xFF, 0x03, /* ##########  */\
    0x0E, 0x03, /*  ###    ##  */\
    0x06, 0x06, /*  ##      ## */\
    0x06, 0x06, /*  ##      ## */\
    0x06, 0x06, /*  ##      ## */\
    0x0E, 0x03, /*  ###    ##  */\
    0xFE, 0x03, /*  #########  */\
    0xF6, 0x00, /*  ## ####    */\
    0x06, 0x00, /*  ##         */\
    0x06, 0x00, /*  ##         */\
    0x1F, 0x00, /* #####       */\
    0x1F, 0x00, /* #####       */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @3024 'q' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x78, 0x07, /*    #### ### */\
    0xFE, 0x07, /*  ########## */\
    0x86, 0x03, /*  ##    ###  */\
    0x03, 0x03, /* ##      ##  */\
    0x03, 0x03, /* ##      ##  */\
    0x03, 0x03, /* ##      ##  */\
    0x86, 0x03, /*  ##    ###  */\
    0xFE, 0x03, /*  #########  */\
    0x78, 0x03, /*    #### ##  */\
    0x00, 0x03, /*         ##  */\
    0x00, 0x03, /*         ##  */\
    0xC0, 0x07, /*       ##### */\
    0xC0, 0x07, /*       ##### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @3066 'r' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xCF, 0x03, /* ####  ####  */\
    0xEF, 0x07, /* #### ###### */\
    0x7C, 0x06, /*   #####  ## */\
    0x1C, 0x00, /*   ###       */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0xFF, 0x00, /* ########    */\
    0xFF, 0x00, /* ########    */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @3108 's' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0xFC, 0x01, /*   ####### */\
    0xFF, 0x01, /* ######### */\
    0x83, 0x01, /* ##     ## */\
    0x1F, 0x00, /* #####     */\
    0xFE, 0x00, /*  #######  */\
    0xE0, 0x01, /*      #### */\
    0x83, 0x01, /* ##     ## */\
    0xFF, 0x01, /* ######### */\
    0x7F, 0x00, /* #######   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @3150 't' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0xFF, 0x03, /* ##########  */\
    0xFF, 0x03, /* ##########  */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x00, /*   ##        */\
    0x0C, 0x06, /*   ##     ## */\
    0xFC, 0x07, /*   ######### */\
    0xF8, 0x01, /*    ######   */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @3192 'u' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x87, 0x03, /* ###    ###  */\
    0x87, 0x03, /* ###    ###  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x86, 0x03, /*  ##    ###  */\
    0xFE, 0x07, /*  ########## */\
    0x7C, 0x07, /*   ##### ### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @3234 'v' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xDF, 0x07, /* ##### ##### */\
    0xDF, 0x07, /* ##### ##### */\
    0x06, 0x03, /*  ##     ##  */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @3276 'w' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x1F, 0x1F, /* #####   ##### */\
    0x1F, 0x1F, /* #####   ##### */\
    0x46, 0x0C, /*  ##   #   ##  */\
    0xC6, 0x0C, /*  ##   ##  ##  */\
    0xEC, 0x06, /*   ## ### ##   */\
    0xAC, 0x06, /*   ## # # ##   */\
    0xBC, 0x07, /*   #### ####   */\
    0x18, 0x03, /*    ##   ##    */\
    0x18, 0x03, /*    ##   ##    */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @3318 'x' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0xDF, 0x07, /* ##### ##### */\
    0xDF, 0x07, /* ##### ##### */\
    0x8C, 0x01, /*   ##   ##   */\
    0xD8, 0x00, /*    ## ##    */\
    0x70, 0x00, /*     ###     */\
    0xD8, 0x00, /*    ## ##    */\
    0x8C, 0x01, /*   ##   ##   */\
    0xDF, 0x07, /* ##### ##### */\
    0xDF, 0x07, /* ##### ##### */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @3360 'y' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x9F, 0x0F, /* #####  ##### */\
    0x9F, 0x0F, /* #####  ##### */\
    0x06, 0x06, /*  ##      ##  */\
    0x0C, 0x03, /*   ##    ##   */\
    0x8C, 0x03, /*   ##   ###   */\
    0x98, 0x01, /*    ##  ##    */\
    0xD8, 0x00, /*    ## ##     */\
    0xF0, 0x00, /*     ####     */\
    0x70, 0x00, /*     ###      */\
    0x60, 0x00, /*      ##      */\
    0x30, 0x00, /*     ##       */\
    0xFF, 0x00, /* ########     */\
    0xFF, 0x00, /* ########     */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @3402 'z' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0xC3, 0x00, /* ##    ##  */\
    0x60, 0x00, /*      ##   */\
    0x38, 0x00, /*    ###    */\
    0x0C, 0x00, /*   ##      */\
    0x86, 0x01, /*  ##    ## */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @3444 '{' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x38, /*    ### */\
    0x3C, /*   #### */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0E, /*  ###   */\
    0x07, /* ###    */\
    0x0E, /*  ###   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x3C, /*   #### */\
    0x38, /*    ### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @3465 '|' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @3486 '}' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x07, /* ###    */\
    0x0F, /* ####   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x1C, /*   ###  */\
    0x38, /*    ### */\
    0x1C, /*   ###  */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0F, /* ####   */\
    0x07, /* ###    */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @3507 '~' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x1C, 0x03, /*   ###   ## */\
    0xBE, 0x03, /*  ##### ### */\
    0xF7, 0x01, /* ### #####  */\
    0xE3, 0x00, /* ##   ###   */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
}


/** @brief Initialization macro for the corierNew_b16pt font information. */
#define FONTS_CORIERNEW_B16PT_INFO \
{\
    -1, /* Character width (-1 means dynamic width). */\
    21, /* Character height (-1 means dynamic height). */\
    -1, /* Bitmap offset (-1 means dynamic offset). */\
    3, /* Deescriptor entry size. */\
    ' ', /* Start character. */\
    '~', /* End character. */\
    fonts_coriernew_b16pt_descriptors, /*  Character descriptor array. */\
    fonts_coriernew_b16pt_bitmaps, /* Character bitmap array. */\
}


/** @brief Initialization macro for the descriptors of the corierNew_b7pt font. */
#define FONTS_CORIERNEW_B7PT_DESCRIPTORS \
{\
         8,  /*   */\
         1,  /* ! */\
         3,  /* " */\
         5,  /* # */\
         4,  /* $ */\
         4,  /* % */\
         4,  /* & */\
         1,  /* ' */\
         2,  /* ( */\
         2,  /* ) */\
         3,  /* * */\
         5,  /* + */\
         2,  /* , */\
         3,  /* - */\
         1,  /* . */\
         3,  /* / */\
         4,  /* 0 */\
         5,  /* 1 */\
         4,  /* 2 */\
         4,  /* 3 */\
         4,  /* 4 */\
         4,  /* 5 */\
         4,  /* 6 */\
         4,  /* 7 */\
         4,  /* 8 */\
         4,  /* 9 */\
         1,  /* : */\
         2,  /* ; */\
         4,  /* < */\
         4,  /* = */\
         4,  /* > */\
         4,  /* ? */\
         4,  /* @ */\
         6,  /* A */\
         5,  /* B */\
         4,  /* C */\
         5,  /* D */\
         5,  /* E */\
         5,  /* F */\
         4,  /* G */\
         6,  /* H */\
         3,  /* I */\
         5,  /* J */\
         6,  /* K */\
         5,  /* L */\
         6,  /* M */\
         6,  /* N */\
         4,  /* O */\
         5,  /* P */\
         4,  /* Q */\
         6,  /* R */\
         4,  /* S */\
         5,  /* T */\
         6,  /* U */\
         6,  /* V */\
         7,  /* W */\
         6,  /* X */\
         6,  /* Y */\
         4,  /* Z */\
         3,  /* [ */\
         4,  /* \ */\
         3,  /* ] */\
         3,  /* ^ */\
         6,  /* _ */\
         2,  /* ` */\
         5,  /* a */\
         5,  /* b */\
         4,  /* c */\
         5,  /* d */\
         4,  /* e */\
         5,  /* f */\
         5,  /* g */\
         6,  /* h */\
         5,  /* i */\
         3,  /* j */\
         5,  /* k */\
         5,  /* l */\
         7,  /* m */\
         6,  /* n */\
         4,  /* o */\
         5,  /* p */\
         5,  /* q */\
         4,  /* r */\
         4,  /* s */\
         4,  /* t */\
         6,  /* u */\
         6,  /* v */\
         6,  /* w */\
         5,  /* x */\
         6,  /* y */\
         4,  /* z */\
         3,  /* { */\
         1,  /* | */\
         3,  /* } */\
         4,  /* ~ */\
}


/** @brief Initialization macro for the bitmaps of the corierNew_b7pt font. */
#define FONTS_CORIERNEW_B7PT_BITMAPS \
{\
\
    /* @0 ' ' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @10 '!' (1 pixels wide) */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @20 '"' (3 pixels wide) */\
    0x00, /*     */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @30 '#' (5 pixels wide) */\
    0x00, /*       */\
    0x0A, /*  # #  */\
    0x0A, /*  # #  */\
    0x1F, /* ##### */\
    0x0A, /*  # #  */\
    0x1F, /* ##### */\
    0x0A, /*  # #  */\
    0x0A, /*  # #  */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @40 '$' (4 pixels wide) */\
    0x00, /*      */\
    0x02, /*  #   */\
    0x0F, /* #### */\
    0x01, /* #    */\
    0x06, /*  ##  */\
    0x08, /*    # */\
    0x07, /* ###  */\
    0x02, /*  #   */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @50 '%' (4 pixels wide) */\
    0x00, /*      */\
    0x07, /* ###  */\
    0x05, /* # #  */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x0A, /*  # # */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @60 '&' (4 pixels wide) */\
    0x00, /*      */\
    0x04, /*   #  */\
    0x02, /*  #   */\
    0x02, /*  #   */\
    0x0F, /* #### */\
    0x05, /* # #  */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @70 ''' (1 pixels wide) */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @80 '(' (2 pixels wide) */\
    0x00, /*    */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x02, /*  # */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @90 ')' (2 pixels wide) */\
    0x00, /*    */\
    0x01, /* #  */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @100 '*' (3 pixels wide) */\
    0x00, /*     */\
    0x02, /*  #  */\
    0x07, /* ### */\
    0x02, /*  #  */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @110 '+' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @120 ',' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x00, /*    */\
\
    /* @130 '-' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @140 '.' (1 pixels wide) */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @150 '/' (3 pixels wide) */\
    0x00, /*     */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @160 '0' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @170 '1' (5 pixels wide) */\
    0x00, /*       */\
    0x07, /* ###   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @180 '2' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x08, /*    # */\
    0x04, /*   #  */\
    0x02, /*  #   */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @190 '3' (4 pixels wide) */\
    0x00, /*      */\
    0x07, /* ###  */\
    0x08, /*    # */\
    0x06, /*  ##  */\
    0x08, /*    # */\
    0x08, /*    # */\
    0x07, /* ###  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @200 '4' (4 pixels wide) */\
    0x00, /*      */\
    0x04, /*   #  */\
    0x06, /*  ##  */\
    0x05, /* # #  */\
    0x0F, /* #### */\
    0x04, /*   #  */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @210 '5' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x01, /* #    */\
    0x07, /* ###  */\
    0x08, /*    # */\
    0x08, /*    # */\
    0x07, /* ###  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @220 '6' (4 pixels wide) */\
    0x00, /*      */\
    0x0E, /*  ### */\
    0x01, /* #    */\
    0x07, /* ###  */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @230 '7' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x09, /* #  # */\
    0x08, /*    # */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @240 '8' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @250 '9' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x0E, /*  ### */\
    0x08, /*    # */\
    0x07, /* ###  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @260 ':' (1 pixels wide) */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @270 ';' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x02, /*  # */\
    0x00, /*    */\
    0x00, /*    */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @280 '<' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x08, /*    # */\
    0x06, /*  ##  */\
    0x01, /* #    */\
    0x06, /*  ##  */\
    0x08, /*    # */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @290 '=' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @300 '>' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x01, /* #    */\
    0x06, /*  ##  */\
    0x08, /*    # */\
    0x06, /*  ##  */\
    0x01, /* #    */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @310 '?' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x04, /*   #  */\
    0x02, /*  #   */\
    0x00, /*      */\
    0x02, /*  #   */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @320 '@' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x0D, /* # ## */\
    0x0D, /* # ## */\
    0x0D, /* # ## */\
    0x01, /* #    */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @330 'A' (6 pixels wide) */\
    0x00, /*        */\
    0x0E, /*  ###   */\
    0x0C, /*   ##   */\
    0x12, /*  #  #  */\
    0x1E, /*  ####  */\
    0x12, /*  #  #  */\
    0x37, /* ### ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @340 'B' (5 pixels wide) */\
    0x00, /*       */\
    0x0F, /* ####  */\
    0x12, /*  #  # */\
    0x0E, /*  ###  */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x0F, /* ####  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @350 'C' (4 pixels wide) */\
    0x00, /*      */\
    0x0E, /*  ### */\
    0x09, /* #  # */\
    0x01, /* #    */\
    0x01, /* #    */\
    0x09, /* #  # */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @360 'D' (5 pixels wide) */\
    0x00, /*       */\
    0x0F, /* ####  */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x0F, /* ####  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @370 'E' (5 pixels wide) */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x12, /*  #  # */\
    0x0E, /*  ###  */\
    0x0A, /*  # #  */\
    0x12, /*  #  # */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @380 'F' (5 pixels wide) */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x12, /*  #  # */\
    0x0E, /*  ###  */\
    0x0A, /*  # #  */\
    0x02, /*  #    */\
    0x0F, /* ####  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @390 'G' (4 pixels wide) */\
    0x00, /*      */\
    0x0E, /*  ### */\
    0x09, /* #  # */\
    0x01, /* #    */\
    0x0D, /* # ## */\
    0x09, /* #  # */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @400 'H' (6 pixels wide) */\
    0x00, /*        */\
    0x3B, /* ## ### */\
    0x12, /*  #  #  */\
    0x1E, /*  ####  */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x37, /* ### ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @410 'I' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @420 'J' (5 pixels wide) */\
    0x00, /*       */\
    0x1E, /*  #### */\
    0x08, /*    #  */\
    0x08, /*    #  */\
    0x09, /* #  #  */\
    0x09, /* #  #  */\
    0x07, /* ###   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @430 'K' (6 pixels wide) */\
    0x00, /*        */\
    0x37, /* ### ## */\
    0x12, /*  #  #  */\
    0x0A, /*  # #   */\
    0x0E, /*  ###   */\
    0x12, /*  #  #  */\
    0x37, /* ### ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @440 'L' (5 pixels wide) */\
    0x00, /*       */\
    0x07, /* ###   */\
    0x02, /*  #    */\
    0x02, /*  #    */\
    0x02, /*  #    */\
    0x12, /*  #  # */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @450 'M' (6 pixels wide) */\
    0x00, /*        */\
    0x33, /* ##  ## */\
    0x12, /*  #  #  */\
    0x1E, /*  ####  */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x37, /* ### ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @460 'N' (6 pixels wide) */\
    0x00, /*        */\
    0x3B, /* ## ### */\
    0x16, /*  ## #  */\
    0x16, /*  ## #  */\
    0x1A, /*  # ##  */\
    0x1A, /*  # ##  */\
    0x17, /* ### #  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @470 'O' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @480 'P' (5 pixels wide) */\
    0x00, /*       */\
    0x0F, /* ####  */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x0E, /*  ###  */\
    0x02, /*  #    */\
    0x07, /* ###   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @490 'Q' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @500 'R' (6 pixels wide) */\
    0x00, /*        */\
    0x0F, /* ####   */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x0E, /*  ###   */\
    0x0A, /*  # #   */\
    0x37, /* ### ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @510 'S' (4 pixels wide) */\
    0x00, /*      */\
    0x0E, /*  ### */\
    0x09, /* #  # */\
    0x02, /*  #   */\
    0x04, /*   #  */\
    0x09, /* #  # */\
    0x07, /* ###  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @520 'T' (5 pixels wide) */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x15, /* # # # */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @530 'U' (6 pixels wide) */\
    0x00, /*        */\
    0x37, /* ### ## */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x0C, /*   ##   */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @540 'V' (6 pixels wide) */\
    0x00, /*        */\
    0x3B, /* ## ### */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @550 'W' (7 pixels wide) */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x14, /*   # #   */\
    0x14, /*   # #   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @560 'X' (6 pixels wide) */\
    0x00, /*        */\
    0x3B, /* ## ### */\
    0x12, /*  #  #  */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x12, /*  #  #  */\
    0x37, /* ### ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @570 'Y' (6 pixels wide) */\
    0x00, /*        */\
    0x33, /* ##  ## */\
    0x12, /*  #  #  */\
    0x0C, /*   ##   */\
    0x04, /*   #    */\
    0x04, /*   #    */\
    0x0E, /*  ###   */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @580 'Z' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x09, /* #  # */\
    0x05, /* # #  */\
    0x0A, /*  # # */\
    0x09, /* #  # */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @590 '[' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @600 '\' (4 pixels wide) */\
    0x00, /*      */\
    0x01, /* #    */\
    0x02, /*  #   */\
    0x02, /*  #   */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x08, /*    # */\
    0x08, /*    # */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @610 ']' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @620 '^' (3 pixels wide) */\
    0x00, /*     */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x05, /* # # */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @630 '_' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x3F, /* ###### */\
\
    /* @640 '`' (2 pixels wide) */\
    0x01, /* #  */\
    0x02, /*  # */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @650 'a' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x07, /* ###   */\
    0x0E, /*  ###  */\
    0x09, /* #  #  */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @660 'b' (5 pixels wide) */\
    0x00, /*       */\
    0x03, /* ##    */\
    0x02, /*  #    */\
    0x0E, /*  ###  */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x0F, /* ####  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @670 'c' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x0E, /*  ### */\
    0x09, /* #  # */\
    0x01, /* #    */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @680 'd' (5 pixels wide) */\
    0x00, /*       */\
    0x0C, /*   ##  */\
    0x08, /*    #  */\
    0x0E, /*  ###  */\
    0x09, /* #  #  */\
    0x09, /* #  #  */\
    0x1E, /*  #### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @690 'e' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x0F, /* #### */\
    0x01, /* #    */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @700 'f' (5 pixels wide) */\
    0x00, /*       */\
    0x1C, /*   ### */\
    0x02, /*  #    */\
    0x07, /* ###   */\
    0x02, /*  #    */\
    0x02, /*  #    */\
    0x0F, /* ####  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @710 'g' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1E, /*  #### */\
    0x09, /* #  #  */\
    0x09, /* #  #  */\
    0x0E, /*  ###  */\
    0x08, /*    #  */\
    0x06, /*  ##   */\
    0x00, /*       */\
\
    /* @720 'h' (6 pixels wide) */\
    0x00, /*        */\
    0x03, /* ##     */\
    0x02, /*  #     */\
    0x0E, /*  ###   */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x37, /* ### ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @730 'i' (5 pixels wide) */\
    0x00, /*       */\
    0x04, /*   #   */\
    0x00, /*       */\
    0x07, /* ###   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @740 'j' (3 pixels wide) */\
    0x00, /*     */\
    0x02, /*  #  */\
    0x00, /*     */\
    0x07, /* ### */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x03, /* ##  */\
    0x00, /*     */\
\
    /* @750 'k' (5 pixels wide) */\
    0x00, /*       */\
    0x03, /* ##    */\
    0x02, /*  #    */\
    0x1A, /*  # ## */\
    0x06, /*  ##   */\
    0x0A, /*  # #  */\
    0x1B, /* ## ## */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @760 'l' (5 pixels wide) */\
    0x00, /*       */\
    0x07, /* ###   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @770 'm' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x1F, /* #####   */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x6B, /* ## # ## */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @780 'n' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x0F, /* ####   */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x37, /* ### ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @790 'o' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @800 'p' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x0F, /* ####  */\
    0x12, /*  #  # */\
    0x12, /*  #  # */\
    0x0E, /*  ###  */\
    0x02, /*  #    */\
    0x07, /* ###   */\
    0x00, /*       */\
\
    /* @810 'q' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1E, /*  #### */\
    0x09, /* #  #  */\
    0x09, /* #  #  */\
    0x0E, /*  ###  */\
    0x08, /*    #  */\
    0x1C, /*   ### */\
    0x00, /*       */\
\
    /* @820 'r' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x02, /*  #   */\
    0x02, /*  #   */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @830 's' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x0E, /*  ### */\
    0x03, /* ##   */\
    0x0C, /*   ## */\
    0x07, /* ###  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @840 't' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x02, /*  #   */\
    0x0F, /* #### */\
    0x02, /*  #   */\
    0x02, /*  #   */\
    0x0C, /*   ## */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @850 'u' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x1B, /* ## ##  */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x3C, /*   #### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @860 'v' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x33, /* ##  ## */\
    0x12, /*  #  #  */\
    0x12, /*  #  #  */\
    0x0C, /*   ##   */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @870 'w' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x37, /* ### ## */\
    0x2A, /*  # # # */\
    0x14, /*   # #  */\
    0x14, /*   # #  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @880 'x' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1B, /* ## ## */\
    0x0A, /*  # #  */\
    0x0E, /*  ###  */\
    0x1B, /* ## ## */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @890 'y' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x37, /* ### ## */\
    0x12, /*  #  #  */\
    0x0C, /*   ##   */\
    0x04, /*   #    */\
    0x04, /*   #    */\
    0x0F, /* ####   */\
    0x00, /*        */\
\
    /* @900 'z' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x05, /* # #  */\
    0x0A, /*  # # */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @910 '{' (3 pixels wide) */\
    0x00, /*     */\
    0x06, /*  ## */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x01, /* #   */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x06, /*  ## */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @920 '|' (1 pixels wide) */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @930 '}' (3 pixels wide) */\
    0x00, /*     */\
    0x03, /* ##  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x04, /*   # */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x03, /* ##  */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @940 '~' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x0B, /* ## # */\
    0x0D, /* # ## */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
}


/** @brief Initialization macro for the corierNew_b7pt font information. */
#define FONTS_CORIERNEW_B7PT_INFO \
{\
    -1, /* Character width (-1 means dynamic width). */\
    10, /* Character height (-1 means dynamic height). */\
    10, /* Bitmap offset (-1 means dynamic offset). */\
    1, /* Deescriptor entry size. */\
    ' ', /* Start character. */\
    '~', /* End character. */\
    fonts_coriernew_b7pt_descriptors, /*  Character descriptor array. */\
    fonts_coriernew_b7pt_bitmaps, /* Character bitmap array. */\
}


/** @brief Initialization macro for the descriptors of the generic_8pt font. */
#define FONTS_GENERIC_8PT_DESCRIPTORS \
{\
    0 /* No descriptor needed for this font. */\
}


/** @brief Initialization macro for the bitmaps of the generic_8pt font. */
#define FONTS_GENERIC_8PT_BITMAPS \
{\
\
    /* @0 ' ' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @8 '!' (8 pixels wide) */\
    0x18, /*    ##    */\
    0x3C, /*   ####   */\
    0x3C, /*   ####   */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x00, /*          */\
    0x18, /*    ##    */\
    0x00, /*          */\
\
    /* @16 '"' (8 pixels wide) */\
    0x36, /*  ## ##   */\
    0x36, /*  ## ##   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @24 '#' (8 pixels wide) */\
    0x36, /*  ## ##   */\
    0x36, /*  ## ##   */\
    0x7F, /* #######  */\
    0x36, /*  ## ##   */\
    0x7F, /* #######  */\
    0x36, /*  ## ##   */\
    0x36, /*  ## ##   */\
    0x00, /*          */\
\
    /* @32 '$' (8 pixels wide) */\
    0x0C, /*   ##     */\
    0x3E, /*  #####   */\
    0x03, /* ##       */\
    0x1E, /*  ####    */\
    0x30, /*     ##   */\
    0x1F, /* #####    */\
    0x0C, /*   ##     */\
    0x00, /*          */\
\
    /* @40 '%' (8 pixels wide) */\
    0x00, /*          */\
    0x63, /* ##   ##  */\
    0x33, /* ##  ##   */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x66, /*  ##  ##  */\
    0x63, /* ##   ##  */\
    0x00, /*          */\
\
    /* @48 '&' (8 pixels wide) */\
    0x1C, /*   ###    */\
    0x36, /*  ## ##   */\
    0x1C, /*   ###    */\
    0x6E, /*  ### ##  */\
    0x3B, /* ## ###   */\
    0x33, /* ##  ##   */\
    0x6E, /*  ### ##  */\
    0x00, /*          */\
\
    /* @56 ''' (8 pixels wide) */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x03, /* ##       */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @64 '(' (8 pixels wide) */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x0C, /*   ##     */\
    0x18, /*    ##    */\
    0x00, /*          */\
\
    /* @72 ')' (8 pixels wide) */\
    0x06, /*  ##      */\
    0x0C, /*   ##     */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x06, /*  ##      */\
    0x00, /*          */\
\
    /* @80 '*' (8 pixels wide) */\
    0x00, /*          */\
    0x66, /*  ##  ##  */\
    0x3C, /*   ####   */\
    0xFF, /* ######## */\
    0x3C, /*   ####   */\
    0x66, /*  ##  ##  */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @88 '+' (8 pixels wide) */\
    0x00, /*          */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x3F, /* ######   */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @96 ',' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x06, /*  ##      */\
\
    /* @104 '-' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x3F, /* ######   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @112 '.' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x00, /*          */\
\
    /* @120 '/' (8 pixels wide) */\
    0x60, /*      ##  */\
    0x30, /*     ##   */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x06, /*  ##      */\
    0x03, /* ##       */\
    0x01, /* #        */\
    0x00, /*          */\
\
    /* @128 '0' (8 pixels wide) */\
    0x3E, /*  #####   */\
    0x63, /* ##   ##  */\
    0x73, /* ##  ###  */\
    0x7B, /* ## ####  */\
    0x6F, /* #### ##  */\
    0x67, /* ###  ##  */\
    0x3E, /*  #####   */\
    0x00, /*          */\
\
    /* @136 '1' (8 pixels wide) */\
    0x0C, /*   ##     */\
    0x0E, /*  ###     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x3F, /* ######   */\
    0x00, /*          */\
\
    /* @144 '2' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x30, /*     ##   */\
    0x1C, /*   ###    */\
    0x06, /*  ##      */\
    0x33, /* ##  ##   */\
    0x3F, /* ######   */\
    0x00, /*          */\
\
    /* @152 '3' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x30, /*     ##   */\
    0x1C, /*   ###    */\
    0x30, /*     ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @160 '4' (8 pixels wide) */\
    0x38, /*    ###   */\
    0x3C, /*   ####   */\
    0x36, /*  ## ##   */\
    0x33, /* ##  ##   */\
    0x7F, /* #######  */\
    0x30, /*     ##   */\
    0x78, /*    ####  */\
    0x00, /*          */\
\
    /* @168 '5' (8 pixels wide) */\
    0x3F, /* ######   */\
    0x03, /* ##       */\
    0x1F, /* #####    */\
    0x30, /*     ##   */\
    0x30, /*     ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @176 '6' (8 pixels wide) */\
    0x1C, /*   ###    */\
    0x06, /*  ##      */\
    0x03, /* ##       */\
    0x1F, /* #####    */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @184 '7' (8 pixels wide) */\
    0x3F, /* ######   */\
    0x33, /* ##  ##   */\
    0x30, /*     ##   */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x00, /*          */\
\
    /* @192 '8' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @200 '9' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x3E, /*  #####   */\
    0x30, /*     ##   */\
    0x18, /*    ##    */\
    0x0E, /*  ###     */\
    0x00, /*          */\
\
    /* @208 ':' (8 pixels wide) */\
    0x00, /*          */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x00, /*          */\
    0x00, /*          */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x00, /*          */\
\
    /* @216 ';' (8 pixels wide) */\
    0x00, /*          */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x00, /*          */\
    0x00, /*          */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x06, /*  ##      */\
\
    /* @224 '<' (8 pixels wide) */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x06, /*  ##      */\
    0x03, /* ##       */\
    0x06, /*  ##      */\
    0x0C, /*   ##     */\
    0x18, /*    ##    */\
    0x00, /*          */\
\
    /* @232 '=' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x3F, /* ######   */\
    0x00, /*          */\
    0x00, /*          */\
    0x3F, /* ######   */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @240 '>' (8 pixels wide) */\
    0x06, /*  ##      */\
    0x0C, /*   ##     */\
    0x18, /*    ##    */\
    0x30, /*     ##   */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x06, /*  ##      */\
    0x00, /*          */\
\
    /* @248 '?' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x30, /*     ##   */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x00, /*          */\
    0x0C, /*   ##     */\
    0x00, /*          */\
\
    /* @256 '@' (8 pixels wide) */\
    0x3E, /*  #####   */\
    0x63, /* ##   ##  */\
    0x7B, /* ## ####  */\
    0x7B, /* ## ####  */\
    0x7B, /* ## ####  */\
    0x03, /* ##       */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @264 'A' (8 pixels wide) */\
    0x0C, /*   ##     */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x3F, /* ######   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x00, /*          */\
\
    /* @272 'B' (8 pixels wide) */\
    0x3F, /* ######   */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x3E, /*  #####   */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x3F, /* ######   */\
    0x00, /*          */\
\
    /* @280 'C' (8 pixels wide) */\
    0x3C, /*   ####   */\
    0x66, /*  ##  ##  */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x66, /*  ##  ##  */\
    0x3C, /*   ####   */\
    0x00, /*          */\
\
    /* @288 'D' (8 pixels wide) */\
    0x1F, /* #####    */\
    0x36, /*  ## ##   */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x36, /*  ## ##   */\
    0x1F, /* #####    */\
    0x00, /*          */\
\
    /* @296 'E' (8 pixels wide) */\
    0x7F, /* #######  */\
    0x46, /*  ##   #  */\
    0x16, /*  ## #    */\
    0x1E, /*  ####    */\
    0x16, /*  ## #    */\
    0x46, /*  ##   #  */\
    0x7F, /* #######  */\
    0x00, /*          */\
\
    /* @304 'F' (8 pixels wide) */\
    0x7F, /* #######  */\
    0x46, /*  ##   #  */\
    0x16, /*  ## #    */\
    0x1E, /*  ####    */\
    0x16, /*  ## #    */\
    0x06, /*  ##      */\
    0x0F, /* ####     */\
    0x00, /*          */\
\
    /* @312 'G' (8 pixels wide) */\
    0x3C, /*   ####   */\
    0x66, /*  ##  ##  */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x73, /* ##  ###  */\
    0x66, /*  ##  ##  */\
    0x7C, /*   #####  */\
    0x00, /*          */\
\
    /* @320 'H' (8 pixels wide) */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x3F, /* ######   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x00, /*          */\
\
    /* @328 'I' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @336 'J' (8 pixels wide) */\
    0x78, /*    ####  */\
    0x30, /*     ##   */\
    0x30, /*     ##   */\
    0x30, /*     ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @344 'K' (8 pixels wide) */\
    0x67, /* ###  ##  */\
    0x66, /*  ##  ##  */\
    0x36, /*  ## ##   */\
    0x1E, /*  ####    */\
    0x36, /*  ## ##   */\
    0x66, /*  ##  ##  */\
    0x67, /* ###  ##  */\
    0x00, /*          */\
\
    /* @352 'L' (8 pixels wide) */\
    0x0F, /* ####     */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x46, /*  ##   #  */\
    0x66, /*  ##  ##  */\
    0x7F, /* #######  */\
    0x00, /*          */\
\
    /* @360 'M' (8 pixels wide) */\
    0x63, /* ##   ##  */\
    0x77, /* ### ###  */\
    0x7F, /* #######  */\
    0x7F, /* #######  */\
    0x6B, /* ## # ##  */\
    0x63, /* ##   ##  */\
    0x63, /* ##   ##  */\
    0x00, /*          */\
\
    /* @368 'N' (8 pixels wide) */\
    0x63, /* ##   ##  */\
    0x67, /* ###  ##  */\
    0x6F, /* #### ##  */\
    0x7B, /* ## ####  */\
    0x73, /* ##  ###  */\
    0x63, /* ##   ##  */\
    0x63, /* ##   ##  */\
    0x00, /*          */\
\
    /* @376 'O' (8 pixels wide) */\
    0x1C, /*   ###    */\
    0x36, /*  ## ##   */\
    0x63, /* ##   ##  */\
    0x63, /* ##   ##  */\
    0x63, /* ##   ##  */\
    0x36, /*  ## ##   */\
    0x1C, /*   ###    */\
    0x00, /*          */\
\
    /* @384 'P' (8 pixels wide) */\
    0x3F, /* ######   */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x3E, /*  #####   */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x0F, /* ####     */\
    0x00, /*          */\
\
    /* @392 'Q' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x3B, /* ## ###   */\
    0x1E, /*  ####    */\
    0x38, /*    ###   */\
    0x00, /*          */\
\
    /* @400 'R' (8 pixels wide) */\
    0x3F, /* ######   */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x3E, /*  #####   */\
    0x36, /*  ## ##   */\
    0x66, /*  ##  ##  */\
    0x67, /* ###  ##  */\
    0x00, /*          */\
\
    /* @408 'S' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x07, /* ###      */\
    0x0E, /*  ###     */\
    0x38, /*    ###   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @416 'T' (8 pixels wide) */\
    0x3F, /* ######   */\
    0x2D, /* # ## #   */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @424 'U' (8 pixels wide) */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x3F, /* ######   */\
    0x00, /*          */\
\
    /* @432 'V' (8 pixels wide) */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x0C, /*   ##     */\
    0x00, /*          */\
\
    /* @440 'W' (8 pixels wide) */\
    0x63, /* ##   ##  */\
    0x63, /* ##   ##  */\
    0x63, /* ##   ##  */\
    0x6B, /* ## # ##  */\
    0x7F, /* #######  */\
    0x77, /* ### ###  */\
    0x63, /* ##   ##  */\
    0x00, /*          */\
\
    /* @448 'X' (8 pixels wide) */\
    0x63, /* ##   ##  */\
    0x63, /* ##   ##  */\
    0x36, /*  ## ##   */\
    0x1C, /*   ###    */\
    0x1C, /*   ###    */\
    0x36, /*  ## ##   */\
    0x63, /* ##   ##  */\
    0x00, /*          */\
\
    /* @456 'Y' (8 pixels wide) */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @464 'Z' (8 pixels wide) */\
    0x7F, /* #######  */\
    0x63, /* ##   ##  */\
    0x31, /* #   ##   */\
    0x18, /*    ##    */\
    0x4C, /*   ##  #  */\
    0x66, /*  ##  ##  */\
    0x7F, /* #######  */\
    0x00, /*          */\
\
    /* @472 '[' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @480 '\' (8 pixels wide) */\
    0x03, /* ##       */\
    0x06, /*  ##      */\
    0x0C, /*   ##     */\
    0x18, /*    ##    */\
    0x30, /*     ##   */\
    0x60, /*      ##  */\
    0x40, /*       #  */\
    0x00, /*          */\
\
    /* @488 ']' (8 pixels wide) */\
    0x1E, /*  ####    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @496 '^' (8 pixels wide) */\
    0x08, /*    #     */\
    0x1C, /*   ###    */\
    0x36, /*  ## ##   */\
    0x63, /* ##   ##  */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @504 '_' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xFF, /* ######## */\
\
    /* @512 '`' (8 pixels wide) */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x18, /*    ##    */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @520 'a' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x1E, /*  ####    */\
    0x30, /*     ##   */\
    0x3E, /*  #####   */\
    0x33, /* ##  ##   */\
    0x6E, /*  ### ##  */\
    0x00, /*          */\
\
    /* @528 'b' (8 pixels wide) */\
    0x07, /* ###      */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x3E, /*  #####   */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x3B, /* ## ###   */\
    0x00, /*          */\
\
    /* @536 'c' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x03, /* ##       */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @544 'd' (8 pixels wide) */\
    0x38, /*    ###   */\
    0x30, /*     ##   */\
    0x30, /*     ##   */\
    0x3E, /*  #####   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x6E, /*  ### ##  */\
    0x00, /*          */\
\
    /* @552 'e' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x3F, /* ######   */\
    0x03, /* ##       */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @560 'f' (8 pixels wide) */\
    0x1C, /*   ###    */\
    0x36, /*  ## ##   */\
    0x06, /*  ##      */\
    0x0F, /* ####     */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x0F, /* ####     */\
    0x00, /*          */\
\
    /* @568 'g' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x6E, /*  ### ##  */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x3E, /*  #####   */\
    0x30, /*     ##   */\
    0x1F, /* #####    */\
\
    /* @576 'h' (8 pixels wide) */\
    0x07, /* ###      */\
    0x06, /*  ##      */\
    0x36, /*  ## ##   */\
    0x6E, /*  ### ##  */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x67, /* ###  ##  */\
    0x00, /*          */\
\
    /* @584 'i' (8 pixels wide) */\
    0x0C, /*   ##     */\
    0x00, /*          */\
    0x0E, /*  ###     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @592 'j' (8 pixels wide) */\
    0x30, /*     ##   */\
    0x00, /*          */\
    0x30, /*     ##   */\
    0x30, /*     ##   */\
    0x30, /*     ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
\
    /* @600 'k' (8 pixels wide) */\
    0x07, /* ###      */\
    0x06, /*  ##      */\
    0x66, /*  ##  ##  */\
    0x36, /*  ## ##   */\
    0x1E, /*  ####    */\
    0x36, /*  ## ##   */\
    0x67, /* ###  ##  */\
    0x00, /*          */\
\
    /* @608 'l' (8 pixels wide) */\
    0x0E, /*  ###     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @616 'm' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x33, /* ##  ##   */\
    0x7F, /* #######  */\
    0x7F, /* #######  */\
    0x6B, /* ## # ##  */\
    0x63, /* ##   ##  */\
    0x00, /*          */\
\
    /* @624 'n' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x1F, /* #####    */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x00, /*          */\
\
    /* @632 'o' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x1E, /*  ####    */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x00, /*          */\
\
    /* @640 'p' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x3B, /* ## ###   */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x3E, /*  #####   */\
    0x06, /*  ##      */\
    0x0F, /* ####     */\
\
    /* @648 'q' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x6E, /*  ### ##  */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x3E, /*  #####   */\
    0x30, /*     ##   */\
    0x78, /*    ####  */\
\
    /* @656 'r' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x3B, /* ## ###   */\
    0x6E, /*  ### ##  */\
    0x66, /*  ##  ##  */\
    0x06, /*  ##      */\
    0x0F, /* ####     */\
    0x00, /*          */\
\
    /* @664 's' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x3E, /*  #####   */\
    0x03, /* ##       */\
    0x1E, /*  ####    */\
    0x30, /*     ##   */\
    0x1F, /* #####    */\
    0x00, /*          */\
\
    /* @672 't' (8 pixels wide) */\
    0x08, /*    #     */\
    0x0C, /*   ##     */\
    0x3E, /*  #####   */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x2C, /*   ## #   */\
    0x18, /*    ##    */\
    0x00, /*          */\
\
    /* @680 'u' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x6E, /*  ### ##  */\
    0x00, /*          */\
\
    /* @688 'v' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x1E, /*  ####    */\
    0x0C, /*   ##     */\
    0x00, /*          */\
\
    /* @696 'w' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x63, /* ##   ##  */\
    0x6B, /* ## # ##  */\
    0x7F, /* #######  */\
    0x7F, /* #######  */\
    0x36, /*  ## ##   */\
    0x00, /*          */\
\
    /* @704 'x' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x63, /* ##   ##  */\
    0x36, /*  ## ##   */\
    0x1C, /*   ###    */\
    0x36, /*  ## ##   */\
    0x63, /* ##   ##  */\
    0x00, /*          */\
\
    /* @712 'y' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x33, /* ##  ##   */\
    0x3E, /*  #####   */\
    0x30, /*     ##   */\
    0x1F, /* #####    */\
\
    /* @720 'z' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x3F, /* ######   */\
    0x19, /* #  ##    */\
    0x0C, /*   ##     */\
    0x26, /*  ##  #   */\
    0x3F, /* ######   */\
    0x00, /*          */\
\
    /* @728 '{' (8 pixels wide) */\
    0x38, /*    ###   */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x07, /* ###      */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x38, /*    ###   */\
    0x00, /*          */\
\
    /* @736 '|' (8 pixels wide) */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x00, /*          */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x00, /*          */\
\
    /* @744 '}' (8 pixels wide) */\
    0x07, /* ###      */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x38, /*    ###   */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x07, /* ###      */\
    0x00, /*          */\
\
    /* @752 '~' (8 pixels wide) */\
    0x6E, /*  ### ##  */\
    0x3B, /* ## ###   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
}


/** @brief Initialization macro for the generic_8pt font information. */
#define FONTS_GENERIC_8PT_INFO \
{\
    8, /* Character width (-1 means dynamic width). */\
    8, /* Character height (-1 means dynamic height). */\
    8, /* Bitmap offset (-1 means dynamic offset). */\
    0, /* Deescriptor entry size. */\
    ' ', /* Start character. */\
    '~', /* End character. */\
    fonts_generic_8pt_descriptors, /*  Character descriptor array. */\
    fonts_generic_8pt_bitmaps, /* Character bitmap array. */\
}


/** @brief Initialization macro for the descriptors of the digital7_limited_b20pt font. */
#define FONTS_DIGITAL7_LIMITED_B20PT_DESCRIPTORS \
{\
    0 /* No descriptor needed for this font. */\
}


/** @brief Initialization macro for the bitmaps of the digital7_limited_b20pt font. */
#define FONTS_DIGITAL7_LIMITED_B20PT_BITMAPS \
{\
\
    /* @0 ' ' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @48 '!' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xC0, 0x00, /*       ##       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @96 '"' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xB8, 0x03, /*    ### ###     */\
    0xB8, 0x03, /*    ### ###     */\
    0xB8, 0x03, /*    ### ###     */\
    0xB8, 0x03, /*    ### ###     */\
    0xB8, 0x03, /*    ### ###     */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @144 '#' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x70, 0x07, /*     ### ###    */\
    0x70, 0x07, /*     ### ###    */\
    0x70, 0x07, /*     ### ###    */\
    0xDC, 0x1D, /*   ### ### ###  */\
    0xDC, 0x1D, /*   ### ### ###  */\
    0x70, 0x07, /*     ### ###    */\
    0x70, 0x07, /*     ### ###    */\
    0xFC, 0x1F, /*   ###########  */\
    0xFC, 0x1F, /*   ###########  */\
    0x70, 0x07, /*     ### ###    */\
    0x70, 0x07, /*     ### ###    */\
    0x70, 0x07, /*     ### ###    */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @192 '$' (14 pixels wide) */\
    0xC0, 0x01, /*       ###      */\
    0xC0, 0x01, /*       ###      */\
    0x00, 0x00, /*                */\
    0xFC, 0x0F, /*   ##########   */\
    0xFC, 0x0F, /*   ##########   */\
    0x0E, 0x00, /*  ###           */\
    0xCE, 0x01, /*  ###  ###      */\
    0xCE, 0x01, /*  ###  ###      */\
    0xCE, 0x01, /*  ###  ###      */\
    0xCE, 0x01, /*  ###  ###      */\
    0x0E, 0x00, /*  ###           */\
    0xFE, 0x07, /*  ##########    */\
    0xF8, 0x07, /*    ########    */\
    0x00, 0x18, /*            ##  */\
    0xC0, 0x1D, /*       ### ###  */\
    0xC0, 0x1D, /*       ### ###  */\
    0xC0, 0x1D, /*       ### ###  */\
    0xC0, 0x1D, /*       ### ###  */\
    0x00, 0x1C, /*           ###  */\
    0xFC, 0x1F, /*   ###########  */\
    0xFE, 0x1F, /*  ############  */\
    0x00, 0x00, /*                */\
    0xC0, 0x01, /*       ###      */\
    0xC0, 0x01, /*       ###      */\
\
    /* @240 '%' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x7F, 0x1C, /* #######   ###  */\
    0x7E, 0x1E, /*  ######  ####  */\
    0x77, 0x0E, /* ### ###  ###   */\
    0x77, 0x0F, /* ### ### ####   */\
    0x3F, 0x07, /* ######  ###    */\
    0xFF, 0x07, /* ###########    */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x01, /*        ##      */\
    0x60, 0x00, /*      ##        */\
    0x70, 0x00, /*     ###        */\
    0x70, 0x00, /*     ###        */\
    0xF8, 0x3F, /*    ########### */\
    0x38, 0x3F, /*    ###  ###### */\
    0xBC, 0x3B, /*   #### ### ### */\
    0x9C, 0x3B, /*   ###  ### ### */\
    0x9E, 0x1F, /*  ####  ######  */\
    0x8E, 0x3F, /*  ###   ####### */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @288 '&' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xF8, 0x0F, /*    #########   */\
    0xF8, 0x0F, /*    #########   */\
    0x1C, 0x0E, /*   ###    ###   */\
    0x1C, 0x0E, /*   ###    ###   */\
    0x1C, 0x0E, /*   ###    ###   */\
    0x1C, 0x0E, /*   ###    ###   */\
    0x0C, 0x0C, /*   ##      ##   */\
    0xF8, 0x07, /*    ########    */\
    0xFE, 0x1F, /*  ############  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x18, /*  ###       ##  */\
    0xFC, 0x0F, /*   ##########   */\
    0xFE, 0x1F, /*  ############  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x18, /*            ##  */\
    0x00, 0x00, /*                */\
\
    /* @336 ''' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @384 '(' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xE0, 0x03, /*      #####     */\
    0xE0, 0x03, /*      #####     */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0x18, 0x00, /*    ##          */\
    0x18, 0x00, /*    ##          */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0x38, 0x00, /*    ###         */\
    0xF0, 0x03, /*     ######     */\
    0xE0, 0x03, /*      #####     */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @432 ')' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xF0, 0x00, /*     ####       */\
    0xF0, 0x00, /*     ####       */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x00, 0x03, /*         ##     */\
    0x00, 0x03, /*         ##     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0xF0, 0x01, /*     #####      */\
    0xF0, 0x00, /*     ####       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @480 '*' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xE0, 0x00, /*      ###       */\
    0xFC, 0x07, /*   #########    */\
    0xFC, 0x07, /*   #########    */\
    0xF8, 0x03, /*    #######     */\
    0xF8, 0x03, /*    #######     */\
    0xFC, 0x07, /*   #########    */\
    0xFC, 0x07, /*   #########    */\
    0xE0, 0x00, /*      ###       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @528 '+' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xFC, 0x07, /*   #########    */\
    0xFC, 0x07, /*   #########    */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @576 ',' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
\
    /* @624 '-' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xF8, 0x07, /*    ########    */\
    0xF8, 0x07, /*    ########    */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @672 '.' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @720 '/' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x0E, /*          ###   */\
    0x00, 0x0E, /*          ###   */\
    0x00, 0x0F, /*         ####   */\
    0x00, 0x07, /*         ###    */\
    0x80, 0x07, /*        ####    */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x03, /*        ###     */\
    0x80, 0x01, /*        ##      */\
    0x80, 0x01, /*        ##      */\
    0x60, 0x00, /*      ##        */\
    0x70, 0x00, /*     ###        */\
    0x70, 0x00, /*     ###        */\
    0x78, 0x00, /*    ####        */\
    0x38, 0x00, /*    ###         */\
    0x3C, 0x00, /*   ####         */\
    0x1C, 0x00, /*   ###          */\
    0x1E, 0x00, /*  ####          */\
    0x0C, 0x00, /*   ##           */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @768 '0' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xFE, 0x1F, /*  ############  */\
    0xFC, 0x1F, /*   ###########  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x06, 0x18, /*  ##        ##  */\
    0x06, 0x18, /*  ##        ##  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0xFC, 0x1F, /*   ###########  */\
    0xFE, 0x1F, /*  ############  */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @816 '1' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xC0, 0x00, /*       ##       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0xC0, 0x00, /*       ##       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @864 '2' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xFE, 0x1F, /*  ############  */\
    0xFC, 0x1F, /*   ###########  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0xF8, 0x1F, /*    ##########  */\
    0xFE, 0x0F, /*  ###########   */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0xFC, 0x07, /*   #########    */\
    0xFE, 0x0F, /*  ###########   */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @912 '3' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xFC, 0x1F, /*   ###########  */\
    0xFC, 0x1F, /*   ###########  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0xF8, 0x1F, /*    ##########  */\
    0xF8, 0x1F, /*    ##########  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0xFC, 0x1F, /*   ###########  */\
    0xFC, 0x1F, /*   ###########  */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @960 '4' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x18, /*            ##  */\
    0x06, 0x1C, /*  ##       ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0xFE, 0x1F, /*  ############  */\
    0xFC, 0x1F, /*   ###########  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x18, /*            ##  */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @1008 '5' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xFE, 0x0F, /*  ###########   */\
    0xFC, 0x07, /*   #########    */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0xFE, 0x07, /*  ##########    */\
    0xFC, 0x1F, /*   ###########  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0xFC, 0x1F, /*   ###########  */\
    0xFE, 0x1F, /*  ############  */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @1056 '6' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xFE, 0x0F, /*  ###########   */\
    0xFC, 0x07, /*   #########    */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0x0E, 0x00, /*  ###           */\
    0xFE, 0x07, /*  ##########    */\
    0xFE, 0x1F, /*  ############  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0xFC, 0x1F, /*   ###########  */\
    0xFE, 0x1F, /*  ############  */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @1104 '7' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xFE, 0x1F, /*  ############  */\
    0xFC, 0x1F, /*   ###########  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x06, 0x18, /*  ##        ##  */\
    0x00, 0x18, /*            ##  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x18, /*            ##  */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @1152 '8' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xFE, 0x1F, /*  ############  */\
    0xFC, 0x1F, /*   ###########  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0xFE, 0x1F, /*  ############  */\
    0xFE, 0x1F, /*  ############  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0xFC, 0x1F, /*   ###########  */\
    0xFE, 0x1F, /*  ############  */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @1200 '9' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xFE, 0x1F, /*  ############  */\
    0xFC, 0x1F, /*   ###########  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0x0E, 0x1C, /*  ###      ###  */\
    0xFE, 0x1F, /*  ############  */\
    0xFC, 0x1F, /*   ###########  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0x00, 0x1C, /*           ###  */\
    0xFC, 0x1F, /*   ###########  */\
    0xFE, 0x1F, /*  ############  */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @1248 ':' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0xE0, 0x00, /*      ###       */\
    0xE0, 0x00, /*      ###       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
}


/** @brief Initialization macro for the digital7_limited_b20pt font information. */
#define FONTS_DIGITAL7_LIMITED_B20PT_INFO \
{\
    14, /* Character width (-1 means dynamic width). */\
    24, /* Character height (-1 means dynamic height). */\
    48, /* Bitmap offset (-1 means dynamic offset). */\
    0, /* Deescriptor entry size. */\
    ' ', /* Start character. */\
    ':', /* End character. */\
    fonts_digital7_limited_b20pt_descriptors, /*  Character descriptor array. */\
    fonts_digital7_limited_b20pt_bitmaps, /* Character bitmap array. */\
}


/** @brief Initialization macro for the descriptors of the arial_16pt font. */
#define FONTS_ARIAL_16PT_DESCRIPTORS \
{\
         8, (   0 & 0xFF)       , (   0 >> 8) & 0xFF,  /*   */\
         2, (  21 & 0xFF)       , (  21 >> 8) & 0xFF,  /* ! */\
         5, (  42 & 0xFF)       , (  42 >> 8) & 0xFF,  /* " */\
        11, (  63 & 0xFF)       , (  63 >> 8) & 0xFF,  /* # */\
         8, ( 105 & 0xFF)       , ( 105 >> 8) & 0xFF,  /* $ */\
        17, ( 126 & 0xFF)       , ( 126 >> 8) & 0xFF,  /* % */\
        10, ( 189 & 0xFF)       , ( 189 >> 8) & 0xFF,  /* & */\
         2, ( 231 & 0xFF)       , ( 231 >> 8) & 0xFF,  /* ' */\
         5, ( 252 & 0xFF)       , ( 252 >> 8) & 0xFF,  /* ( */\
         5, ( 273 & 0xFF)       , ( 273 >> 8) & 0xFF,  /* ) */\
         6, ( 294 & 0xFF)       , ( 294 >> 8) & 0xFF,  /* * */\
        10, ( 315 & 0xFF)       , ( 315 >> 8) & 0xFF,  /* + */\
         2, ( 357 & 0xFF)       , ( 357 >> 8) & 0xFF,  /* , */\
         5, ( 378 & 0xFF)       , ( 378 >> 8) & 0xFF,  /* - */\
         2, ( 399 & 0xFF)       , ( 399 >> 8) & 0xFF,  /* . */\
         6, ( 420 & 0xFF)       , ( 420 >> 8) & 0xFF,  /* / */\
         8, ( 441 & 0xFF)       , ( 441 >> 8) & 0xFF,  /* 0 */\
         5, ( 462 & 0xFF)       , ( 462 >> 8) & 0xFF,  /* 1 */\
         8, ( 483 & 0xFF)       , ( 483 >> 8) & 0xFF,  /* 2 */\
         8, ( 504 & 0xFF)       , ( 504 >> 8) & 0xFF,  /* 3 */\
         9, ( 525 & 0xFF)       , ( 525 >> 8) & 0xFF,  /* 4 */\
         8, ( 567 & 0xFF)       , ( 567 >> 8) & 0xFF,  /* 5 */\
         8, ( 588 & 0xFF)       , ( 588 >> 8) & 0xFF,  /* 6 */\
         8, ( 609 & 0xFF)       , ( 609 >> 8) & 0xFF,  /* 7 */\
         8, ( 630 & 0xFF)       , ( 630 >> 8) & 0xFF,  /* 8 */\
         8, ( 651 & 0xFF)       , ( 651 >> 8) & 0xFF,  /* 9 */\
         2, ( 672 & 0xFF)       , ( 672 >> 8) & 0xFF,  /* : */\
         2, ( 693 & 0xFF)       , ( 693 >> 8) & 0xFF,  /* ; */\
         8, ( 714 & 0xFF)       , ( 714 >> 8) & 0xFF,  /* < */\
         8, ( 735 & 0xFF)       , ( 735 >> 8) & 0xFF,  /* = */\
         8, ( 756 & 0xFF)       , ( 756 >> 8) & 0xFF,  /* > */\
         8, ( 777 & 0xFF)       , ( 777 >> 8) & 0xFF,  /* ? */\
        16, ( 798 & 0xFF)       , ( 798 >> 8) & 0xFF,  /* @ */\
        11, ( 840 & 0xFF)       , ( 840 >> 8) & 0xFF,  /* A */\
         9, ( 882 & 0xFF)       , ( 882 >> 8) & 0xFF,  /* B */\
        10, ( 924 & 0xFF)       , ( 924 >> 8) & 0xFF,  /* C */\
        10, ( 966 & 0xFF)       , ( 966 >> 8) & 0xFF,  /* D */\
         9, (1008 & 0xFF)       , (1008 >> 8) & 0xFF,  /* E */\
         9, (1050 & 0xFF)       , (1050 >> 8) & 0xFF,  /* F */\
        11, (1092 & 0xFF)       , (1092 >> 8) & 0xFF,  /* G */\
        10, (1134 & 0xFF)       , (1134 >> 8) & 0xFF,  /* H */\
         2, (1176 & 0xFF)       , (1176 >> 8) & 0xFF,  /* I */\
         7, (1197 & 0xFF)       , (1197 >> 8) & 0xFF,  /* J */\
        10, (1218 & 0xFF)       , (1218 >> 8) & 0xFF,  /* K */\
         8, (1260 & 0xFF)       , (1260 >> 8) & 0xFF,  /* L */\
        13, (1281 & 0xFF)       , (1281 >> 8) & 0xFF,  /* M */\
        10, (1323 & 0xFF)       , (1323 >> 8) & 0xFF,  /* N */\
        11, (1365 & 0xFF)       , (1365 >> 8) & 0xFF,  /* O */\
         9, (1407 & 0xFF)       , (1407 >> 8) & 0xFF,  /* P */\
        11, (1449 & 0xFF)       , (1449 >> 8) & 0xFF,  /* Q */\
        11, (1491 & 0xFF)       , (1491 >> 8) & 0xFF,  /* R */\
         9, (1533 & 0xFF)       , (1533 >> 8) & 0xFF,  /* S */\
        10, (1575 & 0xFF)       , (1575 >> 8) & 0xFF,  /* T */\
        10, (1617 & 0xFF)       , (1617 >> 8) & 0xFF,  /* U */\
        11, (1659 & 0xFF)       , (1659 >> 8) & 0xFF,  /* V */\
        15, (1701 & 0xFF)       , (1701 >> 8) & 0xFF,  /* W */\
        11, (1743 & 0xFF)       , (1743 >> 8) & 0xFF,  /* X */\
        12, (1785 & 0xFF)       , (1785 >> 8) & 0xFF,  /* Y */\
        10, (1827 & 0xFF)       , (1827 >> 8) & 0xFF,  /* Z */\
         4, (1869 & 0xFF)       , (1869 >> 8) & 0xFF,  /* [ */\
         5, (1890 & 0xFF)       , (1890 >> 8) & 0xFF,  /* \ */\
         4, (1911 & 0xFF)       , (1911 >> 8) & 0xFF,  /* ] */\
         7, (1932 & 0xFF)       , (1932 >> 8) & 0xFF,  /* ^ */\
        10, (1953 & 0xFF)       , (1953 >> 8) & 0xFF,  /* _ */\
         3, (1995 & 0xFF)       , (1995 >> 8) & 0xFF,  /* ` */\
         8, (2016 & 0xFF)       , (2016 >> 8) & 0xFF,  /* a */\
         8, (2037 & 0xFF)       , (2037 >> 8) & 0xFF,  /* b */\
         7, (2058 & 0xFF)       , (2058 >> 8) & 0xFF,  /* c */\
         8, (2079 & 0xFF)       , (2079 >> 8) & 0xFF,  /* d */\
         8, (2100 & 0xFF)       , (2100 >> 8) & 0xFF,  /* e */\
         6, (2121 & 0xFF)       , (2121 >> 8) & 0xFF,  /* f */\
         8, (2142 & 0xFF)       , (2142 >> 8) & 0xFF,  /* g */\
         8, (2163 & 0xFF)       , (2163 >> 8) & 0xFF,  /* h */\
         2, (2184 & 0xFF)       , (2184 >> 8) & 0xFF,  /* i */\
         4, (2205 & 0xFF)       , (2205 >> 8) & 0xFF,  /* j */\
         7, (2226 & 0xFF)       , (2226 >> 8) & 0xFF,  /* k */\
         2, (2247 & 0xFF)       , (2247 >> 8) & 0xFF,  /* l */\
        12, (2268 & 0xFF)       , (2268 >> 8) & 0xFF,  /* m */\
         8, (2310 & 0xFF)       , (2310 >> 8) & 0xFF,  /* n */\
         8, (2331 & 0xFF)       , (2331 >> 8) & 0xFF,  /* o */\
         8, (2352 & 0xFF)       , (2352 >> 8) & 0xFF,  /* p */\
         8, (2373 & 0xFF)       , (2373 >> 8) & 0xFF,  /* q */\
         5, (2394 & 0xFF)       , (2394 >> 8) & 0xFF,  /* r */\
         7, (2415 & 0xFF)       , (2415 >> 8) & 0xFF,  /* s */\
         5, (2436 & 0xFF)       , (2436 >> 8) & 0xFF,  /* t */\
         8, (2457 & 0xFF)       , (2457 >> 8) & 0xFF,  /* u */\
         9, (2478 & 0xFF)       , (2478 >> 8) & 0xFF,  /* v */\
        13, (2520 & 0xFF)       , (2520 >> 8) & 0xFF,  /* w */\
         8, (2562 & 0xFF)       , (2562 >> 8) & 0xFF,  /* x */\
         9, (2583 & 0xFF)       , (2583 >> 8) & 0xFF,  /* y */\
         8, (2625 & 0xFF)       , (2625 >> 8) & 0xFF,  /* z */\
         6, (2646 & 0xFF)       , (2646 >> 8) & 0xFF,  /* { */\
         2, (2667 & 0xFF)       , (2667 >> 8) & 0xFF,  /* | */\
         6, (2688 & 0xFF)       , (2688 >> 8) & 0xFF,  /* } */\
         8, (2709 & 0xFF)       , (2709 >> 8) & 0xFF,  /* ~ */\
}


/** @brief Initialization macro for the bitmaps of the arial_16pt font. */
#define FONTS_ARIAL_16PT_BITMAPS \
{\
\
    /* @0 ' ' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @21 '!' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @42 '"' (5 pixels wide) */\
    0x00, /*       */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @63 '#' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x30, 0x06, /*     ##   ## */\
    0x30, 0x06, /*     ##   ## */\
    0x30, 0x06, /*     ##   ## */\
    0x18, 0x03, /*    ##   ##  */\
    0xFF, 0x07, /* ########### */\
    0xFF, 0x07, /* ########### */\
    0x18, 0x03, /*    ##   ##  */\
    0x9C, 0x03, /*   ###  ###  */\
    0x8C, 0x01, /*   ##   ##   */\
    0xFF, 0x07, /* ########### */\
    0xFF, 0x07, /* ########### */\
    0x8C, 0x01, /*   ##   ##   */\
    0xCE, 0x01, /*  ###  ###   */\
    0xC6, 0x00, /*  ##   ##    */\
    0xC6, 0x00, /*  ##   ##    */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @105 '$' (8 pixels wide) */\
    0x10, /*     #    */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0xD7, /* ### # ## */\
    0xD3, /* ##  # ## */\
    0x13, /* ##  #    */\
    0x17, /* ### #    */\
    0x1E, /*  ####    */\
    0x7C, /*   #####  */\
    0xF0, /*     #### */\
    0xD0, /*     # ## */\
    0xD0, /*     # ## */\
    0xD3, /* ##  # ## */\
    0xD7, /* ### # ## */\
    0x7E, /*  ######  */\
    0x3C, /*   ####   */\
    0x10, /*     #    */\
    0x10, /*     #    */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @126 '%' (17 pixels wide) */\
    0x00, 0x00, 0x00, /*                   */\
    0x1C, 0x30, 0x00, /*   ###       ##    */\
    0x3E, 0x18, 0x00, /*  #####     ##     */\
    0x63, 0x18, 0x00, /* ##   ##    ##     */\
    0x63, 0x0C, 0x00, /* ##   ##   ##      */\
    0x63, 0x0C, 0x00, /* ##   ##   ##      */\
    0x63, 0x06, 0x00, /* ##   ##  ##       */\
    0x3E, 0x03, 0x00, /*  #####  ##        */\
    0x1C, 0x73, 0x00, /*   ###   ##  ###   */\
    0x80, 0xF9, 0x00, /*        ##  #####  */\
    0x80, 0x8D, 0x01, /*        ## ##   ## */\
    0xC0, 0x8C, 0x01, /*       ##  ##   ## */\
    0xC0, 0x8C, 0x01, /*       ##  ##   ## */\
    0x60, 0x8C, 0x01, /*      ##   ##   ## */\
    0x30, 0xF8, 0x00, /*     ##     #####  */\
    0x30, 0x70, 0x00, /*     ##      ###   */\
    0x00, 0x00, 0x00, /*                   */\
    0x00, 0x00, 0x00, /*                   */\
    0x00, 0x00, 0x00, /*                   */\
    0x00, 0x00, 0x00, /*                   */\
    0x00, 0x00, 0x00, /*                   */\
\
    /* @189 '&' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xF8, 0x00, /*    #####   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0xCC, 0x00, /*   ##  ##   */\
    0x78, 0x00, /*    ####    */\
    0x38, 0x00, /*    ###     */\
    0x3C, 0x00, /*   ####     */\
    0xE6, 0x01, /*  ##  ####  */\
    0xC3, 0x01, /* ##    ###  */\
    0xC3, 0x00, /* ##    ##   */\
    0x83, 0x01, /* ##     ##  */\
    0xC7, 0x01, /* ###   ###  */\
    0x7E, 0x03, /*  ###### ## */\
    0x3C, 0x00, /*   ####     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @231 ''' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @252 '(' (5 pixels wide) */\
    0x00, /*       */\
    0x18, /*    ## */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x18, /*    ## */\
    0x00, /*       */\
\
    /* @273 ')' (5 pixels wide) */\
    0x00, /*       */\
    0x03, /* ##    */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x03, /* ##    */\
    0x00, /*       */\
\
    /* @294 '*' (6 pixels wide) */\
    0x00, /*        */\
    0x0C, /*   ##   */\
    0x2D, /* # ## # */\
    0x3F, /* ###### */\
    0x0C, /*   ##   */\
    0x1E, /*  ####  */\
    0x12, /*  #  #  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @315 '+' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @357 ',' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @378 '-' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @399 '.' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @420 '/' (6 pixels wide) */\
    0x00, /*        */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x03, /* ##     */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @441 '0' (8 pixels wide) */\
    0x00, /*          */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0x66, /*  ##  ##  */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0x66, /*  ##  ##  */\
    0x7E, /*  ######  */\
    0x3C, /*   ####   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @462 '1' (5 pixels wide) */\
    0x00, /*       */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x1C, /*   ### */\
    0x1E, /*  #### */\
    0x1B, /* ## ## */\
    0x19, /* #  ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @483 '2' (8 pixels wide) */\
    0x00, /*          */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0x60, /*      ##  */\
    0x60, /*      ##  */\
    0x30, /*     ##   */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x06, /*  ##      */\
    0x02, /*  #       */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @504 '3' (8 pixels wide) */\
    0x00, /*          */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC0, /*       ## */\
    0x60, /*      ##  */\
    0x38, /*    ###   */\
    0x78, /*    ####  */\
    0xE0, /*      ### */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC3, /* ##    ## */\
    0xE7, /* ###  ### */\
    0x7E, /*  ######  */\
    0x3C, /*   ####   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @525 '4' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x70, 0x00, /*     ###   */\
    0x70, 0x00, /*     ###   */\
    0x78, 0x00, /*    ####   */\
    0x78, 0x00, /*    ####   */\
    0x6C, 0x00, /*   ## ##   */\
    0x6C, 0x00, /*   ## ##   */\
    0x66, 0x00, /*  ##  ##   */\
    0x66, 0x00, /*  ##  ##   */\
    0x63, 0x00, /* ##   ##   */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x60, 0x00, /*      ##   */\
    0x60, 0x00, /*      ##   */\
    0x60, 0x00, /*      ##   */\
    0x60, 0x00, /*      ##   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @567 '5' (8 pixels wide) */\
    0x00, /*          */\
    0x7E, /*  ######  */\
    0x7E, /*  ######  */\
    0x06, /*  ##      */\
    0x06, /*  ##      */\
    0x03, /* ##       */\
    0x3F, /* ######   */\
    0x7F, /* #######  */\
    0xE3, /* ##   ### */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC3, /* ##    ## */\
    0x67, /* ###  ##  */\
    0x7E, /*  ######  */\
    0x3C, /*   ####   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @588 '6' (8 pixels wide) */\
    0x00, /*          */\
    0x38, /*    ###   */\
    0x7E, /*  ######  */\
    0xE6, /*  ##  ### */\
    0xC3, /* ##    ## */\
    0x03, /* ##       */\
    0x3B, /* ## ###   */\
    0x7F, /* #######  */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE6, /*  ##  ### */\
    0x7E, /*  ######  */\
    0x3C, /*   ####   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @609 '7' (8 pixels wide) */\
    0x00, /*          */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0x40, /*       #  */\
    0x60, /*      ##  */\
    0x20, /*      #   */\
    0x30, /*     ##   */\
    0x30, /*     ##   */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x0C, /*   ##     */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @630 '8' (8 pixels wide) */\
    0x00, /*          */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE7, /* ###  ### */\
    0x7E, /*  ######  */\
    0x7E, /*  ######  */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE7, /* ###  ### */\
    0x7E, /*  ######  */\
    0x3C, /*   ####   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @651 '9' (8 pixels wide) */\
    0x00, /*          */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0x67, /* ###  ##  */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE7, /* ###  ### */\
    0xFE, /*  ####### */\
    0xDC, /*   ### ## */\
    0xC0, /*       ## */\
    0x43, /* ##    #  */\
    0x67, /* ###  ##  */\
    0x7E, /*  ######  */\
    0x1C, /*   ###    */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @672 ':' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @693 ';' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @714 '<' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x80, /*        # */\
    0xE0, /*      ### */\
    0x70, /*     ###  */\
    0x1C, /*   ###    */\
    0x0F, /* ####     */\
    0x03, /* ##       */\
    0x0F, /* ####     */\
    0x1C, /*   ###    */\
    0x70, /*     ###  */\
    0xE0, /*      ### */\
    0x80, /*        # */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @735 '=' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @756 '>' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x01, /* #        */\
    0x07, /* ###      */\
    0x0E, /*  ###     */\
    0x38, /*    ###   */\
    0xF0, /*     #### */\
    0xC0, /*       ## */\
    0xF0, /*     #### */\
    0x38, /*    ###   */\
    0x0E, /*  ###     */\
    0x07, /* ###      */\
    0x01, /* #        */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @777 '?' (8 pixels wide) */\
    0x00, /*          */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0x60, /*      ##  */\
    0x70, /*     ###  */\
    0x30, /*     ##   */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x00, /*          */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @798 '@' (16 pixels wide) */\
    0x00, 0x00, /*                  */\
    0xC0, 0x07, /*       #####      */\
    0xF0, 0x1F, /*     #########    */\
    0x38, 0x38, /*    ###     ###   */\
    0x1C, 0x70, /*   ###       ###  */\
    0x8E, 0x6D, /*  ###   ## ## ##  */\
    0xC6, 0xCF, /*  ##   ######  ## */\
    0x66, 0xCE, /*  ##  ##  ###  ## */\
    0x63, 0xCC, /* ##   ##   ##  ## */\
    0x33, 0xCC, /* ##  ##    ##  ## */\
    0x33, 0xCC, /* ##  ##    ##  ## */\
    0x33, 0xC6, /* ##  ##   ##   ## */\
    0x33, 0x66, /* ##  ##   ##  ##  */\
    0x73, 0x77, /* ##  ### ### ###  */\
    0xE3, 0x3F, /* ##   #########   */\
    0xE6, 0x1E, /*  ##  ### ####    */\
    0x06, 0xC0, /*  ##           ## */\
    0x0C, 0x60, /*   ##         ##  */\
    0x3C, 0x38, /*   ####     ###   */\
    0xF8, 0x1F, /*    ##########    */\
    0xE0, 0x0F, /*      #######     */\
\
    /* @840 'A' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0xDC, 0x01, /*   ### ###   */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0xFE, 0x03, /*  #########  */\
    0xFE, 0x03, /*  #########  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @882 'B' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x7F, 0x00, /* #######   */\
    0xFF, 0x00, /* ########  */\
    0xC3, 0x01, /* ##    ### */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0xC3, 0x00, /* ##    ##  */\
    0x7F, 0x00, /* #######   */\
    0xFF, 0x00, /* ########  */\
    0xC3, 0x01, /* ##    ### */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0xC3, 0x01, /* ##    ### */\
    0xFF, 0x00, /* ########  */\
    0x7F, 0x00, /* #######   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @924 'C' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFC, 0x01, /*   #######  */\
    0x8E, 0x01, /*  ###   ##  */\
    0x86, 0x03, /*  ##    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x86, 0x01, /*  ##    ##  */\
    0xCE, 0x01, /*  ###  ###  */\
    0xFC, 0x00, /*   ######   */\
    0x78, 0x00, /*    ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @966 'D' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x7F, 0x00, /* #######    */\
    0xFF, 0x00, /* ########   */\
    0xC3, 0x01, /* ##    ###  */\
    0x83, 0x01, /* ##     ##  */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x83, 0x01, /* ##     ##  */\
    0xC3, 0x01, /* ##    ###  */\
    0xFF, 0x00, /* ########   */\
    0x7F, 0x00, /* #######    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1008 'E' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @1050 'F' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0xFF, 0x01, /* ######### */\
    0xFF, 0x01, /* ######### */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0xFF, 0x00, /* ########  */\
    0xFF, 0x00, /* ########  */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @1092 'G' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0xF0, 0x00, /*     ####    */\
    0xFC, 0x03, /*   ########  */\
    0x0E, 0x03, /*  ###    ##  */\
    0x06, 0x06, /*  ##      ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0xC3, 0x07, /* ##    ##### */\
    0xC3, 0x07, /* ##    ##### */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x06, 0x06, /*  ##      ## */\
    0x0E, 0x07, /*  ###    ### */\
    0xFC, 0x03, /*   ########  */\
    0xF0, 0x00, /*     ####    */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1134 'H' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1176 'I' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @1197 'J' (7 pixels wide) */\
    0x00, /*         */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x63, /* ##   ## */\
    0x63, /* ##   ## */\
    0x67, /* ###  ## */\
    0x3E, /*  #####  */\
    0x1C, /*   ###   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @1218 'K' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x83, 0x01, /* ##     ##  */\
    0x83, 0x01, /* ##     ##  */\
    0xC3, 0x00, /* ##    ##   */\
    0x63, 0x00, /* ##   ##    */\
    0x33, 0x00, /* ##  ##     */\
    0x1B, 0x00, /* ## ##      */\
    0x1B, 0x00, /* ## ##      */\
    0x3F, 0x00, /* ######     */\
    0x37, 0x00, /* ### ##     */\
    0x63, 0x00, /* ##   ##    */\
    0x63, 0x00, /* ##   ##    */\
    0xC3, 0x00, /* ##    ##   */\
    0x83, 0x01, /* ##     ##  */\
    0x83, 0x01, /* ##     ##  */\
    0x03, 0x03, /* ##      ## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1260 'L' (8 pixels wide) */\
    0x00, /*          */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @1281 'M' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0x07, 0x1C, /* ###       ### */\
    0x07, 0x1C, /* ###       ### */\
    0x0F, 0x1C, /* ####      ### */\
    0x0F, 0x1E, /* ####     #### */\
    0x0F, 0x1E, /* ####     #### */\
    0x0F, 0x1E, /* ####     #### */\
    0x1B, 0x1B, /* ## ##   ## ## */\
    0x1B, 0x1B, /* ## ##   ## ## */\
    0x1B, 0x1B, /* ## ##   ## ## */\
    0xB3, 0x1B, /* ##  ## ### ## */\
    0xB3, 0x19, /* ##  ## ##  ## */\
    0xB3, 0x19, /* ##  ## ##  ## */\
    0xB3, 0x19, /* ##  ## ##  ## */\
    0xE3, 0x18, /* ##   ###   ## */\
    0xE3, 0x18, /* ##   ###   ## */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @1323 'N' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x03, 0x03, /* ##      ## */\
    0x07, 0x03, /* ###     ## */\
    0x07, 0x03, /* ###     ## */\
    0x0F, 0x03, /* ####    ## */\
    0x0F, 0x03, /* ####    ## */\
    0x1B, 0x03, /* ## ##   ## */\
    0x3B, 0x03, /* ## ###  ## */\
    0x33, 0x03, /* ##  ##  ## */\
    0x73, 0x03, /* ##  ### ## */\
    0x63, 0x03, /* ##   ## ## */\
    0xC3, 0x03, /* ##    #### */\
    0xC3, 0x03, /* ##    #### */\
    0x83, 0x03, /* ##     ### */\
    0x83, 0x03, /* ##     ### */\
    0x03, 0x03, /* ##      ## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1365 'O' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0xF8, 0x00, /*    #####    */\
    0xFC, 0x01, /*   #######   */\
    0x8E, 0x03, /*  ###   ###  */\
    0x06, 0x03, /*  ##     ##  */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x06, 0x03, /*  ##     ##  */\
    0x8E, 0x03, /*  ###   ###  */\
    0xFC, 0x01, /*   #######   */\
    0xF8, 0x00, /*    #####    */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1407 'P' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x7F, 0x00, /* #######   */\
    0xFF, 0x00, /* ########  */\
    0xC3, 0x01, /* ##    ### */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0xC3, 0x01, /* ##    ### */\
    0xFF, 0x00, /* ########  */\
    0x7F, 0x00, /* #######   */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @1449 'Q' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0xF8, 0x00, /*    #####    */\
    0xFC, 0x01, /*   #######   */\
    0x8E, 0x03, /*  ###   ###  */\
    0x06, 0x03, /*  ##     ##  */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x02, /* ##       #  */\
    0x46, 0x03, /*  ##   # ##  */\
    0x8E, 0x01, /*  ###   ##   */\
    0xFC, 0x03, /*   ########  */\
    0x78, 0x06, /*    ####  ## */\
    0x00, 0x04, /*           # */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1491 'R' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0xFF, 0x00, /* ########    */\
    0xFF, 0x01, /* #########   */\
    0x83, 0x03, /* ##     ###  */\
    0x03, 0x03, /* ##      ##  */\
    0x03, 0x03, /* ##      ##  */\
    0x83, 0x03, /* ##     ###  */\
    0xFF, 0x01, /* #########   */\
    0x3F, 0x00, /* ######      */\
    0x63, 0x00, /* ##   ##     */\
    0xC3, 0x00, /* ##    ##    */\
    0x83, 0x01, /* ##     ##   */\
    0x83, 0x01, /* ##     ##   */\
    0x03, 0x03, /* ##      ##  */\
    0x03, 0x03, /* ##      ##  */\
    0x03, 0x06, /* ##       ## */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1533 'S' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x7C, 0x00, /*   #####   */\
    0xFE, 0x00, /*  #######  */\
    0xC7, 0x01, /* ###   ### */\
    0x83, 0x01, /* ##     ## */\
    0x03, 0x00, /* ##        */\
    0x07, 0x00, /* ###       */\
    0x1E, 0x00, /*  ####     */\
    0x78, 0x00, /*    ####   */\
    0xE0, 0x00, /*      ###  */\
    0xC0, 0x01, /*       ### */\
    0x80, 0x01, /*        ## */\
    0x83, 0x01, /* ##     ## */\
    0xC7, 0x01, /* ###   ### */\
    0xFE, 0x00, /*  #######  */\
    0x7C, 0x00, /*   #####   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @1575 'T' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1617 'U' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x86, 0x01, /*  ##    ##  */\
    0xFE, 0x01, /*  ########  */\
    0x7C, 0x00, /*   #####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1659 'V' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1701 'W' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0xC3, 0x61, /* ##    ###    ## */\
    0xC3, 0x61, /* ##    ###    ## */\
    0x43, 0x61, /* ##    # #    ## */\
    0x66, 0x73, /*  ##  ## ##  ### */\
    0x66, 0x33, /*  ##  ## ##  ##  */\
    0x66, 0x33, /*  ##  ## ##  ##  */\
    0x66, 0x33, /*  ##  ## ##  ##  */\
    0x66, 0x33, /*  ##  ## ##  ##  */\
    0x6C, 0x1B, /*   ## ## ## ##   */\
    0x6C, 0x1B, /*   ## ## ## ##   */\
    0x6C, 0x1B, /*   ## ## ## ##   */\
    0x6C, 0x1B, /*   ## ## ## ##   */\
    0x2C, 0x1A, /*   ## #   # ##   */\
    0x38, 0x0E, /*    ###   ###    */\
    0x38, 0x0E, /*    ###   ###    */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
\
    /* @1743 'X' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x03, 0x06, /* ##       ## */\
    0x06, 0x03, /*  ##     ##  */\
    0x8E, 0x01, /*  ###   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8E, 0x03, /*  ###   ###  */\
    0x06, 0x03, /*  ##     ##  */\
    0x03, 0x06, /* ##       ## */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1785 'Y' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x03, 0x0C, /* ##        ## */\
    0x06, 0x06, /*  ##      ##  */\
    0x0E, 0x06, /*  ###     ##  */\
    0x0C, 0x03, /*   ##    ##   */\
    0x98, 0x01, /*    ##  ##    */\
    0x98, 0x01, /*    ##  ##    */\
    0xF0, 0x00, /*     ####     */\
    0xF0, 0x00, /*     ####     */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @1827 'Z' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x03, /*         ## */\
    0x80, 0x01, /*        ##  */\
    0xC0, 0x00, /*       ##   */\
    0x60, 0x00, /*      ##    */\
    0x60, 0x00, /*      ##    */\
    0x30, 0x00, /*     ##     */\
    0x18, 0x00, /*    ##      */\
    0x1C, 0x00, /*   ###      */\
    0x0C, 0x00, /*   ##       */\
    0x06, 0x00, /*  ##        */\
    0x03, 0x00, /* ##         */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1869 '[' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x00, /*      */\
\
    /* @1890 '\' (5 pixels wide) */\
    0x00, /*       */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x07, /* ###   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x0E, /*  ###  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x1C, /*   ### */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @1911 ']' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x00, /*      */\
\
    /* @1932 '^' (7 pixels wide) */\
    0x00, /*         */\
    0x1C, /*   ###   */\
    0x1C, /*   ###   */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x63, /* ##   ## */\
    0x63, /* ##   ## */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @1953 '_' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1995 '`' (3 pixels wide) */\
    0x00, /*     */\
    0x03, /* ##  */\
    0x06, /*  ## */\
    0x04, /*   # */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @2016 'a' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x7C, /*   #####  */\
    0xFE, /*  ####### */\
    0xC3, /* ##    ## */\
    0xE0, /*      ### */\
    0xFC, /*   ###### */\
    0xDE, /*  #### ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE3, /* ##   ### */\
    0xFF, /* ######## */\
    0xCE, /*  ###  ## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2037 'b' (8 pixels wide) */\
    0x00, /*          */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x3B, /* ## ###   */\
    0x7F, /* #######  */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0x67, /* ###  ##  */\
    0x7F, /* #######  */\
    0x3B, /* ## ###   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2058 'c' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x3C, /*   ####  */\
    0x3E, /*  #####  */\
    0x67, /* ###  ## */\
    0x03, /* ##      */\
    0x03, /* ##      */\
    0x03, /* ##      */\
    0x03, /* ##      */\
    0x63, /* ##   ## */\
    0x67, /* ###  ## */\
    0x3E, /*  #####  */\
    0x1C, /*   ###   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @2079 'd' (8 pixels wide) */\
    0x00, /*          */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xDC, /*   ### ## */\
    0xFE, /*  ####### */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE6, /*  ##  ### */\
    0xFE, /*  ####### */\
    0xDC, /*   ### ## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2100 'e' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0xE6, /*  ##  ### */\
    0xC3, /* ##    ## */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0xC7, /* ###   ## */\
    0x7E, /*  ######  */\
    0x3C, /*   ####   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2121 'f' (6 pixels wide) */\
    0x00, /*        */\
    0x3C, /*   #### */\
    0x3E, /*  ##### */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x1F, /* #####  */\
    0x1F, /* #####  */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @2142 'g' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xDC, /*   ### ## */\
    0xFE, /*  ####### */\
    0xE6, /*  ##  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE7, /* ###  ### */\
    0xFE, /*  ####### */\
    0xDC, /*   ### ## */\
    0xC0, /*       ## */\
    0xE3, /* ##   ### */\
    0x7F, /* #######  */\
    0x3E, /*  #####   */\
    0x00, /*          */\
\
    /* @2163 'h' (8 pixels wide) */\
    0x00, /*          */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x7B, /* ## ####  */\
    0xFF, /* ######## */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2184 'i' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @2205 'j' (4 pixels wide) */\
    0x00, /*      */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x00, /*      */\
    0x00, /*      */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0F, /* #### */\
    0x07, /* ###  */\
    0x00, /*      */\
\
    /* @2226 'k' (7 pixels wide) */\
    0x00, /*         */\
    0x03, /* ##      */\
    0x03, /* ##      */\
    0x03, /* ##      */\
    0x03, /* ##      */\
    0x63, /* ##   ## */\
    0x33, /* ##  ##  */\
    0x1B, /* ## ##   */\
    0x0B, /* ## #    */\
    0x0F, /* ####    */\
    0x1F, /* #####   */\
    0x1B, /* ## ##   */\
    0x33, /* ##  ##  */\
    0x33, /* ##  ##  */\
    0x63, /* ##   ## */\
    0x63, /* ##   ## */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @2247 'l' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @2268 'm' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x3B, 0x07, /* ## ###  ###  */\
    0xBF, 0x0F, /* ###### ##### */\
    0xE7, 0x0C, /* ###  ###  ## */\
    0x63, 0x0C, /* ##   ##   ## */\
    0x63, 0x0C, /* ##   ##   ## */\
    0x63, 0x0C, /* ##   ##   ## */\
    0x63, 0x0C, /* ##   ##   ## */\
    0x63, 0x0C, /* ##   ##   ## */\
    0x63, 0x0C, /* ##   ##   ## */\
    0x63, 0x0C, /* ##   ##   ## */\
    0x63, 0x0C, /* ##   ##   ## */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @2310 'n' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x7B, /* ## ####  */\
    0xFF, /* ######## */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2331 'o' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x3C, /*   ####   */\
    0x7E, /*  ######  */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE7, /* ###  ### */\
    0x7E, /*  ######  */\
    0x3C, /*   ####   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2352 'p' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x3B, /* ## ###   */\
    0x7F, /* #######  */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0x67, /* ###  ##  */\
    0x7F, /* #######  */\
    0x3B, /* ## ###   */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x03, /* ##       */\
    0x00, /*          */\
\
    /* @2373 'q' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xDC, /*   ### ## */\
    0xFE, /*  ####### */\
    0xE7, /* ###  ### */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE6, /*  ##  ### */\
    0xFE, /*  ####### */\
    0xDC, /*   ### ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0xC0, /*       ## */\
    0x00, /*          */\
\
    /* @2394 'r' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1B, /* ## ## */\
    0x1F, /* ##### */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @2415 's' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x1E, /*  ####   */\
    0x3E, /*  #####  */\
    0x63, /* ##   ## */\
    0x03, /* ##      */\
    0x1E, /*  ####   */\
    0x3C, /*   ####  */\
    0x60, /*      ## */\
    0x60, /*      ## */\
    0x63, /* ##   ## */\
    0x3F, /* ######  */\
    0x1E, /*  ####   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @2436 't' (5 pixels wide) */\
    0x00, /*       */\
    0x04, /*   #   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x1F, /* ##### */\
    0x1F, /* ##### */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x1E, /*  #### */\
    0x1C, /*   ### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @2457 'u' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xC3, /* ##    ## */\
    0xE7, /* ###  ### */\
    0xFF, /* ######## */\
    0xDE, /*  #### ## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2478 'v' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0xC6, 0x00, /*  ##   ##  */\
    0xC6, 0x00, /*  ##   ##  */\
    0xC6, 0x00, /*  ##   ##  */\
    0xEE, 0x00, /*  ### ###  */\
    0x6C, 0x00, /*   ## ##   */\
    0x6C, 0x00, /*   ## ##   */\
    0x6C, 0x00, /*   ## ##   */\
    0x38, 0x00, /*    ###    */\
    0x38, 0x00, /*    ###    */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @2520 'w' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0xE3, 0x18, /* ##   ###   ## */\
    0xE3, 0x18, /* ##   ###   ## */\
    0xE3, 0x18, /* ##   ###   ## */\
    0xB6, 0x0D, /*  ## ## ## ##  */\
    0xB6, 0x0D, /*  ## ## ## ##  */\
    0xB6, 0x0D, /*  ## ## ## ##  */\
    0xB6, 0x0D, /*  ## ## ## ##  */\
    0xB6, 0x0D, /*  ## ## ## ##  */\
    0x96, 0x0D, /*  ## #  ## ##  */\
    0x1C, 0x07, /*   ###   ###   */\
    0x1C, 0x07, /*   ###   ###   */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @2562 'x' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xC3, /* ##    ## */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0x3C, /*   ####   */\
    0x3C, /*   ####   */\
    0x18, /*    ##    */\
    0x3C, /*   ####   */\
    0x3C, /*   ####   */\
    0x66, /*  ##  ##  */\
    0x66, /*  ##  ##  */\
    0xC3, /* ##    ## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2583 'y' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x83, 0x01, /* ##     ## */\
    0xC6, 0x00, /*  ##   ##  */\
    0xC6, 0x00, /*  ##   ##  */\
    0xC6, 0x00, /*  ##   ##  */\
    0x6C, 0x00, /*   ## ##   */\
    0x6C, 0x00, /*   ## ##   */\
    0x6C, 0x00, /*   ## ##   */\
    0x38, 0x00, /*    ###    */\
    0x38, 0x00, /*    ###    */\
    0x38, 0x00, /*    ###    */\
    0x18, 0x00, /*    ##     */\
    0x18, 0x00, /*    ##     */\
    0x18, 0x00, /*    ##     */\
    0x0E, 0x00, /*  ###      */\
    0x06, 0x00, /*  ##       */\
    0x00, 0x00, /*           */\
\
    /* @2625 'z' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0xC0, /*       ## */\
    0x60, /*      ##  */\
    0x30, /*     ##   */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x06, /*  ##      */\
    0x03, /* ##       */\
    0xFF, /* ######## */\
    0xFF, /* ######## */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @2646 '{' (6 pixels wide) */\
    0x00, /*        */\
    0x38, /*    ### */\
    0x3C, /*   #### */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0E, /*  ###   */\
    0x03, /* ##     */\
    0x07, /* ###    */\
    0x0E, /*  ###   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x3C, /*   #### */\
    0x38, /*    ### */\
    0x00, /*        */\
\
    /* @2667 '|' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
\
    /* @2688 '}' (6 pixels wide) */\
    0x00, /*        */\
    0x07, /* ###    */\
    0x0F, /* ####   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x18, /*    ##  */\
    0x30, /*     ## */\
    0x38, /*    ### */\
    0x1C, /*   ###  */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0F, /* ####   */\
    0x07, /* ###    */\
    0x00, /*        */\
\
    /* @2709 '~' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x8E, /*  ###   # */\
    0xFF, /* ######## */\
    0x71, /* #   ###  */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
}


/** @brief Initialization macro for the arial_16pt font information. */
#define FONTS_ARIAL_16PT_INFO \
{\
    -1, /* Character width (-1 means dynamic width). */\
    21, /* Character height (-1 means dynamic height). */\
    -1, /* Bitmap offset (-1 means dynamic offset). */\
    3, /* Deescriptor entry size. */\
    ' ', /* Start character. */\
    '~', /* End character. */\
    fonts_arial_16pt_descriptors, /*  Character descriptor array. */\
    fonts_arial_16pt_bitmaps, /* Character bitmap array. */\
}


/** @brief Initialization macro for the descriptors of the corierNew_b8pt font. */
#define FONTS_CORIERNEW_B8PT_DESCRIPTORS \
{\
         8, (   0 & 0xFF)       , (   0 >> 8) & 0xFF,  /*   */\
         2, (  11 & 0xFF)       , (  11 >> 8) & 0xFF,  /* ! */\
         3, (  22 & 0xFF)       , (  22 >> 8) & 0xFF,  /* " */\
         7, (  33 & 0xFF)       , (  33 >> 8) & 0xFF,  /* # */\
         5, (  44 & 0xFF)       , (  44 >> 8) & 0xFF,  /* $ */\
         7, (  55 & 0xFF)       , (  55 >> 8) & 0xFF,  /* % */\
         6, (  66 & 0xFF)       , (  66 >> 8) & 0xFF,  /* & */\
         1, (  77 & 0xFF)       , (  77 >> 8) & 0xFF,  /* ' */\
         3, (  88 & 0xFF)       , (  88 >> 8) & 0xFF,  /* ( */\
         3, (  99 & 0xFF)       , (  99 >> 8) & 0xFF,  /* ) */\
         6, ( 110 & 0xFF)       , ( 110 >> 8) & 0xFF,  /* * */\
         5, ( 121 & 0xFF)       , ( 121 >> 8) & 0xFF,  /* + */\
         2, ( 132 & 0xFF)       , ( 132 >> 8) & 0xFF,  /* , */\
         5, ( 143 & 0xFF)       , ( 143 >> 8) & 0xFF,  /* - */\
         2, ( 154 & 0xFF)       , ( 154 >> 8) & 0xFF,  /* . */\
         5, ( 165 & 0xFF)       , ( 165 >> 8) & 0xFF,  /* / */\
         5, ( 176 & 0xFF)       , ( 176 >> 8) & 0xFF,  /* 0 */\
         6, ( 187 & 0xFF)       , ( 187 >> 8) & 0xFF,  /* 1 */\
         5, ( 198 & 0xFF)       , ( 198 >> 8) & 0xFF,  /* 2 */\
         5, ( 209 & 0xFF)       , ( 209 >> 8) & 0xFF,  /* 3 */\
         6, ( 220 & 0xFF)       , ( 220 >> 8) & 0xFF,  /* 4 */\
         5, ( 231 & 0xFF)       , ( 231 >> 8) & 0xFF,  /* 5 */\
         5, ( 242 & 0xFF)       , ( 242 >> 8) & 0xFF,  /* 6 */\
         5, ( 253 & 0xFF)       , ( 253 >> 8) & 0xFF,  /* 7 */\
         5, ( 264 & 0xFF)       , ( 264 >> 8) & 0xFF,  /* 8 */\
         5, ( 275 & 0xFF)       , ( 275 >> 8) & 0xFF,  /* 9 */\
         2, ( 286 & 0xFF)       , ( 286 >> 8) & 0xFF,  /* : */\
         2, ( 297 & 0xFF)       , ( 297 >> 8) & 0xFF,  /* ; */\
         6, ( 308 & 0xFF)       , ( 308 >> 8) & 0xFF,  /* < */\
         7, ( 319 & 0xFF)       , ( 319 >> 8) & 0xFF,  /* = */\
         6, ( 330 & 0xFF)       , ( 330 >> 8) & 0xFF,  /* > */\
         5, ( 341 & 0xFF)       , ( 341 >> 8) & 0xFF,  /* ? */\
         6, ( 352 & 0xFF)       , ( 352 >> 8) & 0xFF,  /* @ */\
         7, ( 363 & 0xFF)       , ( 363 >> 8) & 0xFF,  /* A */\
         6, ( 374 & 0xFF)       , ( 374 >> 8) & 0xFF,  /* B */\
         6, ( 385 & 0xFF)       , ( 385 >> 8) & 0xFF,  /* C */\
         6, ( 396 & 0xFF)       , ( 396 >> 8) & 0xFF,  /* D */\
         6, ( 407 & 0xFF)       , ( 407 >> 8) & 0xFF,  /* E */\
         6, ( 418 & 0xFF)       , ( 418 >> 8) & 0xFF,  /* F */\
         7, ( 429 & 0xFF)       , ( 429 >> 8) & 0xFF,  /* G */\
         7, ( 440 & 0xFF)       , ( 440 >> 8) & 0xFF,  /* H */\
         6, ( 451 & 0xFF)       , ( 451 >> 8) & 0xFF,  /* I */\
         7, ( 462 & 0xFF)       , ( 462 >> 8) & 0xFF,  /* J */\
         7, ( 473 & 0xFF)       , ( 473 >> 8) & 0xFF,  /* K */\
         6, ( 484 & 0xFF)       , ( 484 >> 8) & 0xFF,  /* L */\
         7, ( 495 & 0xFF)       , ( 495 >> 8) & 0xFF,  /* M */\
         7, ( 506 & 0xFF)       , ( 506 >> 8) & 0xFF,  /* N */\
         6, ( 517 & 0xFF)       , ( 517 >> 8) & 0xFF,  /* O */\
         6, ( 528 & 0xFF)       , ( 528 >> 8) & 0xFF,  /* P */\
         6, ( 539 & 0xFF)       , ( 539 >> 8) & 0xFF,  /* Q */\
         7, ( 550 & 0xFF)       , ( 550 >> 8) & 0xFF,  /* R */\
         5, ( 561 & 0xFF)       , ( 561 >> 8) & 0xFF,  /* S */\
         6, ( 572 & 0xFF)       , ( 572 >> 8) & 0xFF,  /* T */\
         7, ( 583 & 0xFF)       , ( 583 >> 8) & 0xFF,  /* U */\
         7, ( 594 & 0xFF)       , ( 594 >> 8) & 0xFF,  /* V */\
         9, ( 605 & 0xFF)       , ( 605 >> 8) & 0xFF,  /* W */\
         7, ( 627 & 0xFF)       , ( 627 >> 8) & 0xFF,  /* X */\
         8, ( 638 & 0xFF)       , ( 638 >> 8) & 0xFF,  /* Y */\
         5, ( 649 & 0xFF)       , ( 649 >> 8) & 0xFF,  /* Z */\
         4, ( 660 & 0xFF)       , ( 660 >> 8) & 0xFF,  /* [ */\
         6, ( 671 & 0xFF)       , ( 671 >> 8) & 0xFF,  /* \ */\
         4, ( 682 & 0xFF)       , ( 682 >> 8) & 0xFF,  /* ] */\
         5, ( 693 & 0xFF)       , ( 693 >> 8) & 0xFF,  /* ^ */\
         8, ( 704 & 0xFF)       , ( 704 >> 8) & 0xFF,  /* _ */\
         2, ( 715 & 0xFF)       , ( 715 >> 8) & 0xFF,  /* ` */\
         7, ( 726 & 0xFF)       , ( 726 >> 8) & 0xFF,  /* a */\
         7, ( 737 & 0xFF)       , ( 737 >> 8) & 0xFF,  /* b */\
         6, ( 748 & 0xFF)       , ( 748 >> 8) & 0xFF,  /* c */\
         7, ( 759 & 0xFF)       , ( 759 >> 8) & 0xFF,  /* d */\
         6, ( 770 & 0xFF)       , ( 770 >> 8) & 0xFF,  /* e */\
         5, ( 781 & 0xFF)       , ( 781 >> 8) & 0xFF,  /* f */\
         7, ( 792 & 0xFF)       , ( 792 >> 8) & 0xFF,  /* g */\
         7, ( 803 & 0xFF)       , ( 803 >> 8) & 0xFF,  /* h */\
         6, ( 814 & 0xFF)       , ( 814 >> 8) & 0xFF,  /* i */\
         5, ( 825 & 0xFF)       , ( 825 >> 8) & 0xFF,  /* j */\
         7, ( 836 & 0xFF)       , ( 836 >> 8) & 0xFF,  /* k */\
         6, ( 847 & 0xFF)       , ( 847 >> 8) & 0xFF,  /* l */\
         9, ( 858 & 0xFF)       , ( 858 >> 8) & 0xFF,  /* m */\
         7, ( 880 & 0xFF)       , ( 880 >> 8) & 0xFF,  /* n */\
         6, ( 891 & 0xFF)       , ( 891 >> 8) & 0xFF,  /* o */\
         7, ( 902 & 0xFF)       , ( 902 >> 8) & 0xFF,  /* p */\
         7, ( 913 & 0xFF)       , ( 913 >> 8) & 0xFF,  /* q */\
         6, ( 924 & 0xFF)       , ( 924 >> 8) & 0xFF,  /* r */\
         6, ( 935 & 0xFF)       , ( 935 >> 8) & 0xFF,  /* s */\
         6, ( 946 & 0xFF)       , ( 946 >> 8) & 0xFF,  /* t */\
         7, ( 957 & 0xFF)       , ( 957 >> 8) & 0xFF,  /* u */\
         7, ( 968 & 0xFF)       , ( 968 >> 8) & 0xFF,  /* v */\
         9, ( 979 & 0xFF)       , ( 979 >> 8) & 0xFF,  /* w */\
         7, (1001 & 0xFF)       , (1001 >> 8) & 0xFF,  /* x */\
         8, (1012 & 0xFF)       , (1012 >> 8) & 0xFF,  /* y */\
         5, (1023 & 0xFF)       , (1023 >> 8) & 0xFF,  /* z */\
         4, (1034 & 0xFF)       , (1034 >> 8) & 0xFF,  /* { */\
         2, (1045 & 0xFF)       , (1045 >> 8) & 0xFF,  /* | */\
         4, (1056 & 0xFF)       , (1056 >> 8) & 0xFF,  /* } */\
         6, (1067 & 0xFF)       , (1067 >> 8) & 0xFF,  /* ~ */\
}


/** @brief Initialization macro for the bitmaps of the corierNew_b8pt font. */
#define FONTS_CORIERNEW_B8PT_BITMAPS \
{\
\
    /* @0 ' ' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @11 '!' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @22 '"' (3 pixels wide) */\
    0x00, /*     */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @33 '#' (7 pixels wide) */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x7F, /* ####### */\
    0x36, /*  ## ##  */\
    0x7F, /* ####### */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @44 '$' (5 pixels wide) */\
    0x04, /*   #   */\
    0x1E, /*  #### */\
    0x13, /* ##  # */\
    0x03, /* ##    */\
    0x0E, /*  ###  */\
    0x18, /*    ## */\
    0x19, /* #  ## */\
    0x0F, /* ####  */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x00, /*       */\
\
    /* @55 '%' (7 pixels wide) */\
    0x00, /*         */\
    0x06, /*  ##     */\
    0x09, /* #  #    */\
    0x29, /* #  # #  */\
    0x3E, /*  #####  */\
    0x4A, /*  # #  # */\
    0x48, /*    #  # */\
    0x30, /*     ##  */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @66 '&' (6 pixels wide) */\
    0x00, /*        */\
    0x1C, /*   ###  */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x37, /* ### ## */\
    0x1D, /* # ###  */\
    0x19, /* #  ##  */\
    0x3E, /*  ##### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @77 ''' (1 pixels wide) */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @88 '(' (3 pixels wide) */\
    0x00, /*     */\
    0x04, /*   # */\
    0x06, /*  ## */\
    0x03, /* ##  */\
    0x03, /* ##  */\
    0x03, /* ##  */\
    0x03, /* ##  */\
    0x06, /*  ## */\
    0x04, /*   # */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @99 ')' (3 pixels wide) */\
    0x00, /*     */\
    0x01, /* #   */\
    0x03, /* ##  */\
    0x06, /*  ## */\
    0x06, /*  ## */\
    0x06, /*  ## */\
    0x06, /*  ## */\
    0x03, /* ##  */\
    0x01, /* #   */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @110 '*' (6 pixels wide) */\
    0x00, /*        */\
    0x0C, /*   ##   */\
    0x3F, /* ###### */\
    0x0C, /*   ##   */\
    0x12, /*  #  #  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @121 '+' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x1F, /* ##### */\
    0x04, /*   #   */\
    0x04, /*   #   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @132 ',' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x00, /*    */\
\
    /* @143 '-' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @154 '.' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @165 '/' (5 pixels wide) */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0E, /*  ###  */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @176 '0' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @187 '1' (6 pixels wide) */\
    0x00, /*        */\
    0x0C, /*   ##   */\
    0x0F, /* ####   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x3F, /* ###### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @198 '2' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x1B, /* ## ## */\
    0x18, /*    ## */\
    0x0C, /*   ##  */\
    0x06, /*  ##   */\
    0x03, /* ##    */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @209 '3' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x19, /* #  ## */\
    0x18, /*    ## */\
    0x0E, /*  ###  */\
    0x18, /*    ## */\
    0x19, /* #  ## */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @220 '4' (6 pixels wide) */\
    0x00, /*        */\
    0x18, /*    ##  */\
    0x1C, /*   ###  */\
    0x1A, /*  # ##  */\
    0x19, /* #  ##  */\
    0x3F, /* ###### */\
    0x18, /*    ##  */\
    0x3C, /*   #### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @231 '5' (5 pixels wide) */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x03, /* ##    */\
    0x0F, /* ####  */\
    0x19, /* #  ## */\
    0x18, /*    ## */\
    0x19, /* #  ## */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @242 '6' (5 pixels wide) */\
    0x00, /*       */\
    0x1C, /*   ### */\
    0x06, /*  ##   */\
    0x03, /* ##    */\
    0x0F, /* ####  */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @253 '7' (5 pixels wide) */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x19, /* #  ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @264 '8' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x0E, /*  ###  */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @275 '9' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1E, /*  #### */\
    0x18, /*    ## */\
    0x0C, /*   ##  */\
    0x07, /* ###   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @286 ':' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @297 ';' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @308 '<' (6 pixels wide) */\
    0x00, /*        */\
    0x20, /*      # */\
    0x18, /*    ##  */\
    0x04, /*   #    */\
    0x03, /* ##     */\
    0x04, /*   #    */\
    0x18, /*    ##  */\
    0x20, /*      # */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @319 '=' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x7F, /* ####### */\
    0x00, /*         */\
    0x7F, /* ####### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @330 '>' (6 pixels wide) */\
    0x00, /*        */\
    0x01, /* #      */\
    0x06, /*  ##    */\
    0x08, /*    #   */\
    0x30, /*     ## */\
    0x08, /*    #   */\
    0x06, /*  ##    */\
    0x01, /* #      */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @341 '?' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x1B, /* ## ## */\
    0x18, /*    ## */\
    0x0E, /*  ###  */\
    0x06, /*  ##   */\
    0x00, /*       */\
    0x06, /*  ##   */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @352 '@' (6 pixels wide) */\
    0x00, /*        */\
    0x1E, /*  ####  */\
    0x21, /* #    # */\
    0x39, /* #  ### */\
    0x25, /* # #  # */\
    0x39, /* #  ### */\
    0x01, /* #      */\
    0x21, /* #    # */\
    0x1E, /*  ####  */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @363 'A' (7 pixels wide) */\
    0x00, /*         */\
    0x1F, /* #####   */\
    0x1C, /*   ###   */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x3E, /*  #####  */\
    0x36, /*  ## ##  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @374 'B' (6 pixels wide) */\
    0x00, /*        */\
    0x1F, /* #####  */\
    0x36, /*  ## ## */\
    0x36, /*  ## ## */\
    0x1E, /*  ####  */\
    0x36, /*  ## ## */\
    0x36, /*  ## ## */\
    0x1F, /* #####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @385 'C' (6 pixels wide) */\
    0x00, /*        */\
    0x3E, /*  ##### */\
    0x23, /* ##   # */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x23, /* ##   # */\
    0x1E, /*  ####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @396 'D' (6 pixels wide) */\
    0x00, /*        */\
    0x1F, /* #####  */\
    0x36, /*  ## ## */\
    0x36, /*  ## ## */\
    0x36, /*  ## ## */\
    0x36, /*  ## ## */\
    0x36, /*  ## ## */\
    0x1F, /* #####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @407 'E' (6 pixels wide) */\
    0x00, /*        */\
    0x3F, /* ###### */\
    0x26, /*  ##  # */\
    0x16, /*  ## #  */\
    0x1E, /*  ####  */\
    0x16, /*  ## #  */\
    0x26, /*  ##  # */\
    0x3F, /* ###### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @418 'F' (6 pixels wide) */\
    0x00, /*        */\
    0x3F, /* ###### */\
    0x26, /*  ##  # */\
    0x16, /*  ## #  */\
    0x1E, /*  ####  */\
    0x16, /*  ## #  */\
    0x06, /*  ##    */\
    0x1F, /* #####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @429 'G' (7 pixels wide) */\
    0x00, /*         */\
    0x3E, /*  #####  */\
    0x23, /* ##   #  */\
    0x03, /* ##      */\
    0x03, /* ##      */\
    0x7B, /* ## #### */\
    0x33, /* ##  ##  */\
    0x3E, /*  #####  */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @440 'H' (7 pixels wide) */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x3E, /*  #####  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @451 'I' (6 pixels wide) */\
    0x00, /*        */\
    0x3F, /* ###### */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x3F, /* ###### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @462 'J' (7 pixels wide) */\
    0x00, /*         */\
    0x7C, /*   ##### */\
    0x30, /*     ##  */\
    0x30, /*     ##  */\
    0x30, /*     ##  */\
    0x33, /* ##  ##  */\
    0x33, /* ##  ##  */\
    0x1F, /* #####   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @473 'K' (7 pixels wide) */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x26, /*  ##  #  */\
    0x16, /*  ## #   */\
    0x0E, /*  ###    */\
    0x1E, /*  ####   */\
    0x36, /*  ## ##  */\
    0x6F, /* #### ## */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @484 'L' (6 pixels wide) */\
    0x00, /*        */\
    0x0F, /* ####   */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x26, /*  ##  # */\
    0x26, /*  ##  # */\
    0x3F, /* ###### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @495 'M' (7 pixels wide) */\
    0x00, /*         */\
    0x63, /* ##   ## */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x2A, /*  # # #  */\
    0x2A, /*  # # #  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @506 'N' (7 pixels wide) */\
    0x00, /*         */\
    0x73, /* ##  ### */\
    0x26, /*  ##  #  */\
    0x2E, /*  ### #  */\
    0x2A, /*  # # #  */\
    0x3A, /*  # ###  */\
    0x32, /*  #  ##  */\
    0x27, /* ###  #  */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @517 'O' (6 pixels wide) */\
    0x00, /*        */\
    0x1E, /*  ####  */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x1E, /*  ####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @528 'P' (6 pixels wide) */\
    0x00, /*        */\
    0x1F, /* #####  */\
    0x36, /*  ## ## */\
    0x36, /*  ## ## */\
    0x36, /*  ## ## */\
    0x1E, /*  ####  */\
    0x06, /*  ##    */\
    0x0F, /* ####   */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @539 'Q' (6 pixels wide) */\
    0x00, /*        */\
    0x1E, /*  ####  */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x1E, /*  ####  */\
    0x3C, /*   #### */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @550 'R' (7 pixels wide) */\
    0x00, /*         */\
    0x1F, /* #####   */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x1E, /*  ####   */\
    0x16, /*  ## #   */\
    0x36, /*  ## ##  */\
    0x6F, /* #### ## */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @561 'S' (5 pixels wide) */\
    0x00, /*       */\
    0x1E, /*  #### */\
    0x13, /* ##  # */\
    0x03, /* ##    */\
    0x0E, /*  ###  */\
    0x18, /*    ## */\
    0x19, /* #  ## */\
    0x0F, /* ####  */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @572 'T' (6 pixels wide) */\
    0x00, /*        */\
    0x3F, /* ###### */\
    0x2D, /* # ## # */\
    0x2D, /* # ## # */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x1E, /*  ####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @583 'U' (7 pixels wide) */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x1C, /*   ###   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @594 'V' (7 pixels wide) */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x14, /*   # #   */\
    0x1C, /*   ###   */\
    0x1C, /*   ###   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @605 'W' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0xEF, 0x01, /* #### #### */\
    0xC6, 0x00, /*  ##   ##  */\
    0xD6, 0x00, /*  ## # ##  */\
    0xD6, 0x00, /*  ## # ##  */\
    0x5C, 0x00, /*   ### #   */\
    0x6C, 0x00, /*   ## ##   */\
    0x6C, 0x00, /*   ## ##   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @627 'X' (7 pixels wide) */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x36, /*  ## ##  */\
    0x1C, /*   ###   */\
    0x1C, /*   ###   */\
    0x1C, /*   ###   */\
    0x36, /*  ## ##  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @638 'Y' (8 pixels wide) */\
    0x00, /*          */\
    0xF7, /* ### #### */\
    0x66, /*  ##  ##  */\
    0x3C, /*   ####   */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x3C, /*   ####   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @649 'Z' (5 pixels wide) */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x19, /* #  ## */\
    0x0D, /* # ##  */\
    0x0C, /*   ##  */\
    0x16, /*  ## # */\
    0x13, /* ##  # */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @660 '[' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @671 '\' (6 pixels wide) */\
    0x03, /* ##     */\
    0x07, /* ###    */\
    0x06, /*  ##    */\
    0x0E, /*  ###   */\
    0x0C, /*   ##   */\
    0x1C, /*   ###  */\
    0x18, /*    ##  */\
    0x38, /*    ### */\
    0x30, /*     ## */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @682 ']' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @693 '^' (5 pixels wide) */\
    0x00, /*       */\
    0x04, /*   #   */\
    0x0A, /*  # #  */\
    0x11, /* #   # */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @704 '_' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xFF, /* ######## */\
\
    /* @715 '`' (2 pixels wide) */\
    0x01, /* #  */\
    0x02, /*  # */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @726 'a' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x1E, /*  ####   */\
    0x30, /*     ##  */\
    0x3E, /*  #####  */\
    0x33, /* ##  ##  */\
    0x7E, /*  ###### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @737 'b' (7 pixels wide) */\
    0x00, /*         */\
    0x07, /* ###     */\
    0x06, /*  ##     */\
    0x3E, /*  #####  */\
    0x66, /*  ##  ## */\
    0x66, /*  ##  ## */\
    0x66, /*  ##  ## */\
    0x3F, /* ######  */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @748 'c' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x3E, /*  ##### */\
    0x23, /* ##   # */\
    0x03, /* ##     */\
    0x23, /* ##   # */\
    0x1E, /*  ####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @759 'd' (7 pixels wide) */\
    0x00, /*         */\
    0x38, /*    ###  */\
    0x30, /*     ##  */\
    0x3E, /*  #####  */\
    0x33, /* ##  ##  */\
    0x33, /* ##  ##  */\
    0x33, /* ##  ##  */\
    0x7E, /*  ###### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @770 'e' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x1E, /*  ####  */\
    0x33, /* ##  ## */\
    0x3F, /* ###### */\
    0x03, /* ##     */\
    0x3E, /*  ##### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @781 'f' (5 pixels wide) */\
    0x00, /*       */\
    0x1C, /*   ### */\
    0x06, /*  ##   */\
    0x1F, /* ##### */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @792 'g' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x7E, /*  ###### */\
    0x33, /* ##  ##  */\
    0x33, /* ##  ##  */\
    0x33, /* ##  ##  */\
    0x3E, /*  #####  */\
    0x30, /*     ##  */\
    0x1E, /*  ####   */\
    0x00, /*         */\
\
    /* @803 'h' (7 pixels wide) */\
    0x00, /*         */\
    0x07, /* ###     */\
    0x06, /*  ##     */\
    0x1E, /*  ####   */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @814 'i' (6 pixels wide) */\
    0x00, /*        */\
    0x0C, /*   ##   */\
    0x00, /*        */\
    0x0F, /* ####   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x3F, /* ###### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @825 'j' (5 pixels wide) */\
    0x00, /*       */\
    0x0C, /*   ##  */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x0F, /* ####  */\
    0x00, /*       */\
\
    /* @836 'k' (7 pixels wide) */\
    0x00, /*         */\
    0x07, /* ###     */\
    0x06, /*  ##     */\
    0x36, /*  ## ##  */\
    0x1E, /*  ####   */\
    0x1E, /*  ####   */\
    0x36, /*  ## ##  */\
    0x67, /* ###  ## */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @847 'l' (6 pixels wide) */\
    0x00, /*        */\
    0x0F, /* ####   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x3F, /* ###### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @858 'm' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x7F, 0x00, /* #######   */\
    0xD6, 0x00, /*  ## # ##  */\
    0xD6, 0x00, /*  ## # ##  */\
    0xD6, 0x00, /*  ## # ##  */\
    0xD7, 0x01, /* ### # ### */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @880 'n' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x1F, /* #####   */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @891 'o' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x1E, /*  ####  */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x33, /* ##  ## */\
    0x1E, /*  ####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @902 'p' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x3F, /* ######  */\
    0x66, /*  ##  ## */\
    0x66, /*  ##  ## */\
    0x66, /*  ##  ## */\
    0x3E, /*  #####  */\
    0x06, /*  ##     */\
    0x1F, /* #####   */\
    0x00, /*         */\
\
    /* @913 'q' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x7E, /*  ###### */\
    0x33, /* ##  ##  */\
    0x33, /* ##  ##  */\
    0x33, /* ##  ##  */\
    0x3E, /*  #####  */\
    0x30, /*     ##  */\
    0x7C, /*   ##### */\
    0x00, /*         */\
\
    /* @924 'r' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x37, /* ### ## */\
    0x2E, /*  ### # */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x1F, /* #####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @935 's' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x3E, /*  ##### */\
    0x23, /* ##   # */\
    0x1E, /*  ####  */\
    0x31, /* #   ## */\
    0x1F, /* #####  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @946 't' (6 pixels wide) */\
    0x00, /*        */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x1F, /* #####  */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x26, /*  ##  # */\
    0x1C, /*   ###  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @957 'u' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x37, /* ### ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x7C, /*   ##### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @968 'v' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x36, /*  ## ##  */\
    0x14, /*   # #   */\
    0x1C, /*   ###   */\
    0x1C, /*   ###   */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @979 'w' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0xEF, 0x01, /* #### #### */\
    0xD6, 0x00, /*  ## # ##  */\
    0xD6, 0x00, /*  ## # ##  */\
    0x6C, 0x00, /*   ## ##   */\
    0x6C, 0x00, /*   ## ##   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @1001 'x' (7 pixels wide) */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x77, /* ### ### */\
    0x36, /*  ## ##  */\
    0x1C, /*   ###   */\
    0x36, /*  ## ##  */\
    0x77, /* ### ### */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @1012 'y' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0xEE, /*  ### ### */\
    0x6C, /*   ## ##  */\
    0x28, /*    # #   */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0x0C, /*   ##     */\
    0x1F, /* #####    */\
    0x00, /*          */\
\
    /* @1023 'z' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x0D, /* # ##  */\
    0x06, /*  ##   */\
    0x13, /* ##  # */\
    0x1F, /* ##### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @1034 '{' (4 pixels wide) */\
    0x00, /*      */\
    0x0E, /*  ### */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x03, /* ##   */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x0E, /*  ### */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @1045 '|' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @1056 '}' (4 pixels wide) */\
    0x00, /*      */\
    0x07, /* ###  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x0C, /*   ## */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x07, /* ###  */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @1067 '~' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x26, /*  ##  # */\
    0x19, /* #  ##  */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
}


/** @brief Initialization macro for the corierNew_b8pt font information. */
#define FONTS_CORIERNEW_B8PT_INFO \
{\
    -1, /* Character width (-1 means dynamic width). */\
    11, /* Character height (-1 means dynamic height). */\
    -1, /* Bitmap offset (-1 means dynamic offset). */\
    3, /* Deescriptor entry size. */\
    ' ', /* Start character. */\
    '~', /* End character. */\
    fonts_coriernew_b8pt_descriptors, /*  Character descriptor array. */\
    fonts_coriernew_b8pt_bitmaps, /* Character bitmap array. */\
}


/** @brief Initialization macro for the descriptors of the irisUPC_b8pt font. */
#define FONTS_IRISUPC_B8PT_DESCRIPTORS \
{\
         8,  /*   */\
         1,  /* ! */\
         2,  /* " */\
         4,  /* # */\
         3,  /* $ */\
         5,  /* % */\
         4,  /* & */\
         1,  /* ' */\
         2,  /* ( */\
         2,  /* ) */\
         2,  /* * */\
         4,  /* + */\
         1,  /* , */\
         3,  /* - */\
         1,  /* . */\
         3,  /* / */\
         3,  /* 0 */\
         2,  /* 1 */\
         3,  /* 2 */\
         3,  /* 3 */\
         3,  /* 4 */\
         3,  /* 5 */\
         3,  /* 6 */\
         3,  /* 7 */\
         3,  /* 8 */\
         3,  /* 9 */\
         1,  /* : */\
         1,  /* ; */\
         3,  /* < */\
         3,  /* = */\
         3,  /* > */\
         3,  /* ? */\
         4,  /* @ */\
         4,  /* A */\
         2,  /* B */\
         5,  /* C */\
         4,  /* D */\
         3,  /* E */\
         3,  /* F */\
         5,  /* G */\
         4,  /* H */\
         1,  /* I */\
         2,  /* J */\
         3,  /* K */\
         3,  /* L */\
         6,  /* M */\
         5,  /* N */\
         5,  /* O */\
         2,  /* P */\
         5,  /* Q */\
         2,  /* R */\
         3,  /* S */\
         4,  /* T */\
         4,  /* U */\
         4,  /* V */\
         7,  /* W */\
         4,  /* X */\
         4,  /* Y */\
         4,  /* Z */\
         2,  /* [ */\
         3,  /* \ */\
         2,  /* ] */\
         4,  /* ^ */\
         3,  /* _ */\
         2,  /* ` */\
         4,  /* a */\
         3,  /* b */\
         4,  /* c */\
         3,  /* d */\
         3,  /* e */\
         3,  /* f */\
         3,  /* g */\
         3,  /* h */\
         1,  /* i */\
         2,  /* j */\
         4,  /* k */\
         1,  /* l */\
         5,  /* m */\
         4,  /* n */\
         3,  /* o */\
         3,  /* p */\
         3,  /* q */\
         3,  /* r */\
         2,  /* s */\
         2,  /* t */\
         3,  /* u */\
         3,  /* v */\
         5,  /* w */\
         3,  /* x */\
         3,  /* y */\
         3,  /* z */\
         3,  /* { */\
         1,  /* | */\
         3,  /* } */\
         4,  /* ~ */\
}


/** @brief Initialization macro for the bitmaps of the irisUPC_b8pt font. */
#define FONTS_IRISUPC_B8PT_BITMAPS \
{\
\
    /* @0 ' ' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @8 '!' (1 pixels wide) */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @16 '"' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @24 '#' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x03, /* ##   */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @32 '$' (3 pixels wide) */\
    0x00, /*     */\
    0x02, /*  #  */\
    0x07, /* ### */\
    0x02, /*  #  */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x02, /*  #  */\
    0x00, /*     */\
\
    /* @40 '%' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x1B, /* ## ## */\
    0x04, /*   #   */\
    0x02, /*  #    */\
    0x19, /* #  ## */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @48 '&' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x07, /* ###  */\
    0x02, /*  #   */\
    0x05, /* # #  */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @56 ''' (1 pixels wide) */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @64 '(' (2 pixels wide) */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x02, /*  # */\
    0x02, /*  # */\
\
    /* @72 ')' (2 pixels wide) */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x01, /* #  */\
    0x01, /* #  */\
\
    /* @80 '*' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @88 '+' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x04, /*   #  */\
    0x0F, /* #### */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @96 ',' (1 pixels wide) */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
\
    /* @104 '-' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @112 '.' (1 pixels wide) */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @120 '/' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x04, /*   # */\
    0x02, /*  #  */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @128 '0' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @136 '1' (2 pixels wide) */\
    0x00, /*    */\
    0x02, /*  # */\
    0x03, /* ## */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @144 '2' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x02, /*  #  */\
    0x01, /* #   */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @152 '3' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x06, /*  ## */\
    0x04, /*   # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @160 '4' (3 pixels wide) */\
    0x00, /*     */\
    0x04, /*   # */\
    0x06, /*  ## */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x04, /*   # */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @168 '5' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x01, /* #   */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @176 '6' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @184 '7' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x02, /*  #  */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @192 '8' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @200 '9' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @208 ':' (1 pixels wide) */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @216 ';' (1 pixels wide) */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
\
    /* @224 '<' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x04, /*   # */\
    0x02, /*  #  */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @232 '=' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @240 '>' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x01, /* #   */\
    0x02, /*  #  */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @248 '?' (3 pixels wide) */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x04, /*   # */\
    0x02, /*  #  */\
    0x00, /*     */\
    0x02, /*  #  */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @256 '@' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @264 'A' (4 pixels wide) */\
    0x00, /*      */\
    0x02, /*  #   */\
    0x06, /*  ##  */\
    0x05, /* # #  */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @272 'B' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @280 'C' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x01, /* #     */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @288 'D' (4 pixels wide) */\
    0x00, /*      */\
    0x07, /* ###  */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x07, /* ###  */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @296 'E' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x01, /* #   */\
    0x07, /* ### */\
    0x01, /* #   */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @304 'F' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x01, /* #   */\
    0x07, /* ### */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @312 'G' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x13, /* ##  # */\
    0x01, /* #     */\
    0x11, /* #   # */\
    0x1E, /*  #### */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @320 'H' (4 pixels wide) */\
    0x00, /*      */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x0F, /* #### */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @328 'I' (1 pixels wide) */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @336 'J' (2 pixels wide) */\
    0x00, /*    */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x03, /* ## */\
    0x00, /*    */\
\
    /* @344 'K' (3 pixels wide) */\
    0x00, /*     */\
    0x05, /* # # */\
    0x03, /* ##  */\
    0x03, /* ##  */\
    0x03, /* ##  */\
    0x05, /* # # */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @352 'L' (3 pixels wide) */\
    0x00, /*     */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @360 'M' (6 pixels wide) */\
    0x00, /*        */\
    0x12, /*  #  #  */\
    0x32, /*  #  ## */\
    0x2F, /* #### # */\
    0x2D, /* # ## # */\
    0x25, /* # #  # */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @368 'N' (5 pixels wide) */\
    0x00, /*       */\
    0x13, /* ##  # */\
    0x13, /* ##  # */\
    0x15, /* # # # */\
    0x1D, /* # ### */\
    0x19, /* #  ## */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @376 'O' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @384 'P' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @392 'Q' (5 pixels wide) */\
    0x00, /*       */\
    0x0E, /*  ###  */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x11, /* #   # */\
    0x0E, /*  ###  */\
    0x18, /*    ## */\
    0x00, /*       */\
\
    /* @400 'R' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x01, /* #  */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @408 'S' (3 pixels wide) */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x03, /* ##  */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @416 'T' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x04, /*   #  */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @424 'U' (4 pixels wide) */\
    0x00, /*      */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @432 'V' (4 pixels wide) */\
    0x00, /*      */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x02, /*  #   */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @440 'W' (7 pixels wide) */\
    0x00, /*         */\
    0x49, /* #  #  # */\
    0x3B, /* ## ###  */\
    0x36, /*  ## ##  */\
    0x36, /*  ## ##  */\
    0x12, /*  #  #   */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @448 'X' (4 pixels wide) */\
    0x00, /*      */\
    0x0B, /* ## # */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x0D, /* # ## */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @456 'Y' (4 pixels wide) */\
    0x00, /*      */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @464 'Z' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x04, /*   #  */\
    0x02, /*  #   */\
    0x02, /*  #   */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @472 '[' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x03, /* ## */\
    0x00, /*    */\
\
    /* @480 '\' (3 pixels wide) */\
    0x00, /*     */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x04, /*   # */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @488 ']' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x03, /* ## */\
    0x00, /*    */\
\
    /* @496 '^' (4 pixels wide) */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x06, /*  ##  */\
    0x09, /* #  # */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @504 '_' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x00, /*     */\
\
    /* @512 '`' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @520 'a' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x06, /*  ##  */\
    0x07, /* ###  */\
    0x0F, /* #### */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @528 'b' (3 pixels wide) */\
    0x00, /*     */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @536 'c' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x07, /* ###  */\
    0x09, /* #  # */\
    0x06, /*  ##  */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @544 'd' (3 pixels wide) */\
    0x00, /*     */\
    0x04, /*   # */\
    0x04, /*   # */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @552 'e' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @560 'f' (3 pixels wide) */\
    0x00, /*     */\
    0x04, /*   # */\
    0x02, /*  #  */\
    0x07, /* ### */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @568 'g' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x06, /*  ## */\
    0x03, /* ##  */\
    0x07, /* ### */\
    0x07, /* ### */\
    0x00, /*     */\
\
    /* @576 'h' (3 pixels wide) */\
    0x00, /*     */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @584 'i' (1 pixels wide) */\
    0x00, /*   */\
    0x01, /* # */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @592 'j' (2 pixels wide) */\
    0x00, /*    */\
    0x02, /*  # */\
    0x00, /*    */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x03, /* ## */\
    0x00, /*    */\
\
    /* @600 'k' (4 pixels wide) */\
    0x00, /*      */\
    0x01, /* #    */\
    0x01, /* #    */\
    0x0D, /* # ## */\
    0x03, /* ##   */\
    0x0D, /* # ## */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @608 'l' (1 pixels wide) */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @616 'm' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x1F, /* ##### */\
    0x15, /* # # # */\
    0x15, /* # # # */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @624 'n' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x07, /* ###  */\
    0x09, /* #  # */\
    0x09, /* #  # */\
    0x00, /*      */\
    0x00, /*      */\
\
    /* @632 'o' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x03, /* ##  */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @640 'p' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x01, /* #   */\
    0x00, /*     */\
\
    /* @648 'q' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x06, /*  ## */\
    0x00, /*     */\
\
    /* @656 'r' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x01, /* #   */\
    0x01, /* #   */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @664 's' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @672 't' (2 pixels wide) */\
    0x00, /*    */\
    0x01, /* #  */\
    0x01, /* #  */\
    0x03, /* ## */\
    0x01, /* #  */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @680 'u' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x05, /* # # */\
    0x05, /* # # */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @688 'v' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x05, /* # # */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @696 'w' (5 pixels wide) */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x15, /* # # # */\
    0x0A, /*  # #  */\
    0x0A, /*  # #  */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @704 'x' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x02, /*  #  */\
    0x05, /* # # */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @712 'y' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x05, /* # # */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x01, /* #   */\
    0x00, /*     */\
\
    /* @720 'z' (3 pixels wide) */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x07, /* ### */\
    0x02, /*  #  */\
    0x07, /* ### */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @728 '{' (3 pixels wide) */\
    0x00, /*     */\
    0x06, /*  ## */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x03, /* ##  */\
    0x02, /*  #  */\
    0x06, /*  ## */\
    0x00, /*     */\
\
    /* @736 '|' (1 pixels wide) */\
    0x00, /*   */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x01, /* # */\
    0x00, /*   */\
    0x00, /*   */\
\
    /* @744 '}' (3 pixels wide) */\
    0x00, /*     */\
    0x03, /* ##  */\
    0x02, /*  #  */\
    0x02, /*  #  */\
    0x06, /*  ## */\
    0x02, /*  #  */\
    0x03, /* ##  */\
    0x00, /*     */\
\
    /* @752 '~' (4 pixels wide) */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x08, /*    # */\
    0x07, /* ###  */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
}


/** @brief Initialization macro for the irisUPC_b8pt font information. */
#define FONTS_IRISUPC_B8PT_INFO \
{\
    -1, /* Character width (-1 means dynamic width). */\
    8, /* Character height (-1 means dynamic height). */\
    8, /* Bitmap offset (-1 means dynamic offset). */\
    1, /* Deescriptor entry size. */\
    ' ', /* Start character. */\
    '~', /* End character. */\
    fonts_irisupc_b8pt_descriptors, /*  Character descriptor array. */\
    fonts_irisupc_b8pt_bitmaps, /* Character bitmap array. */\
}


/** @brief Initialization macro for the descriptors of the arial_20pt font. */
#define FONTS_ARIAL_20PT_DESCRIPTORS \
{\
         8, (   0 & 0xFF)       , (   0 >> 8) & 0xFF,  /*   */\
         2, (  26 & 0xFF)       , (  26 >> 8) & 0xFF,  /* ! */\
         5, (  52 & 0xFF)       , (  52 >> 8) & 0xFF,  /* " */\
        12, (  78 & 0xFF)       , (  78 >> 8) & 0xFF,  /* # */\
        10, ( 130 & 0xFF)       , ( 130 >> 8) & 0xFF,  /* $ */\
        17, ( 182 & 0xFF)       , ( 182 >> 8) & 0xFF,  /* % */\
        13, ( 260 & 0xFF)       , ( 260 >> 8) & 0xFF,  /* & */\
         2, ( 312 & 0xFF)       , ( 312 >> 8) & 0xFF,  /* ' */\
         5, ( 338 & 0xFF)       , ( 338 >> 8) & 0xFF,  /* ( */\
         5, ( 364 & 0xFF)       , ( 364 >> 8) & 0xFF,  /* ) */\
         8, ( 390 & 0xFF)       , ( 390 >> 8) & 0xFF,  /* * */\
        12, ( 416 & 0xFF)       , ( 416 >> 8) & 0xFF,  /* + */\
         2, ( 468 & 0xFF)       , ( 468 >> 8) & 0xFF,  /* , */\
         6, ( 494 & 0xFF)       , ( 494 >> 8) & 0xFF,  /* - */\
         2, ( 520 & 0xFF)       , ( 520 >> 8) & 0xFF,  /* . */\
         6, ( 546 & 0xFF)       , ( 546 >> 8) & 0xFF,  /* / */\
        10, ( 572 & 0xFF)       , ( 572 >> 8) & 0xFF,  /* 0 */\
         6, ( 624 & 0xFF)       , ( 624 >> 8) & 0xFF,  /* 1 */\
        10, ( 650 & 0xFF)       , ( 650 >> 8) & 0xFF,  /* 2 */\
        10, ( 702 & 0xFF)       , ( 702 >> 8) & 0xFF,  /* 3 */\
        11, ( 754 & 0xFF)       , ( 754 >> 8) & 0xFF,  /* 4 */\
        10, ( 806 & 0xFF)       , ( 806 >> 8) & 0xFF,  /* 5 */\
        10, ( 858 & 0xFF)       , ( 858 >> 8) & 0xFF,  /* 6 */\
        10, ( 910 & 0xFF)       , ( 910 >> 8) & 0xFF,  /* 7 */\
        10, ( 962 & 0xFF)       , ( 962 >> 8) & 0xFF,  /* 8 */\
        10, (1014 & 0xFF)       , (1014 >> 8) & 0xFF,  /* 9 */\
         2, (1066 & 0xFF)       , (1066 >> 8) & 0xFF,  /* : */\
         2, (1092 & 0xFF)       , (1092 >> 8) & 0xFF,  /* ; */\
        10, (1118 & 0xFF)       , (1118 >> 8) & 0xFF,  /* < */\
        10, (1170 & 0xFF)       , (1170 >> 8) & 0xFF,  /* = */\
        10, (1222 & 0xFF)       , (1222 >> 8) & 0xFF,  /* > */\
        10, (1274 & 0xFF)       , (1274 >> 8) & 0xFF,  /* ? */\
        21, (1326 & 0xFF)       , (1326 >> 8) & 0xFF,  /* @ */\
        15, (1404 & 0xFF)       , (1404 >> 8) & 0xFF,  /* A */\
        13, (1456 & 0xFF)       , (1456 >> 8) & 0xFF,  /* B */\
        14, (1508 & 0xFF)       , (1508 >> 8) & 0xFF,  /* C */\
        13, (1560 & 0xFF)       , (1560 >> 8) & 0xFF,  /* D */\
        12, (1612 & 0xFF)       , (1612 >> 8) & 0xFF,  /* E */\
        11, (1664 & 0xFF)       , (1664 >> 8) & 0xFF,  /* F */\
        15, (1716 & 0xFF)       , (1716 >> 8) & 0xFF,  /* G */\
        12, (1768 & 0xFF)       , (1768 >> 8) & 0xFF,  /* H */\
         2, (1820 & 0xFF)       , (1820 >> 8) & 0xFF,  /* I */\
         9, (1846 & 0xFF)       , (1846 >> 8) & 0xFF,  /* J */\
        12, (1898 & 0xFF)       , (1898 >> 8) & 0xFF,  /* K */\
        10, (1950 & 0xFF)       , (1950 >> 8) & 0xFF,  /* L */\
        15, (2002 & 0xFF)       , (2002 >> 8) & 0xFF,  /* M */\
        12, (2054 & 0xFF)       , (2054 >> 8) & 0xFF,  /* N */\
        15, (2106 & 0xFF)       , (2106 >> 8) & 0xFF,  /* O */\
        12, (2158 & 0xFF)       , (2158 >> 8) & 0xFF,  /* P */\
        15, (2210 & 0xFF)       , (2210 >> 8) & 0xFF,  /* Q */\
        13, (2262 & 0xFF)       , (2262 >> 8) & 0xFF,  /* R */\
        13, (2314 & 0xFF)       , (2314 >> 8) & 0xFF,  /* S */\
        12, (2366 & 0xFF)       , (2366 >> 8) & 0xFF,  /* T */\
        12, (2418 & 0xFF)       , (2418 >> 8) & 0xFF,  /* U */\
        15, (2470 & 0xFF)       , (2470 >> 8) & 0xFF,  /* V */\
        21, (2522 & 0xFF)       , (2522 >> 8) & 0xFF,  /* W */\
        15, (2600 & 0xFF)       , (2600 >> 8) & 0xFF,  /* X */\
        14, (2652 & 0xFF)       , (2652 >> 8) & 0xFF,  /* Y */\
        12, (2704 & 0xFF)       , (2704 >> 8) & 0xFF,  /* Z */\
         4, (2756 & 0xFF)       , (2756 >> 8) & 0xFF,  /* [ */\
         6, (2782 & 0xFF)       , (2782 >> 8) & 0xFF,  /* \ */\
         4, (2808 & 0xFF)       , (2808 >> 8) & 0xFF,  /* ] */\
         9, (2834 & 0xFF)       , (2834 >> 8) & 0xFF,  /* ^ */\
        12, (2886 & 0xFF)       , (2886 >> 8) & 0xFF,  /* _ */\
         3, (2938 & 0xFF)       , (2938 >> 8) & 0xFF,  /* ` */\
        10, (2964 & 0xFF)       , (2964 >> 8) & 0xFF,  /* a */\
        10, (3016 & 0xFF)       , (3016 >> 8) & 0xFF,  /* b */\
         9, (3068 & 0xFF)       , (3068 >> 8) & 0xFF,  /* c */\
        10, (3120 & 0xFF)       , (3120 >> 8) & 0xFF,  /* d */\
        10, (3172 & 0xFF)       , (3172 >> 8) & 0xFF,  /* e */\
         7, (3224 & 0xFF)       , (3224 >> 8) & 0xFF,  /* f */\
        10, (3250 & 0xFF)       , (3250 >> 8) & 0xFF,  /* g */\
        10, (3302 & 0xFF)       , (3302 >> 8) & 0xFF,  /* h */\
         2, (3354 & 0xFF)       , (3354 >> 8) & 0xFF,  /* i */\
         4, (3380 & 0xFF)       , (3380 >> 8) & 0xFF,  /* j */\
         9, (3406 & 0xFF)       , (3406 >> 8) & 0xFF,  /* k */\
         2, (3458 & 0xFF)       , (3458 >> 8) & 0xFF,  /* l */\
        16, (3484 & 0xFF)       , (3484 >> 8) & 0xFF,  /* m */\
        10, (3536 & 0xFF)       , (3536 >> 8) & 0xFF,  /* n */\
        10, (3588 & 0xFF)       , (3588 >> 8) & 0xFF,  /* o */\
        10, (3640 & 0xFF)       , (3640 >> 8) & 0xFF,  /* p */\
        10, (3692 & 0xFF)       , (3692 >> 8) & 0xFF,  /* q */\
         6, (3744 & 0xFF)       , (3744 >> 8) & 0xFF,  /* r */\
         9, (3770 & 0xFF)       , (3770 >> 8) & 0xFF,  /* s */\
         5, (3822 & 0xFF)       , (3822 >> 8) & 0xFF,  /* t */\
        10, (3848 & 0xFF)       , (3848 >> 8) & 0xFF,  /* u */\
        11, (3900 & 0xFF)       , (3900 >> 8) & 0xFF,  /* v */\
        15, (3952 & 0xFF)       , (3952 >> 8) & 0xFF,  /* w */\
        11, (4004 & 0xFF)       , (4004 >> 8) & 0xFF,  /* x */\
        11, (4056 & 0xFF)       , (4056 >> 8) & 0xFF,  /* y */\
        10, (4108 & 0xFF)       , (4108 >> 8) & 0xFF,  /* z */\
         6, (4160 & 0xFF)       , (4160 >> 8) & 0xFF,  /* { */\
         2, (4186 & 0xFF)       , (4186 >> 8) & 0xFF,  /* | */\
         6, (4212 & 0xFF)       , (4212 >> 8) & 0xFF,  /* } */\
        11, (4238 & 0xFF)       , (4238 >> 8) & 0xFF,  /* ~ */\
}


/** @brief Initialization macro for the bitmaps of the arial_20pt font. */
#define FONTS_ARIAL_20PT_BITMAPS \
{\
\
    /* @0 ' ' (8 pixels wide) */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @26 '!' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @52 '"' (5 pixels wide) */\
    0x00, /*       */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x1B, /* ## ## */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @78 '#' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x30, 0x06, /*     ##   ##  */\
    0x30, 0x06, /*     ##   ##  */\
    0x30, 0x06, /*     ##   ##  */\
    0x18, 0x03, /*    ##   ##   */\
    0x18, 0x03, /*    ##   ##   */\
    0x18, 0x03, /*    ##   ##   */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x18, 0x03, /*    ##   ##   */\
    0x9C, 0x03, /*   ###  ###   */\
    0x8C, 0x01, /*   ##   ##    */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x8C, 0x01, /*   ##   ##    */\
    0x8C, 0x01, /*   ##   ##    */\
    0x8C, 0x01, /*   ##   ##    */\
    0xC6, 0x00, /*  ##   ##     */\
    0xC6, 0x00, /*  ##   ##     */\
    0xC6, 0x00, /*  ##   ##     */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @130 '$' (10 pixels wide) */\
    0x20, 0x00, /*      #     */\
    0xF8, 0x00, /*    #####   */\
    0xFE, 0x01, /*  ########  */\
    0xA6, 0x01, /*  ##  # ##  */\
    0x23, 0x03, /* ##   #  ## */\
    0x23, 0x03, /* ##   #  ## */\
    0x23, 0x00, /* ##   #     */\
    0x27, 0x00, /* ###  #     */\
    0x2E, 0x00, /*  ### #     */\
    0x7C, 0x00, /*   #####    */\
    0xF0, 0x01, /*     #####  */\
    0xA0, 0x01, /*      # ##  */\
    0x20, 0x03, /*      #  ## */\
    0x20, 0x03, /*      #  ## */\
    0x23, 0x03, /* ##   #  ## */\
    0x23, 0x03, /* ##   #  ## */\
    0x27, 0x03, /* ###  #  ## */\
    0xAE, 0x01, /*  ### # ##  */\
    0xFC, 0x01, /*   #######  */\
    0xF8, 0x00, /*    #####   */\
    0x20, 0x00, /*      #     */\
    0x20, 0x00, /*      #     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @182 '%' (17 pixels wide) */\
    0x00, 0x00, 0x00, /*                   */\
    0x1C, 0x30, 0x00, /*   ###       ##    */\
    0x3E, 0x18, 0x00, /*  #####     ##     */\
    0x77, 0x18, 0x00, /* ### ###    ##     */\
    0x63, 0x18, 0x00, /* ##   ##    ##     */\
    0x63, 0x0C, 0x00, /* ##   ##   ##      */\
    0x63, 0x0C, 0x00, /* ##   ##   ##      */\
    0x63, 0x06, 0x00, /* ##   ##  ##       */\
    0x77, 0x06, 0x00, /* ### ###  ##       */\
    0x3E, 0x03, 0x00, /*  #####  ##        */\
    0x1C, 0x73, 0x00, /*   ###   ##  ###   */\
    0x80, 0xF9, 0x00, /*        ##  #####  */\
    0x80, 0xDD, 0x01, /*        ## ### ### */\
    0xC0, 0x8C, 0x01, /*       ##  ##   ## */\
    0xC0, 0x8C, 0x01, /*       ##  ##   ## */\
    0xC0, 0x8C, 0x01, /*       ##  ##   ## */\
    0x60, 0x8C, 0x01, /*      ##   ##   ## */\
    0x60, 0xDC, 0x01, /*      ##   ### ### */\
    0x30, 0xF8, 0x00, /*     ##     #####  */\
    0x30, 0x70, 0x00, /*     ##      ###   */\
    0x00, 0x00, 0x00, /*                   */\
    0x00, 0x00, 0x00, /*                   */\
    0x00, 0x00, 0x00, /*                   */\
    0x00, 0x00, 0x00, /*                   */\
    0x00, 0x00, 0x00, /*                   */\
    0x00, 0x00, 0x00, /*                   */\
\
    /* @260 '&' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0xF0, 0x00, /*     ####      */\
    0xF8, 0x01, /*    ######     */\
    0x9C, 0x03, /*   ###  ###    */\
    0x0C, 0x03, /*   ##    ##    */\
    0x0C, 0x03, /*   ##    ##    */\
    0x0C, 0x03, /*   ##    ##    */\
    0x88, 0x01, /*    #   ##     */\
    0xF8, 0x00, /*    #####      */\
    0x78, 0x00, /*    ####       */\
    0x7C, 0x00, /*   #####       */\
    0x6E, 0x0C, /*  ### ##   ##  */\
    0xC7, 0x0C, /* ###   ##  ##  */\
    0x83, 0x07, /* ##     ####   */\
    0x03, 0x07, /* ##      ###   */\
    0x03, 0x07, /* ##      ###   */\
    0x07, 0x07, /* ###     ###   */\
    0x8E, 0x0F, /*  ###   #####  */\
    0xFC, 0x19, /*   #######  ## */\
    0xF8, 0x08, /*    #####   #  */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @312 ''' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @338 '(' (5 pixels wide) */\
    0x00, /*       */\
    0x18, /*    ## */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x03, /* ##    */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x18, /*    ## */\
    0x00, /*       */\
\
    /* @364 ')' (5 pixels wide) */\
    0x00, /*       */\
    0x03, /* ##    */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x18, /*    ## */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0C, /*   ##  */\
    0x0E, /*  ###  */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x03, /* ##    */\
    0x00, /*       */\
\
    /* @390 '*' (8 pixels wide) */\
    0x00, /*          */\
    0x18, /*    ##    */\
    0x18, /*    ##    */\
    0xDB, /* ## ## ## */\
    0xFF, /* ######## */\
    0x3C, /*   ####   */\
    0x3C, /*   ####   */\
    0x66, /*  ##  ##  */\
    0x24, /*   #  #   */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
    0x00, /*          */\
\
    /* @416 '+' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @468 ',' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x03, /* ## */\
    0x01, /* #  */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @494 '-' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x3F, /* ###### */\
    0x3F, /* ###### */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @520 '.' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @546 '/' (6 pixels wide) */\
    0x00, /*        */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @572 '0' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFC, 0x00, /*   ######   */\
    0xCE, 0x01, /*  ###  ###  */\
    0x86, 0x01, /*  ##    ##  */\
    0x02, 0x03, /*  #      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x01, /* ##      #  */\
    0x86, 0x01, /*  ##    ##  */\
    0xCE, 0x01, /*  ###  ###  */\
    0xFC, 0x00, /*   ######   */\
    0x78, 0x00, /*    ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @624 '1' (6 pixels wide) */\
    0x00, /*        */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x38, /*    ### */\
    0x3C, /*   #### */\
    0x3E, /*  ##### */\
    0x33, /* ##  ## */\
    0x31, /* #   ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @650 '2' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFC, 0x01, /*   #######  */\
    0x86, 0x01, /*  ##    ##  */\
    0x07, 0x03, /* ###     ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x00, 0x03, /*         ## */\
    0x80, 0x03, /*        ### */\
    0x80, 0x01, /*        ##  */\
    0xC0, 0x01, /*       ###  */\
    0xE0, 0x00, /*      ###   */\
    0x70, 0x00, /*     ###    */\
    0x30, 0x00, /*     ##     */\
    0x38, 0x00, /*    ###     */\
    0x1C, 0x00, /*   ###      */\
    0x0E, 0x00, /*  ###       */\
    0x06, 0x00, /*  ##        */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @702 '3' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFE, 0x00, /*  #######   */\
    0xC6, 0x01, /*  ##   ###  */\
    0x83, 0x01, /* ##     ##  */\
    0x83, 0x01, /* ##     ##  */\
    0x80, 0x01, /*        ##  */\
    0x80, 0x01, /*        ##  */\
    0xC0, 0x00, /*       ##   */\
    0x70, 0x00, /*     ###    */\
    0xF0, 0x01, /*     #####  */\
    0x80, 0x01, /*        ##  */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x03, 0x03, /* ##      ## */\
    0x83, 0x03, /* ##     ### */\
    0xC6, 0x01, /*  ##   ###  */\
    0xFE, 0x00, /*  #######   */\
    0x78, 0x00, /*    ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @754 '4' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x80, 0x01, /*        ##   */\
    0xC0, 0x01, /*       ###   */\
    0xC0, 0x01, /*       ###   */\
    0xE0, 0x01, /*      ####   */\
    0xB0, 0x01, /*     ## ##   */\
    0xB0, 0x01, /*     ## ##   */\
    0x98, 0x01, /*    ##  ##   */\
    0x98, 0x01, /*    ##  ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8E, 0x01, /*  ###   ##   */\
    0x86, 0x01, /*  ##    ##   */\
    0x83, 0x01, /* ##     ##   */\
    0xFF, 0x07, /* ########### */\
    0xFF, 0x07, /* ########### */\
    0x80, 0x01, /*        ##   */\
    0x80, 0x01, /*        ##   */\
    0x80, 0x01, /*        ##   */\
    0x80, 0x01, /*        ##   */\
    0x80, 0x01, /*        ##   */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @806 '5' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0xFC, 0x01, /*   #######  */\
    0xFC, 0x01, /*   #######  */\
    0x0E, 0x00, /*  ###       */\
    0x06, 0x00, /*  ##        */\
    0x06, 0x00, /*  ##        */\
    0x06, 0x00, /*  ##        */\
    0x7E, 0x00, /*  ######    */\
    0xFF, 0x00, /* ########   */\
    0xC7, 0x01, /* ###   ###  */\
    0x83, 0x03, /* ##     ### */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x03, 0x03, /* ##      ## */\
    0x83, 0x01, /* ##     ##  */\
    0xC6, 0x01, /*  ##   ###  */\
    0xFE, 0x00, /*  #######   */\
    0x78, 0x00, /*    ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @858 '6' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0xF8, 0x00, /*    #####   */\
    0xFC, 0x01, /*   #######  */\
    0x8E, 0x01, /*  ###   ##  */\
    0x06, 0x03, /*  ##     ## */\
    0x02, 0x03, /*  #      ## */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x73, 0x00, /* ##  ###    */\
    0xFF, 0x01, /* #########  */\
    0x87, 0x01, /* ###    ##  */\
    0x87, 0x03, /* ###    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x86, 0x03, /*  ##    ### */\
    0x8E, 0x01, /*  ###   ##  */\
    0xFC, 0x00, /*   ######   */\
    0x78, 0x00, /*    ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @910 '7' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x03, /*         ## */\
    0x80, 0x01, /*        ##  */\
    0xC0, 0x00, /*       ##   */\
    0xC0, 0x00, /*       ##   */\
    0x60, 0x00, /*      ##    */\
    0x60, 0x00, /*      ##    */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x18, 0x00, /*    ##      */\
    0x18, 0x00, /*    ##      */\
    0x18, 0x00, /*    ##      */\
    0x18, 0x00, /*    ##      */\
    0x0C, 0x00, /*   ##       */\
    0x0C, 0x00, /*   ##       */\
    0x0C, 0x00, /*   ##       */\
    0x0C, 0x00, /*   ##       */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @962 '8' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFC, 0x00, /*   ######   */\
    0xCE, 0x01, /*  ###  ###  */\
    0x86, 0x01, /*  ##    ##  */\
    0x86, 0x01, /*  ##    ##  */\
    0x86, 0x01, /*  ##    ##  */\
    0x86, 0x01, /*  ##    ##  */\
    0xCC, 0x00, /*   ##  ##   */\
    0x78, 0x00, /*    ####    */\
    0xFC, 0x00, /*   ######   */\
    0x86, 0x01, /*  ##    ##  */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x86, 0x01, /*  ##    ##  */\
    0xFE, 0x01, /*  ########  */\
    0x78, 0x00, /*    ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1014 '9' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFC, 0x00, /*   ######   */\
    0x8E, 0x01, /*  ###   ##  */\
    0x87, 0x01, /* ###    ##  */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x87, 0x03, /* ###    ### */\
    0x86, 0x03, /*  ##    ### */\
    0xFE, 0x03, /*  ######### */\
    0x38, 0x03, /*    ###  ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x83, 0x01, /* ##     ##  */\
    0x83, 0x01, /* ##     ##  */\
    0xC6, 0x01, /*  ##   ###  */\
    0xFE, 0x00, /*  #######   */\
    0x3C, 0x00, /*   ####     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1066 ':' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @1092 ';' (2 pixels wide) */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x02, /*  # */\
    0x02, /*  # */\
    0x03, /* ## */\
    0x01, /* #  */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @1118 '<' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x02, /*          # */\
    0x80, 0x03, /*        ### */\
    0xC0, 0x01, /*       ###  */\
    0x70, 0x00, /*     ###    */\
    0x3C, 0x00, /*   ####     */\
    0x0F, 0x00, /* ####       */\
    0x03, 0x00, /* ##         */\
    0x0F, 0x00, /* ####       */\
    0x3C, 0x00, /*   ####     */\
    0x70, 0x00, /*     ###    */\
    0xC0, 0x01, /*       ###  */\
    0x80, 0x03, /*        ### */\
    0x00, 0x02, /*          # */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1170 '=' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1222 '>' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x01, 0x00, /* #          */\
    0x07, 0x00, /* ###        */\
    0x0E, 0x00, /*  ###       */\
    0x38, 0x00, /*    ###     */\
    0xF0, 0x00, /*     ####   */\
    0xC0, 0x03, /*       #### */\
    0x00, 0x03, /*         ## */\
    0xC0, 0x03, /*       #### */\
    0xF0, 0x00, /*     ####   */\
    0x38, 0x00, /*    ###     */\
    0x0E, 0x00, /*  ###       */\
    0x07, 0x00, /* ###        */\
    0x01, 0x00, /* #          */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1274 '?' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFC, 0x01, /*   #######  */\
    0x8E, 0x01, /*  ###   ##  */\
    0x86, 0x03, /*  ##    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x00, 0x03, /*         ## */\
    0x80, 0x03, /*        ### */\
    0x80, 0x01, /*        ##  */\
    0xC0, 0x00, /*       ##   */\
    0xE0, 0x00, /*      ###   */\
    0x70, 0x00, /*     ###    */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x30, 0x00, /*     ##     */\
    0x30, 0x00, /*     ##     */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @1326 '@' (21 pixels wide) */\
    0x00, 0x00, 0x00, /*                       */\
    0x80, 0x3F, 0x00, /*        #######        */\
    0xE0, 0xFF, 0x00, /*      ###########      */\
    0xF0, 0xE0, 0x01, /*     ####     ####     */\
    0x38, 0x80, 0x03, /*    ###         ###    */\
    0x1C, 0x00, 0x07, /*   ###           ###   */\
    0x0C, 0x67, 0x06, /*   ##    ###  ##  ##   */\
    0xCE, 0x6F, 0x06, /*  ###  ###### ##  ##   */\
    0xC6, 0x78, 0x0C, /*  ##   ##   ####   ##  */\
    0x66, 0x30, 0x0C, /*  ##  ##     ##    ##  */\
    0x63, 0x30, 0x0C, /* ##   ##     ##    ##  */\
    0x33, 0x30, 0x0C, /* ##  ##      ##    ##  */\
    0x33, 0x30, 0x0C, /* ##  ##      ##    ##  */\
    0x33, 0x30, 0x0C, /* ##  ##      ##    ##  */\
    0x33, 0x30, 0x06, /* ##  ##      ##   ##   */\
    0x33, 0x18, 0x06, /* ##  ##     ##    ##   */\
    0x33, 0x18, 0x03, /* ##  ##     ##   ##    */\
    0x63, 0x9C, 0x03, /* ##   ##   ###  ###    */\
    0xE6, 0xFF, 0x01, /*  ##  ############     */\
    0xC6, 0x71, 0x00, /*  ##   ###   ###       */\
    0x0E, 0x00, 0x18, /*  ###               ## */\
    0x1C, 0x00, 0x0C, /*   ###             ##  */\
    0x38, 0x00, 0x06, /*    ###           ##   */\
    0xF0, 0x80, 0x03, /*     ####       ###    */\
    0xE0, 0xFF, 0x01, /*      ############     */\
    0x80, 0x7F, 0x00, /*        ########       */\
\
    /* @1404 'A' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0xC0, 0x01, /*       ###       */\
    0xC0, 0x01, /*       ###       */\
    0x60, 0x03, /*      ## ##      */\
    0x60, 0x03, /*      ## ##      */\
    0x60, 0x03, /*      ## ##      */\
    0x70, 0x06, /*     ###  ##     */\
    0x30, 0x06, /*     ##   ##     */\
    0x30, 0x06, /*     ##   ##     */\
    0x18, 0x0C, /*    ##     ##    */\
    0x18, 0x0C, /*    ##     ##    */\
    0x18, 0x0C, /*    ##     ##    */\
    0xFC, 0x1F, /*   ###########   */\
    0xFC, 0x1F, /*   ###########   */\
    0x0C, 0x18, /*   ##       ##   */\
    0x06, 0x30, /*  ##         ##  */\
    0x06, 0x30, /*  ##         ##  */\
    0x06, 0x30, /*  ##         ##  */\
    0x07, 0x70, /* ###         ### */\
    0x03, 0x60, /* ##           ## */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
\
    /* @1456 'B' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0xFF, 0x01, /* #########     */\
    0xFF, 0x07, /* ###########   */\
    0x03, 0x06, /* ##       ##   */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x06, /* ##       ##   */\
    0xFF, 0x03, /* ##########    */\
    0xFF, 0x0F, /* ############  */\
    0x03, 0x0E, /* ##       ###  */\
    0x03, 0x1C, /* ##        ### */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x1C, /* ##        ### */\
    0x03, 0x0E, /* ##       ###  */\
    0xFF, 0x07, /* ###########   */\
    0xFF, 0x03, /* ##########    */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @1508 'C' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0xE0, 0x03, /*      #####     */\
    0xF8, 0x0F, /*    #########   */\
    0x1C, 0x1C, /*   ###     ###  */\
    0x0E, 0x18, /*  ###       ##  */\
    0x06, 0x38, /*  ##        ### */\
    0x06, 0x10, /*  ##         #  */\
    0x03, 0x00, /* ##             */\
    0x03, 0x00, /* ##             */\
    0x03, 0x00, /* ##             */\
    0x03, 0x00, /* ##             */\
    0x03, 0x00, /* ##             */\
    0x03, 0x00, /* ##             */\
    0x03, 0x10, /* ##          #  */\
    0x06, 0x30, /*  ##         ## */\
    0x06, 0x38, /*  ##        ### */\
    0x0E, 0x18, /*  ###       ##  */\
    0x1C, 0x0E, /*   ###    ###   */\
    0xF8, 0x0F, /*    #########   */\
    0xE0, 0x03, /*      #####     */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @1560 'D' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0xFF, 0x01, /* #########     */\
    0xFF, 0x03, /* ##########    */\
    0x03, 0x07, /* ##      ###   */\
    0x03, 0x0E, /* ##       ###  */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x1C, /* ##        ### */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x0E, /* ##       ###  */\
    0x03, 0x07, /* ##      ###   */\
    0xFF, 0x03, /* ##########    */\
    0xFF, 0x01, /* #########     */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @1612 'E' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0xFF, 0x07, /* ###########  */\
    0xFF, 0x07, /* ###########  */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @1664 'F' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0xFF, 0x07, /* ########### */\
    0xFF, 0x07, /* ########### */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0xFF, 0x03, /* ##########  */\
    0xFF, 0x03, /* ##########  */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x03, 0x00, /* ##          */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @1716 'G' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0xE0, 0x07, /*      ######     */\
    0xF8, 0x1F, /*    ##########   */\
    0x3C, 0x3C, /*   ####    ####  */\
    0x0C, 0x70, /*   ##        ### */\
    0x06, 0x70, /*  ##         ### */\
    0x06, 0x20, /*  ##          #  */\
    0x03, 0x00, /* ##              */\
    0x03, 0x00, /* ##              */\
    0x03, 0x00, /* ##              */\
    0x03, 0x7F, /* ##      ####### */\
    0x03, 0x7F, /* ##      ####### */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x06, 0x60, /*  ##          ## */\
    0x06, 0x60, /*  ##          ## */\
    0x0C, 0x70, /*   ##        ### */\
    0x3C, 0x78, /*   ####     #### */\
    0xF8, 0x1F, /*    ##########   */\
    0xE0, 0x07, /*      ######     */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
\
    /* @1768 'H' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @1820 'I' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @1846 'J' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0xC6, 0x00, /*  ##   ##  */\
    0xFE, 0x00, /*  #######  */\
    0x7C, 0x00, /*   #####   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @1898 'K' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x03, 0x06, /* ##       ##  */\
    0x03, 0x06, /* ##       ##  */\
    0x03, 0x03, /* ##      ##   */\
    0x83, 0x01, /* ##     ##    */\
    0xC3, 0x00, /* ##    ##     */\
    0x63, 0x00, /* ##   ##      */\
    0x63, 0x00, /* ##   ##      */\
    0x33, 0x00, /* ##  ##       */\
    0x3B, 0x00, /* ## ###       */\
    0x6F, 0x00, /* #### ##      */\
    0x6F, 0x00, /* #### ##      */\
    0xC7, 0x00, /* ###   ##     */\
    0xC3, 0x00, /* ##    ##     */\
    0x83, 0x01, /* ##     ##    */\
    0x83, 0x01, /* ##     ##    */\
    0x03, 0x03, /* ##      ##   */\
    0x03, 0x06, /* ##       ##  */\
    0x03, 0x06, /* ##       ##  */\
    0x03, 0x0C, /* ##        ## */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @1950 'L' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @2002 'M' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0x07, 0x70, /* ###         ### */\
    0x0F, 0x70, /* ####        ### */\
    0x0F, 0x78, /* ####       #### */\
    0x0F, 0x78, /* ####       #### */\
    0x0F, 0x78, /* ####       #### */\
    0x1B, 0x78, /* ## ##      #### */\
    0x1B, 0x6C, /* ## ##     ## ## */\
    0x1B, 0x6C, /* ## ##     ## ## */\
    0x1B, 0x6C, /* ## ##     ## ## */\
    0x33, 0x66, /* ##  ##   ##  ## */\
    0x33, 0x66, /* ##  ##   ##  ## */\
    0x33, 0x66, /* ##  ##   ##  ## */\
    0x73, 0x66, /* ##  ###  ##  ## */\
    0x63, 0x63, /* ##   ## ##   ## */\
    0x63, 0x63, /* ##   ## ##   ## */\
    0x63, 0x63, /* ##   ## ##   ## */\
    0xC3, 0x61, /* ##    ###    ## */\
    0xC3, 0x61, /* ##    ###    ## */\
    0xC3, 0x61, /* ##    ###    ## */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
\
    /* @2054 'N' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x03, 0x0C, /* ##        ## */\
    0x07, 0x0C, /* ###       ## */\
    0x07, 0x0C, /* ###       ## */\
    0x0F, 0x0C, /* ####      ## */\
    0x0F, 0x0C, /* ####      ## */\
    0x1B, 0x0C, /* ## ##     ## */\
    0x1B, 0x0C, /* ## ##     ## */\
    0x33, 0x0C, /* ##  ##    ## */\
    0x33, 0x0C, /* ##  ##    ## */\
    0x63, 0x0C, /* ##   ##   ## */\
    0x43, 0x0C, /* ##    #   ## */\
    0xC3, 0x0C, /* ##    ##  ## */\
    0x83, 0x0D, /* ##     ## ## */\
    0x83, 0x0D, /* ##     ## ## */\
    0x03, 0x0F, /* ##      #### */\
    0x03, 0x0F, /* ##      #### */\
    0x03, 0x0E, /* ##       ### */\
    0x03, 0x0E, /* ##       ### */\
    0x03, 0x0C, /* ##        ## */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @2106 'O' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0xE0, 0x03, /*      #####      */\
    0xF8, 0x0F, /*    #########    */\
    0x1C, 0x1C, /*   ###     ###   */\
    0x0E, 0x38, /*  ###       ###  */\
    0x06, 0x30, /*  ##         ##  */\
    0x06, 0x30, /*  ##         ##  */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x06, 0x30, /*  ##         ##  */\
    0x06, 0x30, /*  ##         ##  */\
    0x0C, 0x38, /*   ##       ###  */\
    0x1C, 0x1C, /*   ###     ###   */\
    0xF8, 0x0F, /*    #########    */\
    0xE0, 0x03, /*      #####      */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
\
    /* @2158 'P' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0xFF, 0x01, /* #########    */\
    0xFF, 0x07, /* ###########  */\
    0x03, 0x06, /* ##       ##  */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x06, /* ##       ##  */\
    0xFF, 0x07, /* ###########  */\
    0xFF, 0x01, /* #########    */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x03, 0x00, /* ##           */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @2210 'Q' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0xE0, 0x03, /*      #####      */\
    0xF8, 0x0F, /*    #########    */\
    0x1C, 0x1C, /*   ###     ###   */\
    0x0E, 0x38, /*  ###       ###  */\
    0x06, 0x30, /*  ##         ##  */\
    0x06, 0x30, /*  ##         ##  */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x03, 0x60, /* ##           ## */\
    0x06, 0x30, /*  ##         ##  */\
    0x06, 0x33, /*  ##     ##  ##  */\
    0x0C, 0x3F, /*   ##    ######  */\
    0x1C, 0x1C, /*   ###     ###   */\
    0xF8, 0x3F, /*    ###########  */\
    0xE0, 0x73, /*      #####  ### */\
    0x00, 0x60, /*              ## */\
    0x00, 0x40, /*               # */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
\
    /* @2262 'R' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0xFF, 0x03, /* ##########    */\
    0xFF, 0x07, /* ###########   */\
    0x03, 0x06, /* ##       ##   */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x06, /* ##       ##   */\
    0xFF, 0x03, /* ##########    */\
    0xFF, 0x00, /* ########      */\
    0x83, 0x01, /* ##     ##     */\
    0x03, 0x03, /* ##      ##    */\
    0x03, 0x07, /* ##      ###   */\
    0x03, 0x06, /* ##       ##   */\
    0x03, 0x06, /* ##       ##   */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x0C, /* ##        ##  */\
    0x03, 0x18, /* ##         ## */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @2314 'S' (13 pixels wide) */\
    0x00, 0x00, /*               */\
    0xF0, 0x01, /*     #####     */\
    0xFC, 0x07, /*   #########   */\
    0x0C, 0x06, /*   ##     ##   */\
    0x06, 0x0C, /*  ##       ##  */\
    0x06, 0x0C, /*  ##       ##  */\
    0x06, 0x00, /*  ##           */\
    0x06, 0x00, /*  ##           */\
    0x0C, 0x00, /*   ##          */\
    0xFC, 0x00, /*   ######      */\
    0xF0, 0x07, /*     #######   */\
    0x80, 0x0F, /*        #####  */\
    0x00, 0x1C, /*           ### */\
    0x00, 0x18, /*            ## */\
    0x03, 0x18, /* ##         ## */\
    0x03, 0x18, /* ##         ## */\
    0x06, 0x1C, /*  ##       ### */\
    0x0E, 0x0E, /*  ###     ###  */\
    0xFC, 0x07, /*   #########   */\
    0xF0, 0x03, /*     ######    */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
    0x00, 0x00, /*               */\
\
    /* @2366 'T' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x60, 0x00, /*      ##      */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @2418 'U' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x03, 0x0C, /* ##        ## */\
    0x06, 0x06, /*  ##      ##  */\
    0x0E, 0x07, /*  ###    ###  */\
    0xFC, 0x03, /*   ########   */\
    0xF8, 0x01, /*    ######    */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @2470 'V' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0x03, 0x60, /* ##           ## */\
    0x07, 0x70, /* ###         ### */\
    0x06, 0x30, /*  ##         ##  */\
    0x06, 0x30, /*  ##         ##  */\
    0x0E, 0x38, /*  ###       ###  */\
    0x0C, 0x18, /*   ##       ##   */\
    0x0C, 0x18, /*   ##       ##   */\
    0x1C, 0x1C, /*   ###     ###   */\
    0x18, 0x0C, /*    ##     ##    */\
    0x18, 0x0C, /*    ##     ##    */\
    0x38, 0x0E, /*    ###   ###    */\
    0x30, 0x06, /*     ##   ##     */\
    0x30, 0x06, /*     ##   ##     */\
    0x70, 0x07, /*     ### ###     */\
    0x60, 0x03, /*      ## ##      */\
    0x60, 0x03, /*      ## ##      */\
    0x60, 0x03, /*      ## ##      */\
    0xC0, 0x01, /*       ###       */\
    0xC0, 0x01, /*       ###       */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
\
    /* @2522 'W' (21 pixels wide) */\
    0x00, 0x00, 0x00, /*                       */\
    0x03, 0x0E, 0x18, /* ##       ###       ## */\
    0x06, 0x0E, 0x18, /*  ##      ###       ## */\
    0x06, 0x1B, 0x0C, /*  ##     ## ##     ##  */\
    0x06, 0x1B, 0x0C, /*  ##     ## ##     ##  */\
    0x06, 0x1B, 0x0C, /*  ##     ## ##     ##  */\
    0x06, 0x1B, 0x0C, /*  ##     ## ##     ##  */\
    0x0C, 0x3B, 0x0E, /*   ##    ## ###   ###  */\
    0x8C, 0x31, 0x06, /*   ##   ##   ##   ##   */\
    0x8C, 0x31, 0x06, /*   ##   ##   ##   ##   */\
    0x8C, 0x31, 0x06, /*   ##   ##   ##   ##   */\
    0x8C, 0x31, 0x06, /*   ##   ##   ##   ##   */\
    0xD8, 0x71, 0x07, /*    ## ###   ### ###   */\
    0xD8, 0x60, 0x03, /*    ## ##     ## ##    */\
    0xD8, 0x60, 0x03, /*    ## ##     ## ##    */\
    0xD8, 0x60, 0x03, /*    ## ##     ## ##    */\
    0xD8, 0x60, 0x03, /*    ## ##     ## ##    */\
    0x78, 0xC0, 0x01, /*    ####       ###     */\
    0x70, 0xC0, 0x01, /*     ###       ###     */\
    0x70, 0xC0, 0x01, /*     ###       ###     */\
    0x00, 0x00, 0x00, /*                       */\
    0x00, 0x00, 0x00, /*                       */\
    0x00, 0x00, 0x00, /*                       */\
    0x00, 0x00, 0x00, /*                       */\
    0x00, 0x00, 0x00, /*                       */\
    0x00, 0x00, 0x00, /*                       */\
\
    /* @2600 'X' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0x06, 0x30, /*  ##         ##  */\
    0x0C, 0x18, /*   ##       ##   */\
    0x18, 0x0C, /*    ##     ##    */\
    0x18, 0x0C, /*    ##     ##    */\
    0x30, 0x06, /*     ##   ##     */\
    0x70, 0x06, /*     ###  ##     */\
    0x60, 0x03, /*      ## ##      */\
    0xC0, 0x01, /*       ###       */\
    0xC0, 0x01, /*       ###       */\
    0xC0, 0x01, /*       ###       */\
    0x60, 0x03, /*      ## ##      */\
    0x60, 0x03, /*      ## ##      */\
    0x30, 0x06, /*     ##   ##     */\
    0x18, 0x0C, /*    ##     ##    */\
    0x18, 0x0C, /*    ##     ##    */\
    0x0C, 0x18, /*   ##       ##   */\
    0x0C, 0x18, /*   ##       ##   */\
    0x06, 0x30, /*  ##         ##  */\
    0x03, 0x60, /* ##           ## */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
\
    /* @2652 'Y' (14 pixels wide) */\
    0x00, 0x00, /*                */\
    0x03, 0x30, /* ##          ## */\
    0x06, 0x18, /*  ##        ##  */\
    0x06, 0x18, /*  ##        ##  */\
    0x0C, 0x0C, /*   ##      ##   */\
    0x1C, 0x0E, /*   ###    ###   */\
    0x18, 0x06, /*    ##    ##    */\
    0x30, 0x03, /*     ##  ##     */\
    0x30, 0x03, /*     ##  ##     */\
    0xE0, 0x01, /*      ####      */\
    0xE0, 0x01, /*      ####      */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0xC0, 0x00, /*       ##       */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
    0x00, 0x00, /*                */\
\
    /* @2704 'Z' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0xFE, 0x0F, /*  ########### */\
    0xFE, 0x0F, /*  ########### */\
    0x00, 0x0C, /*           ## */\
    0x00, 0x06, /*          ##  */\
    0x00, 0x03, /*         ##   */\
    0x00, 0x03, /*         ##   */\
    0x80, 0x01, /*        ##    */\
    0xC0, 0x00, /*       ##     */\
    0xC0, 0x00, /*       ##     */\
    0x60, 0x00, /*      ##      */\
    0x30, 0x00, /*     ##       */\
    0x30, 0x00, /*     ##       */\
    0x18, 0x00, /*    ##        */\
    0x0C, 0x00, /*   ##         */\
    0x0C, 0x00, /*   ##         */\
    0x06, 0x00, /*  ##          */\
    0x03, 0x00, /* ##           */\
    0xFF, 0x0F, /* ############ */\
    0xFF, 0x0F, /* ############ */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @2756 '[' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x03, /* ##   */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x00, /*      */\
\
    /* @2782 '\' (6 pixels wide) */\
    0x00, /*        */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x06, /*  ##    */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x18, /*    ##  */\
    0x30, /*     ## */\
    0x30, /*     ## */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @2808 ']' (4 pixels wide) */\
    0x00, /*      */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0F, /* #### */\
    0x0F, /* #### */\
    0x00, /*      */\
\
    /* @2834 '^' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x38, 0x00, /*    ###    */\
    0x38, 0x00, /*    ###    */\
    0x7C, 0x00, /*   #####   */\
    0x6C, 0x00, /*   ## ##   */\
    0x6C, 0x00, /*   ## ##   */\
    0xEE, 0x00, /*  ### ###  */\
    0xC6, 0x00, /*  ##   ##  */\
    0xC6, 0x00, /*  ##   ##  */\
    0xC7, 0x01, /* ###   ### */\
    0x83, 0x01, /* ##     ## */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @2886 '_' (12 pixels wide) */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0xFF, 0x0F, /* ############ */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
    0x00, 0x00, /*              */\
\
    /* @2938 '`' (3 pixels wide) */\
    0x03, /* ##  */\
    0x02, /*  #  */\
    0x06, /*  ## */\
    0x04, /*   # */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
    0x00, /*     */\
\
    /* @2964 'a' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xF8, 0x00, /*    #####   */\
    0xFE, 0x01, /*  ########  */\
    0x87, 0x03, /* ###    ### */\
    0x03, 0x03, /* ##      ## */\
    0x00, 0x03, /*         ## */\
    0x80, 0x03, /*        ### */\
    0xFC, 0x03, /*   ######## */\
    0x7E, 0x03, /*  ###### ## */\
    0x07, 0x03, /* ###     ## */\
    0x03, 0x03, /* ##      ## */\
    0x83, 0x03, /* ##     ### */\
    0xC7, 0x03, /* ###   #### */\
    0x7E, 0x03, /*  ###### ## */\
    0x3C, 0x03, /*   ####  ## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @3016 'b' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x7B, 0x00, /* ## ####    */\
    0xFF, 0x00, /* ########   */\
    0xCF, 0x01, /* ####  ###  */\
    0x87, 0x01, /* ###    ##  */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x87, 0x01, /* ###    ##  */\
    0xCF, 0x01, /* ####  ###  */\
    0xFF, 0x00, /* ########   */\
    0x7B, 0x00, /* ## ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @3068 'c' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x78, 0x00, /*    ####   */\
    0xFC, 0x00, /*   ######  */\
    0xC6, 0x01, /*  ##   ### */\
    0x87, 0x01, /* ###    ## */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x83, 0x01, /* ##     ## */\
    0x87, 0x01, /* ###    ## */\
    0xC6, 0x00, /*  ##   ##  */\
    0xFC, 0x00, /*   ######  */\
    0x78, 0x00, /*    ####   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @3120 'd' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x78, 0x03, /*    #### ## */\
    0xFC, 0x03, /*   ######## */\
    0xCE, 0x03, /*  ###  #### */\
    0x86, 0x03, /*  ##    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x86, 0x03, /*  ##    ### */\
    0xCE, 0x03, /*  ###  #### */\
    0xFC, 0x03, /*   ######## */\
    0x78, 0x03, /*    #### ## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @3172 'e' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFC, 0x00, /*   ######   */\
    0x8E, 0x01, /*  ###   ##  */\
    0x86, 0x01, /*  ##    ##  */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x07, 0x03, /* ###     ## */\
    0x8E, 0x01, /*  ###   ##  */\
    0xFC, 0x01, /*   #######  */\
    0x78, 0x00, /*    ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @3224 'f' (7 pixels wide) */\
    0x00, /*         */\
    0x78, /*    #### */\
    0x7C, /*   ##### */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x3F, /* ######  */\
    0x3F, /* ######  */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x0C, /*   ##    */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
    0x00, /*         */\
\
    /* @3250 'g' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x78, 0x03, /*    #### ## */\
    0xFC, 0x03, /*   ######## */\
    0xCE, 0x03, /*  ###  #### */\
    0x86, 0x03, /*  ##    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x86, 0x03, /*  ##    ### */\
    0xCE, 0x03, /*  ###  #### */\
    0xFC, 0x03, /*   ######## */\
    0x78, 0x03, /*    #### ## */\
    0x00, 0x03, /*         ## */\
    0x03, 0x03, /* ##      ## */\
    0x87, 0x01, /* ###    ##  */\
    0xFE, 0x01, /*  ########  */\
    0x7C, 0x00, /*   #####    */\
    0x00, 0x00, /*            */\
\
    /* @3302 'h' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0xF3, 0x00, /* ##  ####   */\
    0xFF, 0x01, /* #########  */\
    0x8F, 0x03, /* ####   ### */\
    0x07, 0x03, /* ###     ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @3354 'i' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @3380 'j' (4 pixels wide) */\
    0x00, /*      */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x00, /*      */\
    0x00, /*      */\
    0x00, /*      */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0C, /*   ## */\
    0x0F, /* #### */\
    0x07, /* ###  */\
    0x00, /*      */\
\
    /* @3406 'k' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x03, 0x00, /* ##        */\
    0x83, 0x01, /* ##     ## */\
    0xC3, 0x00, /* ##    ##  */\
    0x63, 0x00, /* ##   ##   */\
    0x33, 0x00, /* ##  ##    */\
    0x1B, 0x00, /* ## ##     */\
    0x1F, 0x00, /* #####     */\
    0x1F, 0x00, /* #####     */\
    0x37, 0x00, /* ### ##    */\
    0x63, 0x00, /* ##   ##   */\
    0x63, 0x00, /* ##   ##   */\
    0xC3, 0x00, /* ##    ##  */\
    0xC3, 0x00, /* ##    ##  */\
    0x83, 0x01, /* ##     ## */\
    0x83, 0x01, /* ##     ## */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @3458 'l' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
    0x00, /*    */\
\
    /* @3484 'm' (16 pixels wide) */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
    0x73, 0x3C, /* ##  ###   ####   */\
    0xFF, 0x7E, /* ######## ######  */\
    0xC7, 0xE3, /* ###   ####   ### */\
    0x87, 0xC1, /* ###    ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x83, 0xC1, /* ##     ##     ## */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
    0x00, 0x00, /*                  */\
\
    /* @3536 'n' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xF3, 0x00, /* ##  ####   */\
    0xFB, 0x01, /* ## ######  */\
    0x8F, 0x03, /* ####   ### */\
    0x07, 0x03, /* ###     ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @3588 'o' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x78, 0x00, /*    ####    */\
    0xFC, 0x00, /*   ######   */\
    0xCE, 0x01, /*  ###  ###  */\
    0x87, 0x03, /* ###    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x86, 0x03, /*  ##    ### */\
    0xCE, 0x01, /*  ###  ###  */\
    0xFC, 0x00, /*   ######   */\
    0x78, 0x00, /*    ####    */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @3640 'p' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x7B, 0x00, /* ## ####    */\
    0xFF, 0x00, /* ########   */\
    0xCF, 0x01, /* ####  ###  */\
    0x87, 0x03, /* ###    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x87, 0x01, /* ###    ##  */\
    0xCF, 0x01, /* ####  ###  */\
    0xFF, 0x00, /* ########   */\
    0x7B, 0x00, /* ## ####    */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x03, 0x00, /* ##         */\
    0x00, 0x00, /*            */\
\
    /* @3692 'q' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x78, 0x03, /*    #### ## */\
    0xFC, 0x03, /*   ######## */\
    0xCE, 0x03, /*  ###  #### */\
    0x86, 0x03, /*  ##    ### */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x86, 0x03, /*  ##    ### */\
    0xCE, 0x03, /*  ###  #### */\
    0xFC, 0x03, /*   ######## */\
    0x78, 0x03, /*    #### ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x03, /*         ## */\
    0x00, 0x00, /*            */\
\
    /* @3744 'r' (6 pixels wide) */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x1B, /* ## ##  */\
    0x3F, /* ###### */\
    0x07, /* ###    */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x03, /* ##     */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
    0x00, /*        */\
\
    /* @3770 's' (9 pixels wide) */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x7C, 0x00, /*   #####   */\
    0xFE, 0x00, /*  #######  */\
    0xC7, 0x01, /* ###   ### */\
    0x83, 0x01, /* ##     ## */\
    0x03, 0x00, /* ##        */\
    0x0E, 0x00, /*  ###      */\
    0x7C, 0x00, /*   #####   */\
    0xF0, 0x00, /*     ####  */\
    0x80, 0x01, /*        ## */\
    0x80, 0x01, /*        ## */\
    0x83, 0x01, /* ##     ## */\
    0xC7, 0x01, /* ###   ### */\
    0xFE, 0x00, /*  #######  */\
    0x7C, 0x00, /*   #####   */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
    0x00, 0x00, /*           */\
\
    /* @3822 't' (5 pixels wide) */\
    0x00, /*       */\
    0x04, /*   #   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x1F, /* ##### */\
    0x1F, /* ##### */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x06, /*  ##   */\
    0x1E, /*  #### */\
    0x1C, /*   ### */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
    0x00, /*       */\
\
    /* @3848 'u' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x03, 0x03, /* ##      ## */\
    0x83, 0x03, /* ##     ### */\
    0xC7, 0x03, /* ###   #### */\
    0x7E, 0x03, /*  ###### ## */\
    0x3C, 0x03, /*   ####  ## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @3900 'v' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x03, 0x06, /* ##       ## */\
    0x03, 0x06, /* ##       ## */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0xDC, 0x01, /*   ### ###   */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @3952 'w' (15 pixels wide) */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0xC3, 0x61, /* ##    ###    ## */\
    0xC3, 0x61, /* ##    ###    ## */\
    0xC6, 0x31, /*  ##   ###   ##  */\
    0x46, 0x31, /*  ##   # #   ##  */\
    0x66, 0x33, /*  ##  ## ##  ##  */\
    0x66, 0x33, /*  ##  ## ##  ##  */\
    0x66, 0x33, /*  ##  ## ##  ##  */\
    0x6C, 0x1B, /*   ## ## ## ##   */\
    0x6C, 0x1B, /*   ## ## ## ##   */\
    0x6C, 0x1B, /*   ## ## ## ##   */\
    0x2C, 0x1A, /*   ## #   # ##   */\
    0x38, 0x0E, /*    ###   ###    */\
    0x38, 0x0E, /*    ###   ###    */\
    0x38, 0x0E, /*    ###   ###    */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
    0x00, 0x00, /*                 */\
\
    /* @4004 'x' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x03, 0x06, /* ##       ## */\
    0x06, 0x03, /*  ##     ##  */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0xD8, 0x00, /*    ## ##    */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0xD8, 0x00, /*    ## ##    */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0x06, 0x03, /*  ##     ##  */\
    0x03, 0x06, /* ##       ## */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
\
    /* @4056 'y' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x03, 0x06, /* ##       ## */\
    0x07, 0x07, /* ###     ### */\
    0x06, 0x03, /*  ##     ##  */\
    0x06, 0x03, /*  ##     ##  */\
    0x8E, 0x03, /*  ###   ###  */\
    0x8C, 0x01, /*   ##   ##   */\
    0x8C, 0x01, /*   ##   ##   */\
    0xDC, 0x01, /*   ### ###   */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0xD8, 0x00, /*    ## ##    */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0x70, 0x00, /*     ###     */\
    0x30, 0x00, /*     ##      */\
    0x30, 0x00, /*     ##      */\
    0x18, 0x00, /*    ##       */\
    0x1E, 0x00, /*  ####       */\
    0x0E, 0x00, /*  ###        */\
    0x00, 0x00, /*             */\
\
    /* @4108 'z' (10 pixels wide) */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x80, 0x01, /*        ##  */\
    0x80, 0x01, /*        ##  */\
    0xC0, 0x00, /*       ##   */\
    0x60, 0x00, /*      ##    */\
    0x30, 0x00, /*     ##     */\
    0x38, 0x00, /*    ###     */\
    0x18, 0x00, /*    ##      */\
    0x0C, 0x00, /*   ##       */\
    0x06, 0x00, /*  ##        */\
    0x03, 0x00, /* ##         */\
    0xFF, 0x03, /* ########## */\
    0xFF, 0x03, /* ########## */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
    0x00, 0x00, /*            */\
\
    /* @4160 '{' (6 pixels wide) */\
    0x00, /*        */\
    0x38, /*    ### */\
    0x3C, /*   #### */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x06, /*  ##    */\
    0x03, /* ##     */\
    0x07, /* ###    */\
    0x0E, /*  ###   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x38, /*    ### */\
    0x38, /*    ### */\
    0x00, /*        */\
\
    /* @4186 '|' (2 pixels wide) */\
    0x00, /*    */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
    0x03, /* ## */\
\
    /* @4212 '}' (6 pixels wide) */\
    0x00, /*        */\
    0x07, /* ###    */\
    0x07, /* ###    */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x18, /*    ##  */\
    0x30, /*     ## */\
    0x38, /*    ### */\
    0x1C, /*   ###  */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x0C, /*   ##   */\
    0x07, /* ###    */\
    0x07, /* ###    */\
    0x00, /*        */\
\
    /* @4238 '~' (11 pixels wide) */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x1E, 0x04, /*  ####     # */\
    0x7F, 0x06, /* #######  ## */\
    0xF3, 0x07, /* ##  ####### */\
    0xC1, 0x03, /* #     ####  */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
    0x00, 0x00, /*             */\
}


/** @brief Initialization macro for the arial_20pt font information. */
#define FONTS_ARIAL_20PT_INFO \
{\
    -1, /* Character width (-1 means dynamic width). */\
    26, /* Character height (-1 means dynamic height). */\
    -1, /* Bitmap offset (-1 means dynamic offset). */\
    3, /* Deescriptor entry size. */\
    ' ', /* Start character. */\
    '~', /* End character. */\
    fonts_arial_20pt_descriptors, /*  Character descriptor array. */\
    fonts_arial_20pt_bitmaps, /* Character bitmap array. */\
}

extern const font_info_t font_calibri_14pt_info;
extern const font_info_t font_calibri_18pt_info;
extern const font_info_t font_calibri_12pt_info;

#endif // FONTS_H__


